/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.awt.Point;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import sjima.cgf.layer.CGFDialogLayer;

public class CGFDialogMode extends CGFMode {
	public static final String MODE_NAME = "DialogMode";

	private String dialogName = null;
	private String menuName = null;
	private Map handlerMap = null;
	private Map actionMap = null;

	public CGFDialogMode(CGFMain main) {
		super(main);
		init();
	}

	public void init() {
		buildActionMap();
	}

	public void buildActionMap() {
		Map map = getActionMap();
		if (map == null) {
			map = new HashMap();
			setActionMap(map);
		}
		CGFDialogAction action = new DefaultDialogAction();
		map.put(action.getDialogName(), action);
		setHandlerMap(createHandlerMap());
	}

	public synchronized void dispDialog(CGFDialogAction action) {
		CGFDialogLayer dialogLayer = (CGFDialogLayer) getMain().getGameData()
				.getLayer(CGFDialogLayer.LAYER_NAME);
		setDialogName(action.getDialogName());
		List list = dialogLayer.displayDialog(action);
		getMain().getGameData().repaintRectangleList(list);
	}

	public synchronized void repaintDialog() {
		CGFDialogLayer dialogLayer = (CGFDialogLayer) getMain().getGameData()
				.getLayer(CGFDialogLayer.LAYER_NAME);
		List list = dialogLayer.redrawDialog();
		getMain().getGameData().repaintRectangleList(list);
	}

	public synchronized void closeDialog() {
		CGFDialogLayer dialogLayer = (CGFDialogLayer) getMain().getGameData()
				.getLayer(CGFDialogLayer.LAYER_NAME);
		List list = dialogLayer.clearDialog();
		getMain().getGameData().repaintRectangleList(list);
		getMain().getGameData().changeCurrentMode(CGFMode.MODE_NAME);
		dialogLayer.getCurrentDispDialog().setEnable(false);
	}

	public synchronized void dispMenu(CGFDialogAction action) {
		CGFDialogLayer dialogLayer = (CGFDialogLayer) getMain().getGameData()
				.getLayer(CGFDialogLayer.LAYER_NAME);
		setMenuName(action.getDialogName());
		List list = dialogLayer.displayMenu(action);
		getMain().getGameData().repaintRectangleList(list);
	}

	public synchronized void repaintMenu() {
		CGFDialogLayer dialogLayer = (CGFDialogLayer) getMain().getGameData()
				.getLayer(CGFDialogLayer.LAYER_NAME);
		List list = dialogLayer.redrawMenu();
		getMain().getGameData().repaintRectangleList(list);
	}

	public synchronized void closeMenu() {
		CGFDialogLayer dialogLayer = (CGFDialogLayer) getMain().getGameData()
				.getLayer(CGFDialogLayer.LAYER_NAME);
		if (dialogLayer.getCurrentDispMenu() == null) {
			return;
		}
		dialogLayer.getCurrentDispMenu().setEnable(false);
		List list = dialogLayer.clearMenu();
		getMain().getGameData().repaintRectangleList(list);
		getMain().getGameData().changeCurrentMode(CGFMode.MODE_NAME);
	}

	public void moveMouse(Point point) {
		CGFDialogLayer dialogLayer = (CGFDialogLayer) getMain().getGameData()
				.getLayer(CGFDialogLayer.LAYER_NAME);
		if (dialogLayer == null) {
			return;
		}
		CGFDialog dialog = dialogLayer.getCurrentDispDialog();
		if ((dialog == null) || (dialog.getPoint() == null)) {
			return;
		}
		Point newPoint = new Point(point.x - dialog.getPoint().x, point.y
				- dialog.getPoint().y);
		dialog.onMove(newPoint);
	}

	public void click(Point point) {
		if (! isMouseEnable() ) {
			return;
		}
		CGFDialogLayer dialogLayer = (CGFDialogLayer) getMain().getGameData()
				.getLayer(CGFDialogLayer.LAYER_NAME);
		CGFDialog dialog = dialogLayer.getCurrentDispDialog();
		CGFMenuSet menu = dialogLayer.getCurrentDispMenu();
		if (dialog != null && dialog.isEnable()) {
			Point newPoint = new Point(point.x - dialog.getPoint().x, point.y
					- dialog.getPoint().y);
			dialog.onClick(newPoint);
		} else if (menu != null && menu.isEnable()) {
			Point newPoint = new Point(point.x - menu.getPoint().x, point.y
					- menu.getPoint().y);
			menu.onClick(newPoint);
		}
	}

	public void rightClick(Point point) {
		if (! isMouseEnable() ) {
			return;
		}
		CGFDialogLayer dialogLayer = (CGFDialogLayer) getMain().getGameData()
				.getLayer(CGFDialogLayer.LAYER_NAME);
		CGFDialog dialog = dialogLayer.getCurrentDispDialog();
		CGFMenuSet menu = dialogLayer.getCurrentDispMenu();
		if (dialog != null && dialog.isEnable()) {
			Point newPoint = new Point(point.x - dialog.getPoint().x, point.y
					- dialog.getPoint().y);
			dialog.onRightClick(newPoint);
		} else if (menu != null && menu.isEnable()) {
			Point newPoint = new Point(point.x - menu.getPoint().x, point.y
					- menu.getPoint().y);
			menu.onRightClick(newPoint);
		}
	}

	public synchronized void draggMouse(CGFLocation loc) {
		return;
	}
	
	public String getModeName() {
		return MODE_NAME;
	}

	public String getDialogName() {
		return dialogName;
	}

	public void setDialogName(String dialogName) {
		this.dialogName = dialogName;
	}

	public Map getHandlerMap() {
		return handlerMap;
	}

	public void setHandlerMap(Map handlerMap) {
		this.handlerMap = handlerMap;
	}

	public Map createHandlerMap() {
		Map map = new HashMap();
		setHandlerMap(map);

		for (Iterator ite = getActionMap().values().iterator(); ite.hasNext();) {
			CGFDialogAction action = (CGFDialogAction) ite.next();
			map.putAll(action.createHandlerMap());
			map.put(action.getDialogName(), action.createDialogHandler());
		}

		return map;
	}

	public Map getActionMap() {
		return actionMap;
	}

	public void setActionMap(Map actionMap) {
		this.actionMap = actionMap;
	}

	public void addAction(CGFDialogAction action) {
		actionMap.put(action.getDialogName(), action);
	}

	public CGFDialogAction getAction(String dialogName) {
		return (CGFDialogAction) actionMap.get(dialogName);
	}
	
	public class DefaultDialogAction extends CGFDialogAction {

		public void commit() {
		}

		public Map createHandlerMap() {
			Map map = new HashMap();
			CGFUIHandler handler = new OkButtonHandler();
			map.put(handler.getName(), handler);
			return map;
		}

		public String getDialogName() {
			return "default";
		}

		public void setup() {
		}

		public class OkButtonHandler implements CGFUIHandler {
			public static final String HANDLER_NAME = "okButton";

			public String getName() {
				return HANDLER_NAME;
			}

			public void onClick() {
				closeDialog();
			}

			public void onMove() {
				// 何もしない
			}

			public void onRightClick() {
				// 何もしない
			}

		}

	}

	public String getMenuName() {
		return menuName;
	}

	public void setMenuName(String menuName) {
		this.menuName = menuName;
	}

}
