/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

public class CGFFactory {
	private CGFMain main;
	private CellBuilder cellBuilder;

	public CGFFactory(CGFMain main) {
		setMain(main);
		setCellBuilder(new CellBuilder());
	}
	
	public CGFMainFrame createMainFrame() {
		return new CGFMainFrame(getMain(), "Default Main Frame");
	}

	public CGFGameData createGameData() {
		return new CGFGameData(getMain());
	}

	public CGFTerrain createTerrain(String ttype, String imagePath, String name) {
		return new CGFTerrain(getMain(), ttype, imagePath, name);
	}

	public CGFBorderTerrain createBorderTerrain(String btype, String name,
			String vimagePath, String himagePath) {
		return new CGFBorderTerrain(getMain(), btype, name, vimagePath,
				himagePath);

	}
	
	public CGFCell createCell(CGFLocation loc, CGFTerrain terrain) {
		return new CGFCell(getMain(), loc, terrain);
	}
	
	public CGFCell createCell(Node node) {
		return new CGFCell(getMain(), node);
	}

	public CGFPlayer createPlayer(Node node) {
		return new CGFPlayer(getMain(), node);
	}
	
	public CGFMapDocument createMapDocument(Document doc) {
		return new CGFMapDocument(getMain(), doc);
	}
	
	public CGFMain getMain() {
		return main;
	}

	public void setMain(CGFMain main) {
		this.main = main;
	}
	
	public CellBuilder getCellBuilder() {
		return cellBuilder;
	}

	public void setCellBuilder(CellBuilder cellBuilder) {
		this.cellBuilder = cellBuilder;
	}

	public class CellBuilder implements CGFRowColsTableDataBuilder {
		protected CellBuilder() {
			super();
		}

		public Object createData(CGFLocation loc) {
			return createCell(loc, getMain().getGameData().getDefaultTerrain());
		}

		public Object createFromNode(Node node) {
			return createCell(node);
		}
	}
	
	public CGFDialogDocument createDialogDocumnet(Map handlerMap) throws Exception {
		Document doc = CGFXMLUtil.loadXML("resource/Dialog.xml");
		return new CGFDialogDocument(getMain(), doc, handlerMap);
	}

	public CGFMenuDocument createMenuDocumnet(Map handlerMap) throws Exception {
		Document doc = CGFXMLUtil.loadXML("resource/Menu.xml");
		return new CGFMenuDocument(getMain(), doc, handlerMap);
	}
}
