package sjima.cgf;

import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;

public abstract class CGFFullSizeLayer extends CGFLayer {
	private BufferedImage layerImage;

	public CGFFullSizeLayer(CGFMain main, CGFSize size) {
		super(main, size);
		setLayerImage(CGFUtil.createImage(size));
	}

	public void paint(BufferedImage boardImage) {
		if (isVisible()) {
			Graphics2D gc = boardImage.createGraphics();
			AlphaComposite ac = AlphaComposite.getInstance(
					AlphaComposite.SRC_OVER, getTransparency());
			gc.setComposite(ac);

			gc.drawImage(getLayerImage(), 0, 0, null);
		}
		return;
	}

	public void paintRectangle(BufferedImage boardImage, Rectangle rect) {
		if (isVisible()) {
			Graphics2D gc = boardImage.createGraphics();
			AlphaComposite ac = AlphaComposite.getInstance(
					AlphaComposite.SRC_OVER, getTransparency());
			gc.setComposite(ac);
			gc.drawImage(getLayerImage(), rect.x, rect.y, rect.x + rect.width,
					rect.y + rect.height, rect.x, rect.y, rect.x + rect.width,
					rect.y + rect.height, null);
		}
		return;
	}
	
	public void clearAll() {
		Graphics2D gc = getLayerImage().createGraphics();
		gc.setBackground(CLEAR_COLOR);
		gc.clearRect(0, 0, getLayerImage().getWidth(), getLayerImage()
				.getHeight());
	}
	
	public BufferedImage getLayerImage() {
		return layerImage;
	}

	public void setLayerImage(BufferedImage layerImage) {
		this.layerImage = layerImage;
	}

}
