/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.util.Iterator;

public abstract class CGFLayer {

	private CGFMain main;
	private boolean isVisible;
	private float transparency = 1.0f;
	public static final Color CLEAR_COLOR = CGFUtil.stringToColor("0-0-0-0");
	private CGFSize size;
	
	public CGFLayer(CGFMain main, CGFSize size) {
		this.main = main;

		setSize(size);

		setVisible(true);
	}

	public void update() {
		Iterator ite = new CGFLocationIterator(getMain().getGameData()
				.getMapSize());
		while (ite.hasNext()) {
			CGFLocation loc = (CGFLocation) ite.next();
			updateAtLocation(loc);
		}

		return;
	}

	public void updateAtLocation(CGFLocation loc) {
	}

	public abstract String getLayerName();

	abstract public void paint(BufferedImage boardImage);
	abstract public void paintRectangle(BufferedImage boardImage, Rectangle rect);
	abstract public void clearAll();

	public CGFMain getMain() {
		return main;
	}

	public void setMain(CGFMain main) {
		this.main = main;
	}

	public boolean isVisible() {
		return isVisible;
	}

	public void setVisible(boolean isVisible) {
		this.isVisible = isVisible;
	}

	public float getTransparency() {
		return transparency;
	}

	public void setTransparency(float transparency) {
		this.transparency = transparency;
	}

	public void setSize(CGFSize size) {
		this.size = size;
	}

	public CGFSize getSize() {
		return size;
	}
	
	public float getDefaultTransparency() {
		return 1.0f;
	}

}
