/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

public class CGFLocation implements Cloneable, Comparable, Serializable {
	private static final long serialVersionUID = 2499885542705557118L;
	public int row;
	public int col;
	
	public CGFLocation() {
	}
	
	public CGFLocation(int p_row, int p_col) {
		row = p_row;
		col = p_col;
	}

	public CGFLocation(String locString) {
		String[] rowCols = locString.substring(0).split("c");
		if (rowCols.length >= 2) {
			row = Integer.parseInt(rowCols[0].substring(1));
			col = Integer.parseInt(rowCols[1]);
		}
	}
	
	public CGFLocation(CGFLocation loc) {
		row = loc.row;
		col = loc.col;
	}

	public String toString() {
		return "r" + row + "c" + col;
	}

	public boolean isEnable(CGFSize size) {
		if (row < 0) {
			return false;
		}
		if (col < 0) {
			return false;
		}
		if (row >= size.height) {
			return false;
		}
		if (col >= size.width) {
			return false;
		}

		return true;
	}
	
	public Object clone() {
		return new CGFLocation(row, col);
	}
	
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (! (obj instanceof CGFLocation) ) {
			return false;
		}
		CGFLocation another = (CGFLocation)obj;
		if (another.row != this.row) {
			return false;
		}
		if (another.col != this.col) {
			return false;
		}
		return true;
	}
	
	public int compareTo(Object obj) {
		int result = 0;
		CGFLocation another = (CGFLocation)obj;
		result = this.row - another.row;
		if (result == 0) {
			result = this.col - another.col;
		}
		
		return result;
	}
	
	public int hashCode() {
		return toString().hashCode();
	}
	
	public boolean isAjust(CGFLocation another) {
		return (distance(another) <= 3);
	}
	
	public int distance(CGFLocation another) {
		int dr = Math.abs(this.row - another.row);
		int dc = Math.abs(this.col - another.col);
		int max = Math.max(dr, dc);
		int min = Math.min(dr, dc);
		
		return max * 2 + min;
	}

	public List getAjustList() {
		List result = new ArrayList();
		for (int r = row - 1; r <= row + 1; r++) {
			if (r < 0) {
				continue;
			}
			for (int c = col - 1; c <= col + 1; c++) {
				if (c < 0) {
					continue;
				}
				if (r == row && c == col) {
					continue;
				}
				CGFLocation loc = new CGFLocation(r, c);
				result.add(loc);
			}
		}
		return result;
	}

}
