/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.ScrollPaneConstants;

public class CGFMainFrame extends Frame {
	private static final long serialVersionUID = 6422199068719093500L;

	private Listener listener = new Listener();
	private CGFMain main = null;
	private ImagePanel panel;
	private JScrollPane scrollPane;
	private Image openingImage;

	public CGFMainFrame(CGFMain p_main, String title, String openingImage) {
		super(title);
		main = p_main;
		scrollPane = new JScrollPane();
		panel = new ImagePanel();
		panel.setFont(main.getFontManager().getNormalFont());
		scrollPane.getViewport().add(panel);
		add(scrollPane);
		this.scrollPane
				.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		this.scrollPane
				.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		setVisible(true);
		if (openingImage != null) {
			BufferedImage image = CGFUtil.getImage(openingImage);
			setImage(image);
			int width = image.getWidth(this) + getInsets().left
					+ scrollPane.getInsets().left + getInsets().right
					+ scrollPane.getInsets().right;
			int height = image.getHeight(this) + getInsets().top
					+ getInsets().bottom + scrollPane.getInsets().top
					+ scrollPane.getInsets().bottom;
			this.setSize(width, height);
			this.setMinimumSize(new Dimension(width, height));
			this.setMaximumSize(new Dimension(width, height));
			this.setMaximizedBounds(new Rectangle(0, 0, width, height));
		}
	}

	public CGFMainFrame(CGFMain main2, String string) {
		this(main2, string, null);
	}

	public void activateMouseListner() {
		addWindowListener(listener);
		CGFBoardMouseListener mouseListener = new CGFBoardMouseListener(
				getMain());
		panel.addMouseListener(mouseListener);
		panel.addMouseMotionListener(mouseListener);
	}

	public void close() {
		super.dispose();
		main.finish();
	}

	public void setImage(BufferedImage image) {
		panel.setImage(image);
	}

	public Image getImage() {
		return panel.getImage();
	}

	public void autoFitSize() {
		GraphicsEnvironment env = GraphicsEnvironment
				.getLocalGraphicsEnvironment();
		this.setExtendedState(Frame.MAXIMIZED_BOTH);
		Rectangle bounds = env.getMaximumWindowBounds();

		setLocation(bounds.x, bounds.y);

		this.scrollPane
				.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		this.scrollPane
				.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		CGFSize boardImageSize = main.getGameData().getBoardImageSize();
		int viewportHeight = boardImageSize.height;
		int viewportWidth = boardImageSize.width;
		Rectangle maximumViewPort = new Rectangle(bounds);
		maximumViewPort.height -= this.getInsets().top
				+ this.getInsets().bottom;
		maximumViewPort.width -= this.getInsets().right + this.getInsets().left;

		if ((maximumViewPort.height) < viewportHeight) {
			viewportHeight = maximumViewPort.height;
		}
		if ((maximumViewPort.width) < viewportWidth) {
			viewportWidth = maximumViewPort.width;
		}
		int width = viewportWidth + this.scrollPane.getInsets().left
				+ this.scrollPane.getInsets().right + this.getInsets().left
				+ this.getInsets().right;
		int height = viewportHeight + this.scrollPane.getInsets().top
				+ this.scrollPane.getInsets().bottom + this.getInsets().top
				+ this.getInsets().bottom;
		if (width > bounds.width) {
			width = bounds.width;
		}
		if (height > bounds.height) {
			height = bounds.height;
		}
		setImage(getMain().getGameData().getBoardImage());
		this.setSize(width, height);
		this.setMaximizedBounds(new Rectangle(0, 0, width, height));
		this.setMaximumSize(this.getSize());
		validate();

		Rectangle viewPoint = getViewPort();
		if ((viewportWidth != viewPoint.width)
				|| (viewportHeight != viewPoint.height)) {
			width += viewportWidth - viewPoint.width;
			height += viewportHeight - viewPoint.height;
			if (width > bounds.width) {
				width = bounds.width;
			}
			if (height > bounds.height) {
				height = bounds.height;
			}

			this.setSize(width, height);
			this.setMaximizedBounds(new Rectangle(0, 0, width, height));
			this.setMaximumSize(this.getSize());
			validate();

		}
	}

	public Rectangle getViewPort() {
		JViewport vp = scrollPane.getViewport();
		Point p = vp.getViewPosition();
		Dimension d = vp.getExtentSize();
		return new Rectangle(p, d);
	}

	private class Listener implements WindowListener {

		public void windowActivated(WindowEvent e) {
		}

		public void windowClosed(WindowEvent e) {
		}

		public void windowClosing(WindowEvent e) {
			close();
		}

		public void windowDeactivated(WindowEvent e) {
		}

		public void windowDeiconified(WindowEvent e) {
		}

		public void windowIconified(WindowEvent e) {
		}

		public void windowOpened(WindowEvent e) {
		}

	}

	public CGFMain getMain() {
		return main;
	}

	public void setMain(CGFMain main) {
		this.main = main;
	}

	public void updateImage(BufferedImage boardImage, Rectangle rect) {
		if (rect == null) {
			return;
		}
		Rectangle imageRact = new Rectangle(0, 0, boardImage.getWidth(),
				boardImage.getHeight());
		rect = imageRact.intersection(rect);
		Image image = panel.getImage();
		Graphics gc = image.getGraphics();
		if (getOpeningImage() == null) {
			panel.getGraphics().setPaintMode();
			gc.setPaintMode();
		} else {
			gc.drawImage(getOpeningImage(), rect.x, rect.y,
					rect.x + rect.width, rect.y + rect.height, rect.x, rect.y,
					rect.x + rect.width, rect.y + rect.height, null);
			panel.getGraphics().drawImage(getOpeningImage(), rect.x, rect.y,
					rect.x + rect.width, rect.y + rect.height, rect.x, rect.y,
					rect.x + rect.width, rect.y + rect.height, null);
		}
		gc.drawImage(boardImage, rect.x, rect.y, rect.x + rect.width, rect.y
				+ rect.height, rect.x, rect.y, rect.x + rect.width, rect.y
				+ rect.height, null);
		panel.getGraphics().drawImage(boardImage, rect.x, rect.y,
				rect.x + rect.width, rect.y + rect.height, rect.x, rect.y,
				rect.x + rect.width, rect.y + rect.height, null);
	}

	public void scrollToPoint(Point point) {
		Dimension size = this.scrollPane.getViewport().getSize();
		Rectangle rect = new Rectangle(point.x - size.width / 2, point.y
				- size.height / 2, size.width, size.height);
		panel.scrollRectToVisible(rect);
		panel.repaint();
	}

	public void setOpeningImage(Image openingImage) {
		this.openingImage = openingImage;
	}

	public Image getOpeningImage() {
		return openingImage;
	}

	public class ImagePanel extends JPanel {

		private static final long serialVersionUID = 8815461797167876745L;
		private BufferedImage image = null;

		public ImagePanel() {
			super();
		}

		public Image getImage() {
			return this.image;
		}

		void setImage(BufferedImage image) {
			this.image = image;
			setPreferredSize(new Dimension(image.getWidth(), image.getHeight()));
			if (getGraphics() != null) {
				getGraphics().drawImage(image, 0, 0, image.getWidth(null),
						image.getHeight(null), null);
			}
		}

		public synchronized void paintComponent(Graphics g) {
			if (image != null) {
				g.drawImage(image, 0, 0, null);
			}
			Dimension size = CGFMainFrame.this.getSize();
			if (CGFMainFrame.this.getExtendedState() == Frame.MAXIMIZED_BOTH) {
				return;
			}
			if (CGFMainFrame.this.getExtendedState() == Frame.ICONIFIED) {
				return;
			}
			Dimension maximumSize = CGFMainFrame.this.getMaximumSize();
			if (maximumSize == null) {
				return;
			}
			Dimension newSize = new Dimension(size);
			if (maximumSize.width < size.width) {
				newSize.width = maximumSize.width;
			}
			if (maximumSize.height < size.height) {
				newSize.height = maximumSize.height;
			}
			if (!newSize.equals(size)) {
				CGFMainFrame.this.setSize(newSize);
			}
		}

	}
}
