/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

public class CGFMapDocument {
	private CGFMain main;

	private Document mapDocument;

	public CGFMapDocument(CGFMain main, Document doc) {
		setMain(main);
		setMapDocument(doc);
	}

	public void clear() {
		setMapDocument(null);
	}

	public CGFTerrain createTerrainInstance(Node node) {
		CGFTerrain terrain = CGFTerrain.create(getMain(), node);
		return terrain;
	}

	public CGFBorderTerrain createBorderTerrainInstance(Node node) {
		CGFBorderTerrain borderTerrain = new CGFBorderTerrain(getMain(), node);
		return borderTerrain;
	}

	public CGFPlayer createPlayerInstance(Node node) {
		CGFPlayer player = getMain().getFactory().createPlayer(node);
		return player;
	}

	public CGFCell createCellInstance(CGFRowColsTableDataBuilder builder,
			Node node) {
		CGFCell cell = (CGFCell) builder.createFromNode(node);
		return cell;
	}

	public CGFSize getMapSize() {
		try {
			CGFSize size = new CGFSize(0, 0);
			String width = CGFXMLUtil.readByXPath(getMapDocument(),
					"/gamedata/map/@width");
			String height = CGFXMLUtil.readByXPath(getMapDocument(),
					"/gamedata/map/@height");
			size.width = Integer.parseInt(width);
			size.height = Integer.parseInt(height);

			return size;
		} catch (NumberFormatException e) {
			e.printStackTrace();
		}
		return null;
	}

	public Map getTerrainMap() {
		String xpath = "/gamedata/map/ttypelist/ttype";
		List list = CGFXMLUtil.readListByXPath(getMapDocument(), xpath);
		Map map = new HashMap();
		for (Iterator ite = list.iterator(); ite.hasNext();) {
			Node node = (Node) ite.next();
			CGFTerrain terrain = createTerrainInstance(node);
			map.put(terrain.getTerrain(), terrain);
		}
		return map;
	}

	public CGFTerrain getDefaultTerrain(Map terrainMap) {
		String xpath = "/gamedata/map/celllist/@defaultttype";
		String ttype = CGFXMLUtil.readByXPath(getMapDocument(), xpath);

		return (CGFTerrain) getTerrainMap().get(ttype);
	}

	public CGFRowColsTable getCellTable(CGFRowColsTableDataBuilder builder) {
		CGFSize size = getMapSize();
		CGFRowColsTable cellTable = new CGFRowColsTable(size, builder);
		String xpath = "/gamedata/map/celllist/cell";
		List list = CGFXMLUtil.readListByXPath(getMapDocument(), xpath);
		for (Iterator ite = list.iterator(); ite.hasNext();) {
			Node node = (Node) ite.next();
			CGFCell cell = (CGFCell) builder.createFromNode(node);
			cellTable.setData(cell.getLocation(), cell);
		}

		return cellTable;
	}

	public Map getBorderTerrainMap() {
		String xpath = "/gamedata/map/btypelist/btype";
		List list = CGFXMLUtil.readListByXPath(getMapDocument(), xpath);
		Map map = new HashMap();
		for (Iterator ite = list.iterator(); ite.hasNext();) {
			Node node = (Node) ite.next();
			CGFBorderTerrain borderTerrain = createBorderTerrainInstance(node);
			map.put(borderTerrain.getBtype(), borderTerrain);
		}
		return map;
	}

	public void setCellSide(CGFRowColsTable cellTable) {
		String xpath = "/gamedata/map/borderlist/border";
		List list = CGFXMLUtil.readListByXPath(getMapDocument(), xpath);
		for (Iterator ite = list.iterator(); ite.hasNext();) {
			Node node = (Node) ite.next();
			CGFCellSide cellSide = createCellSide(node);
			CGFCell cell = getMain().getGameData().getCell(
					cellSide.getLocation());
			if ("H".equals(cellSide.getDirection())) {
				cell.setSideH(cellSide);
			}
			if ("V".equals(cellSide.getDirection())) {
				cell.setSideV(cellSide);
			}
		}
	}

	protected CGFCellSide createCellSide(Node node) {
		return new CGFCellSide(getMain(), node);
	}

	public List getColorList() {
		List result = new ArrayList();

		String xpath = "/gamedata/colorlist/color";
		List list = CGFXMLUtil.readListByXPath(getMapDocument(), xpath);
		for (Iterator ite = list.iterator(); ite.hasNext();) {
			Node node = (Node) ite.next();
			String colorString = CGFXMLUtil.getAttributeByName(node, "value");
			Color color = CGFUtil.stringToColor(colorString);
			result.add(color);
		}

		return result;
	}

	public List getPlayerList() {
		List result = new ArrayList();

		String xpath = "/gamedata/playerlist/player";
		List list = CGFXMLUtil.readListByXPath(getMapDocument(), xpath);
		for (Iterator ite = list.iterator(); ite.hasNext();) {
			Node node = (Node) ite.next();
			CGFPlayer player = createPlayerInstance(node);
			result.add(player);
		}

		return result;
	}

	public int getPlayerCount() {
		String xpath = "/gamedata/playerlist/@count";
		String countStr = CGFXMLUtil.readByXPath(getMapDocument(), xpath);
		int result = 0;
		try {
			if (countStr != null) {
				result = Integer.parseInt(countStr);
			}
		} catch (NumberFormatException e) {
		}

		return result;
	}

	public Document getMapDocument() {
		return mapDocument;
	}

	public void setMapDocument(Document mapDocument) {
		this.mapDocument = mapDocument;
	}

	public CGFMain getMain() {
		return main;
	}

	public void setMain(CGFMain main) {
		this.main = main;
	}

}
