/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import sjima.cgf.layer.CGFBorderTerrainLayer;
import sjima.cgf.layer.CGFCursolLayer;
import sjima.cgf.layer.CGFDialogLayer;
import sjima.cgf.layer.CGFInfoBoardLayer;
import sjima.cgf.layer.CGFTerrainLayer;

public class CGFMode {
	public static final String MODE_NAME = "DefaultMode";
	private CGFMain main;
	private boolean isBoardDisplay = false;
	private final static Class[] ENABLE_LAYERS = { CGFTerrainLayer.class,
			CGFBorderTerrainLayer.class, CGFCursolLayer.class,
			CGFInfoBoardLayer.class, CGFDialogLayer.class };
	private Set enableLayerSet = new HashSet(Arrays.asList(ENABLE_LAYERS));
	private boolean isMouseEnable = true;

	public CGFMode(CGFMain main) {
		setMain(main);
	}

	public String getModeName() {
		return MODE_NAME;
	}

	public CGFMain getMain() {
		return main;
	}

	public void setMain(CGFMain main) {
		this.main = main;
	}

	public boolean updateLayers() {
		boolean result = false;
		for (Iterator ite = getMain().getGameData().getLayerList().values()
				.iterator(); ite.hasNext();) {
			CGFLayer layer = (CGFLayer) ite.next();
			boolean oldIsVisible = layer.isVisible();
			if (getEnableLayerSet().contains(layer.getClass())) {
				layer.setVisible(true);
			} else {
				layer.setVisible(false);
			}
			if (oldIsVisible != layer.isVisible()) {
				result = true;
			}
		}
		return result;
	}

	public void moveMouse(Point point) {
		if (! isMouseEnable() ) {
			return;
		}
		Rectangle rect = getMain().getGameData().getBoardRectangle();
		if (! rect.contains(point)) {
			return;
		}
		moveMouse(getMain().getGameData().getLocationAtPoint(point));
	}

	protected void moveMouse(CGFLocation newLoc) {
		CGFLocation oldLoc = getMain().getGameData().getCursolLocation();
		CGFCursolLayer layer = (CGFCursolLayer) getMain().getGameData()
				.getLayer(CGFCursolLayer.LAYER_NAME);
		if ((newLoc != null) && (newLoc.equals(oldLoc))) {
			// 何もしない
		} else {
			setBoardDisplay(true);
			getMain().getGameData().setCursolLocation(newLoc);
			updateLayers();
			List updateList = layer.moveToCursol(oldLoc, newLoc);
			if (getInfoLayer() != null) {
				getInfoLayer().setMessageList(createMessageList(newLoc));
				updateList.addAll(getInfoLayer().drawBoard());
			}
			getMain().getGameData().repaintRectangleList(updateList);
		}
	}

	protected CGFInfoBoardLayer getInfoLayer() {
		CGFInfoBoardLayer infoLayer = (CGFInfoBoardLayer) getMain()
				.getGameData().getLayer(CGFInfoBoardLayer.LAYER_NAME);
		return infoLayer;
	}

	public List createMessageList(CGFLocation loc) {
		String[] msgArray = { loc.toString() };
		return Arrays.asList(msgArray);
	}

	public synchronized void click(Point point) {
		if (! isMouseEnable() ) {
			return;
		}
		Rectangle rect = getMain().getGameData().getBoardRectangle();
		if (! rect.contains(point)) {
			setMouseEnable(true);
			return;
		}
		click(getMain().getGameData().getLocationAtPoint(point));
	}

	public synchronized void rightClick(Point point) {
		if (! isMouseEnable() ) {
			return;
		}
		Rectangle rect = getMain().getGameData().getBoardRectangle();
		if (rect.contains(point)) {
			rightClick(getMain().getGameData().getLocationAtPoint(point));
		}
	}

	protected void click(CGFLocation loc) {
	}

	protected void rightClick(CGFLocation loc) {
		getMain().getGameData().changeCurrentMode(CGFDialogMode.MODE_NAME);
		CGFDialogAction action = ((CGFDialogMode) getMain().getGameData()
				.getCurrentMode()).getAction("default");
		((CGFDialogMode) getMain().getGameData().getCurrentMode())
				.dispDialog(action);
	}

	
	public boolean isBoardDisplay() {
		return isBoardDisplay;
	}

	public void setBoardDisplay(boolean isBoardDisplay) {
		this.isBoardDisplay = isBoardDisplay;
	}

	public Set getEnableLayerSet() {
		return enableLayerSet;
	}

	public void setEnableLayerSet(Set enableLayerSet) {
		this.enableLayerSet = enableLayerSet;
	}

	public void cancel() {
	}
	
	public void commit() {
	}

	public void draggMouse(Point point) {
		if (! isMouseEnable() ) {
			return;
		}
		draggMouse(getMain().getGameData().getLocationAtPoint(point));
	}

	public void draggMouse(CGFLocation loc) {
		if (! isMouseEnable() ) {
			return;
		}
		moveMouse(loc);
	}

	public void releaseMouse(Point point) {
		if (! isMouseEnable() ) {
			return;
		}
		releaseMouse(getMain().getGameData().getLocationAtPoint(point));
	}

	public void releaseMouse(CGFLocation locationAtPoint) {
	}

	public void setMouseEnable(boolean isMouseEnable) {
		this.isMouseEnable = isMouseEnable;
	}

	public boolean isMouseEnable() {
		return isMouseEnable;
	}
	
	

}
