/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.util.List;

public abstract class CGFPhaseManager {
	private List phaseList;
	private int phaseNo;

	public abstract void buildPhaseList();

	public CGFGamePhase nextPhase() {
		int nextPhaseNo = getPhaseNo() + 1;
		if (nextPhaseNo < getPhaseList().size()) {
			setPhaseNo(nextPhaseNo);
			return getCurrentPhase();
		}
		setPhaseNo(0);
		return null;
	}
	
	public CGFGamePhase prevPhase() {
		int prevPhaseNo = getPhaseNo() - 1;
		if (prevPhaseNo >= 0) {
			setPhaseNo(prevPhaseNo);
			return getCurrentPhase();
		}
		setPhaseNo(0);
		return null;
	}

	public CGFGamePhase getCurrentPhase() {
		return (CGFGamePhase) getPhaseList().get(getPhaseNo());
	}

	public void finishPhase() {
		getCurrentPhase().commit();
		CGFGamePhase nextPhase = nextPhase();
		if (nextPhase != null) {
			nextPhase.execute();
		}
	}
	
	public List getPhaseList() {
		return phaseList;
	}

	public void setPhaseList(List phaseList) {
		this.phaseList = phaseList;
	}

	public int getPhaseNo() {
		return phaseNo;
	}

	public void setPhaseNo(int phaseNo) {
		this.phaseNo = phaseNo;
	}

	public void returnPrevPhase() {
		getCurrentPhase().cancel();
		CGFGamePhase prevPhase = prevPhase();
		if (prevPhase != null) {
			prevPhase.execute();
		}
	}

}
