package sjima.cgf;

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class CGFSoundSetting {
	public static final String SOUND_SETTING_FILE_NAME = "sound_setting.xml";
	public static final String SOUND_SETTING_FILE = "resource/"
			+ SOUND_SETTING_FILE_NAME;
	public static final String SOUND_SETTING_DEFAULT_NAME = "sound_setting_default.xml";
	public static final String SOUND_SETTING_DEFAULT_FILE = "resource/"
			+ SOUND_SETTING_DEFAULT_NAME;
	public static final String SOUND_MP3_EXT = "mp3";
	public static final String SOUND_MIDI_EXT = "mid";

	private Document document = null;
	private LinkedHashMap soundList = new LinkedHashMap();
	private String soundFileDirectory;
	private boolean isMute = false;
	private boolean isMusicEnable = true;
	private boolean isEffectEnable = true;
	private boolean isDebug = true;
	private Set enableMidioutSet = new HashSet();

	public CGFSoundSetting() {
		File file = new File(SOUND_SETTING_FILE);
		if (!file.canRead()) {
			file = new File(SOUND_SETTING_DEFAULT_FILE);
		}
		try {
			init(file);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void init(File file) throws Exception {
		LinkedHashMap oldList = new LinkedHashMap(getSoundList());
		try {
			getSoundList().clear();
			load(file);
		} catch (Exception e) {
			e.printStackTrace();
			this.soundList = oldList;
			throw e;
		}
	}

	public void load(File file) throws Exception {
		Document doc = CGFXMLUtil.loadXML(file.getAbsolutePath());
		if (doc == null) {
			return;
		}

		Node rootNode = CGFXMLUtil.readNodeByXPath(doc, "/soundsetting");
		setSoundFileDirectory(CGFXMLUtil.getAttributeByName(rootNode,
				"soundpath"));
		String str = CGFXMLUtil.getAttributeByName(rootNode, "mute");
		setMute("1".equals(str));
		str = CGFXMLUtil.getAttributeByName(rootNode, "music");
		setMusicEnable("1".equals(str));
		str = CGFXMLUtil.getAttributeByName(rootNode, "effect");
		setEffectEnable("1".equals(str));
		str = CGFXMLUtil.getAttributeByName(rootNode, "debug");
		setDebug("1".equals(str));

		List nodeList = CGFXMLUtil.readListByXPath(doc, "/soundsetting/sound");
		for (Iterator ite = nodeList.iterator(); ite.hasNext();) {
			Node node = (Node) ite.next();
			SoundSetting sound = new SoundSetting(node);
			getSoundList().put(sound.getKey(), sound);
		}

		nodeList = CGFXMLUtil.readListByXPath(doc, "/soundsetting/midiout");
		for (Iterator ite = nodeList.iterator(); ite.hasNext();) {
			Node node = (Node) ite.next();
			getEnableMidioutSet().add(
					CGFXMLUtil.getAttributeByName(node, "devname"));
		}
		// ロードがうまく行ってからドキュメントを入れ替える
		setDocument(doc);
	}

	public void save(File file) throws Exception {
		Element rootElement = getDocument().getDocumentElement();
		List list = CGFXMLUtil.readListByXPath(getDocument(),
				"/soundsetting/sound");
		for (int i = 0; i < rootElement.getChildNodes().getLength(); i++) {
			Node node = rootElement.getChildNodes().item(i);
			if (node.getNodeType() == Node.TEXT_NODE) {
				list.add(node);
			}
		}
		for (Iterator ite = list.iterator(); ite.hasNext();) {
			Node node = (Node) ite.next();
			rootElement.removeChild(node);
		}
		for (Iterator ite = getSoundList().values().iterator(); ite.hasNext();) {
			SoundSetting sound = (SoundSetting) ite.next();
			Node node = sound.createNode(getDocument());
			rootElement.appendChild(node);
		}
		rootElement.setAttribute("soundpath", getSoundFileDirectory());
		if (isMute()) {
			rootElement.setAttribute("mute", "1");
		} else {
			rootElement.setAttribute("mute", "0");
		}
		if (isMusicEnable()) {
			rootElement.setAttribute("music", "1");
		} else {
			rootElement.setAttribute("music", "0");
		}
		if (isEffectEnable()) {
			rootElement.setAttribute("effect", "1");
		} else {
			rootElement.setAttribute("effect", "0");
		}
		if (isDebug()) {
			rootElement.setAttribute("debug", "1");
		} else {
			rootElement.setAttribute("debug", "0");
		}
		CGFXMLUtil.saveXML(getDocument(), file.getPath());
	}

	public String getOpeningSound() {
		String result = getOpeningSoundFile();
		if (result == null) {
			return result;
		}
		return getSoundFileDirectory() + "/" + result;
	}

	public String getOpeningSoundFile() {
		String result = null;

		for (Iterator ite = getSoundList().values().iterator(); ite.hasNext();) {
			SoundSetting sound = (SoundSetting) ite.next();
			if ("opening".equals(sound.getType())) {
				result = sound.getSoundFile();
				break;
			}
		}

		return result;
	}

	public void setOpeningSoundFile(String fileName) {
		for (Iterator ite = getSoundList().values().iterator(); ite.hasNext();) {
			SoundSetting sound = (SoundSetting) ite.next();
			if ("opening".equals(sound.getType())) {
				sound.setSoundFile(fileName);
				return;
			}
		}
		SoundSetting sound = new SoundSetting("opening", fileName, null);
		getSoundList().put(sound.getKey(), sound);
	}

	public String getEndingSound() {
		String result = getEndingSoundFile();
		if (result == null) {
			return result;
		}
		return getSoundFileDirectory() + "/" + result;
	}

	public String getEndingSoundFile() {
		String result = null;

		for (Iterator ite = getSoundList().values().iterator(); ite.hasNext();) {
			SoundSetting sound = (SoundSetting) ite.next();
			if ("ending".equals(sound.getType())) {
				result = sound.getSoundFile();
				break;
			}
		}

		return result;
	}

	public void setEndingSoundFile(String fileName) {
		for (Iterator ite = getSoundList().values().iterator(); ite.hasNext();) {
			SoundSetting sound = (SoundSetting) ite.next();
			if ("ending".equals(sound.getType())) {
				sound.setSoundFile(fileName);
				return;
			}
		}
		SoundSetting sound = new SoundSetting("ending", fileName, null);
		getSoundList().put(sound.getKey(), sound);
	}

	public String getEffectSound(String event) {
		String result = getEffectSoundFile(event);
		if (result == null) {
			return result;
		}
		return getSoundFileDirectory() + "/" + result;
	}

	public String getEffectSoundFile(String event) {
		for (Iterator ite = getSoundList().values().iterator(); ite.hasNext();) {
			SoundSetting sound = (SoundSetting) ite.next();
			if ("effect".equals(sound.getType())) {
				String stageNo = (String) sound.getAttrMap().get("event");
				if (stageNo.equals(event)) {
					return sound.getSoundFile();
				}
			}
		}
		return null;
	}

	public void setEffectSoundFile(String event, String fileName) {
		for (Iterator ite = getSoundList().values().iterator(); ite.hasNext();) {
			SoundSetting sound = (SoundSetting) ite.next();
			if ("effect".equals(sound.getType())) {
				if ((event == null)
						&& (sound.getAttrMap().get("event") == null)) {
					sound.setSoundFile(fileName);
					return;
				} else if ((event != null)
						&& (event.equals(sound.getAttrMap().get("event")))) {
					sound.setSoundFile(fileName);
					return;
				}
			}
		}
		Map attrMap = new HashMap();
		attrMap.put("event", event);
		SoundSetting sound = new SoundSetting("effect", fileName, null);
		getSoundList().put(sound.getKey(), sound);

	}

	public LinkedHashMap getSoundList() {
		return soundList;
	}

	public void setDocument(Document document) {
		this.document = document;
	}

	public Document getDocument() {
		return document;
	}

	public void setSoundFileDirectory(String soundFileDirectory) {
		this.soundFileDirectory = soundFileDirectory;
	}

	public String getSoundFileDirectory() {
		return soundFileDirectory;
	}

	public void setMusicEnable(boolean isMusicEnable) {
		this.isMusicEnable = isMusicEnable;
	}

	public boolean isMute() {
		return isMute;
	}

	public void setMute(boolean mute) {
		isMute = mute;
	}

	public boolean isMusicEnable() {
		return isMusicEnable;
	}

	public void setEffectEnable(boolean isEffectEnable) {
		this.isEffectEnable = isEffectEnable;
	}

	public boolean isEffectEnable() {
		return isEffectEnable;
	}

	public boolean isMusicEnableAndNoMute() {
		return (!isMute()) & isMusicEnable();
	}

	public boolean isEffectEnableAndNoMute() {
		return (!isMute()) & isEffectEnable();
	}

	public Set getEnableMidioutSet() {
		return enableMidioutSet;
	}

	public void setDebug(boolean isDebug) {
		this.isDebug = isDebug;
	}

	public boolean isDebug() {
		return isDebug;
	}

	public class SoundSetting {
		private String type;
		private String soundFile;
		private Map attrMap = new HashMap();

		public SoundSetting(Node node) {
			setType(CGFXMLUtil.getAttributeByName(node, "type"));
			setSoundFile(CGFXMLUtil.getAttributeByName(node, "soundfile"));
			Element elm = (Element) node;
			for (int i = 0; i < elm.getAttributes().getLength(); i++) {
				Node attr = elm.getAttributes().item(i);
				getAttrMap().put(attr.getNodeName(), attr.getNodeValue());
			}

		}

		public Node createNode(Document document) {
			Element elm = document.createElement("sound");
			for (Iterator ite = getAttrMap().keySet().iterator(); ite.hasNext();) {
				String key = (String) ite.next();
				String value = (String) getAttrMap().get(key);
				elm.setAttribute(key, value);
			}
			elm.setAttribute("type", getType());
			elm.setAttribute("soundfile", getSoundFile());
			return elm;
		}

		public SoundSetting(String type, String soundFile, Map attrMap) {
			setType(type);
			setSoundFile(soundFile);
			if (attrMap != null) {
				getAttrMap().putAll(attrMap);
			}
		}

		public void setType(String type) {
			this.type = type;
		}

		public String getType() {
			return type;
		}

		public void setSoundFile(String soundFile) {
			this.soundFile = soundFile;
		}

		public String getSoundFile() {
			return soundFile;
		}

		public Map getAttrMap() {
			return attrMap;
		}

		public String getKey() {
			SortedMap keyMap = new TreeMap(getAttrMap());
			keyMap.remove("soundfile");
			keyMap.remove("type");
			StringBuffer keyBuf = new StringBuffer(100);
			keyBuf.append(getType());
			for (Iterator ite = keyMap.keySet().iterator(); ite.hasNext();) {
				String key = (String) ite.next();
				String value = (String) keyMap.get(key);
				keyBuf.append("\t" + key + "=" + value);
			}
			return keyBuf.toString();
		}
	}

}
