/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.util.List;

import javax.swing.JOptionPane;

import org.w3c.dom.Element;

public class CGFTurnManager {
	private CGFMain main;
	private CGFPhaseManager phaseManager;
	private int turn;
	private int playerNo;

	public CGFTurnManager(CGFMain main, CGFPhaseManager phaseManager) {
		setMain(main);
		setPhaseManager(phaseManager);
		init();
	}

	public void init() {
		setup(1, 0, 0);
	}

	public void setup(int turn, int playerNo, int phaseNo) {
		setTurn(turn);
		setPlayerNo(playerNo);
		setPhaseNo(phaseNo);
	}

	public void setTurnPlayerPhase(CGFMapDocument doc) {
		Element node = (Element) CGFXMLUtil.readNodeByXPath(doc
				.getMapDocument(), "/gamedata");
		node.setAttribute("turn", String.valueOf(getTurn()));
		node.setAttribute("turnplayer", String.valueOf(getPlayerNo()));
		node.setAttribute("phase", String.valueOf(getPhaseNo()));
	}

	public void readTurnPlayerPhase(CGFMapDocument doc) {
		Element node = (Element) CGFXMLUtil.readNodeByXPath(doc
				.getMapDocument(), "/gamedata");
		setTurn(CGFXMLUtil.getAttributeIntegerByName(node, "turn"));
		setPlayerNo(CGFXMLUtil.getAttributeIntegerByName(node, "turnplayer"));
		setPhaseNo(CGFXMLUtil.getAttributeIntegerByName(node, "phase"));
	}

	public void startTurn() {
		getPhaseManager().getCurrentPhase().execute();
	}

	public void nextPhase() {
		CGFGamePhase nextPhase = getPhaseManager().nextPhase();
		if (nextPhase != null) {
			nextPhase.execute();
		} else {
			CGFPlayer nextPlayer = nextPlayerTurn();
			if (nextPlayer == null) {
				nextTurn();
			}
		}
	}

	public void nextTurn() {
		setTurn(getTurn() + 1);
		setPlayerNo(0);
		setPhaseNo(0);
		getPhaseManager().getCurrentPhase().execute();
	}

	public CGFPlayer nextPlayerTurn() {
		int nextPlayerNo = getPlayerNo() + 1;
		if (nextPlayerNo < getMain().getGameData().getNinzu()) {
			setPlayerNo(nextPlayerNo);
			getPhaseManager().setPhaseNo(0);
			getPhaseManager().getCurrentPhase().execute();
			return getTurnPlayer();
		}
		return null;
	}

	public List getPlayerList() {
		return getMain().getGameData().getPlayerList();
	}

	public CGFMain getMain() {
		return main;
	}

	public void setMain(CGFMain main) {
		this.main = main;
	}

	public CGFPhaseManager getPhaseManager() {
		return phaseManager;
	}

	public void setPhaseManager(CGFPhaseManager phaseManager) {
		this.phaseManager = phaseManager;
	}

	public int getTurn() {
		return turn;
	}

	public void setTurn(int turn) {
		this.turn = turn;
	}

	public int getPlayerNo() {
		return playerNo;
	}

	public void setPlayerNo(int playerNo) {
		this.playerNo = playerNo;
	}

	public int getPhaseNo() {
		return getPhaseManager().getPhaseNo();
	}

	public void setPhaseNo(int phaseNo) {
		getPhaseManager().setPhaseNo(phaseNo);
	}

	public CGFPlayer getTurnPlayer() {
		return (CGFPlayer) getPlayerList().get(getPlayerNo());
	}

	public void dispPlayerStartMessage() {
		String msg = getTurnPlayer().getName() + "さんの手番です";
		JOptionPane.showMessageDialog(null, msg);
	}

	public void dispVictory(String no) {
		String msg = getTurnPlayer().getName() + "さんの勝利です。" + CGFUtil.BR
				+ "おめでとうございます！";
		JOptionPane.showMessageDialog(null, msg);
	}

}
