/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Node;

public class CGFUIImage {
	private Map handlerMap;
	private String name;
	private BufferedImage image;
	private Map partsMap;
	private Dimension size;
	private Color bgColor = new Color(0, 0, 0, 0);
	private Color fgColor = new Color(255, 255, 255, 223);
	private String label;
	private String imagePath;
	private Point point = new Point(0, 0);
	private Point labelPoint = new Point(0, 0);
	private int fontSize = 10;
	private boolean enable = true;
	private boolean shadow = false;
	private boolean fontShadow = false;
	private final Color SHADOW_COLOR = new Color(0, 0, 0, 255);
	private final Color LIGHT_COLOR = new Color(255, 255, 255, 255);
	private final Color CLEAR_COLOR = new Color(255, 255, 255, 0);
	private CGFFontManager fontManager;

	public CGFUIImage(Node node, Map handlerMap, CGFFontManager fontManager) {
		setFontManager(fontManager);
		setHandlerMap(handlerMap);
		init(node);
		createImage();
		drawPartsList();
	}

	public CGFUIImage() {
	}

	public void updateImage() {
		if (this.isEnable()) {
			updatePartsList();
			createImage();
			drawPartsList();
		}
	}

	public void init(Node node) {
		setName(CGFXMLUtil.getAttributeByName(node, "name"));
		setLabel(CGFXMLUtil.getAttributeByName(node, "label"));
		setImagePath(CGFXMLUtil.getAttributeByName(node, "imagepath"));
		Dimension size = new Dimension();
		String strWidth = CGFXMLUtil.getAttributeByName(node, "width");
		if (strWidth != null) {
			size.width = Integer.parseInt(strWidth);
		}
		String strHeight = CGFXMLUtil.getAttributeByName(node, "height");
		if (strHeight != null) {
			size.height = Integer.parseInt(strHeight);
		}
		setSize(size);
		String top = CGFXMLUtil.getAttributeByName(node, "top");
		if (top != null) {
			getPoint().y = Integer.parseInt(top);
		}
		String left = CGFXMLUtil.getAttributeByName(node, "left");
		if (left != null) {
			getPoint().x = Integer.parseInt(left);
		}

		String labelTop = CGFXMLUtil.getAttributeByName(node, "labeltop");
		if (labelTop != null) {
			getLabelPoint().y = Integer.parseInt(labelTop);
		}
		String labelLeft = CGFXMLUtil.getAttributeByName(node, "labelleft");
		if (labelLeft != null) {
			getLabelPoint().x = Integer.parseInt(labelLeft);
		}

		String bgColorString = CGFXMLUtil.getAttributeByName(node, "bgcolor");
		if (bgColorString != null) {
			setBgColor(CGFUtil.stringToColor(bgColorString));
		}
		String fgColorString = CGFXMLUtil.getAttributeByName(node, "fgcolor");
		if (fgColorString != null) {
			setFgColor(CGFUtil.stringToColor(fgColorString));
		}
		Map parts = new LinkedHashMap();
		List partsNodeList = CGFXMLUtil.readListByXPath(node, "part");
		if (partsNodeList != null) {
			for (Iterator ite = partsNodeList.iterator(); ite.hasNext();) {
				Node partsNode = (Node) ite.next();
				CGFUIImage part = new CGFUIImage(partsNode, getHandlerMap(),
						getFontManager());
				parts.put(part.getName(), part);
			}
		}
		String shadowString = CGFXMLUtil.getAttributeByName(node, "shadow");
		setShadow(("1".equals(shadowString)));
		String fontShadowString = CGFXMLUtil.getAttributeByName(node,
				"fontshadow");
		setFontShadow(("1".equals(fontShadowString)));
		String fontSizeStr = CGFXMLUtil.getAttributeByName(node, "fontsize");
		if (fontSizeStr != null) {
			setFontSize(CGFXMLUtil.getAttributeIntegerByName(node, "fontsize"));
		}
		String enableString = CGFXMLUtil.getAttributeByName(node, "enable");
		setEnable(!("0".equals(enableString)));

		setPartsMap(parts);

	}

	public void createImage() {
		final String cr = System.getProperty("line.separator");
		BufferedImage img = new BufferedImage(getSize().width,
				getSize().height, BufferedImage.TYPE_INT_ARGB);
		setImage(img);
		if (!isEnable()) {
			return;
		}
		Graphics2D gc = img.createGraphics();
		AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
				1.0f);
		gc.setComposite(ac);
		if (isShadow()) {
			gc.setBackground(CLEAR_COLOR);
			gc.setColor(SHADOW_COLOR);
			gc.clearRect(2, 2, getSize().width - 2, getSize().height - 2);
			gc.fillRect(2, 2, getSize().width - 2, getSize().height - 2);
			gc.setColor(LIGHT_COLOR);
			gc.clearRect(0, 0, getSize().width - 2, getSize().height - 2);
			gc.fillRect(0, 0, getSize().width - 2, getSize().height - 2);
			gc.setColor(getBgColor());
			gc.clearRect(1, 1, getSize().width - 2, getSize().height - 2);
			gc.fillRect(1, 1, getSize().width - 2, getSize().height - 2);
		} else {
			gc.setColor(getBgColor());
			gc.fillRect(0, 0, getSize().width, getSize().height);
		}
		if (getImagePath() != null) {
			BufferedImage image = CGFUtil.getImage(getImagePath());
			gc
					.drawImage(image, 0, 0, image.getWidth(),
							image.getHeight(), null);
		}
		gc.setFont(getFontManager().getMonoFont());
		gc.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		if (getLabel() != null) {
			String[] labels = getLabel().split(cr);
			for (int i = 0; i < labels.length; i++) {
				if (isFontShadow()) {
					gc.setFont(CGFUtil.changeFontStyleSize(getFontManager()
							.getMonoFont(), getFontManager().getMonoFont()
							.getStyle(), getFontSize()));
					gc.setColor(SHADOW_COLOR);
					gc.drawString(labels[i], getLabelPoint().x + 4,
							getLabelPoint().y + (getFontSize() * (i+1)) + 3);
				}
				gc.setFont(CGFUtil.changeFontStyleSize(getFontManager()
						.getMonoFont(), getFontManager().getMonoFont().getStyle(),
						getFontSize()));
				gc.setColor(getFgColor());
				gc.drawString(labels[i], getLabelPoint().x + 3, getLabelPoint().y
						+ (getFontSize() * (i+1)) + 2);
			}
		}
	}

	public void drawPartsList() {
		if (getPartsMap() == null) {
			return;
		}
		Graphics2D gc = getImage().createGraphics();
		for (Iterator ite = getPartsMap().values().iterator(); ite.hasNext();) {
			CGFUIImage part = (CGFUIImage) ite.next();
			if (part.isEnable()) {
				gc.drawImage(part.getImage(), part.getPoint().x, part
						.getPoint().y, null);
			}
		}
	}

	public void updatePartsList() {
		if (getPartsMap() == null) {
			return;
		}
		for (Iterator ite = getPartsMap().values().iterator(); ite.hasNext();) {
			CGFUIImage part = (CGFUIImage) ite.next();
			if (part.isEnable()) {
				part.updateImage();
			}
		}
	}

	public boolean onRightClick(Point point) {
		boolean result = false;
		if (!isEnable()) {
			return result;
		}
		if (getPartsMap() != null) {
			for (Iterator ite = this.getPartsMap().values().iterator(); ite
					.hasNext();) {
				CGFUIImage part = (CGFUIImage) ite.next();
				Rectangle rect = new Rectangle(part.getPoint(), part.getSize());
				if (rect.contains(point)) {
					Point partPoint = new Point(point.x - rect.x, point.y
							- rect.y);
					result = part.onRightClick(partPoint);
					if (result) {
						return result;
					}
				}
			}
		}
		CGFUIHandler handler = (CGFUIHandler) getHandlerMap().get(getName());
		if (handler != null) {
			handler.onRightClick();
			result = true;
		}
		return result;
	}

	public boolean onClick(Point point) {
		boolean result = false;
		if (!isEnable()) {
			return result;
		}
		if (getPartsMap() != null) {
			for (Iterator ite = this.getPartsMap().values().iterator(); ite
					.hasNext();) {
				CGFUIImage part = (CGFUIImage) ite.next();
				Rectangle rect = new Rectangle(part.getPoint(), part.getSize());
				if (rect.contains(point)) {
					Point partPoint = new Point(point.x - rect.x, point.y
							- rect.y);
					result = part.onClick(partPoint);
					if (result) {
						return result;
					}
				}
			}
		}
		CGFUIHandler handler = (CGFUIHandler) getHandlerMap().get(getName());
		if (handler != null) {
			handler.onClick();
			result = true;
		}
		return result;
	}

	public boolean onMove(Point point) {
		boolean result = false;
		for (Iterator ite = this.getPartsMap().values().iterator(); ite
				.hasNext();) {
			CGFUIImage part = (CGFUIImage) ite.next();
			Rectangle rect = new Rectangle(part.getPoint(), part.getSize());
			if (rect.contains(point)) {
				Point partPoint = new Point(point.x - rect.x, point.y - rect.y);
				result = part.onMove(partPoint);
				if (result) {
					return result;
				}
			}
		}
		CGFUIHandler handler = (CGFUIHandler) getHandlerMap().get(getName());
		if (handler != null) {
			handler.onMove();
			result = true;
		}
		return result;
	}

	public BufferedImage getImage() {
		return this.image;
	}

	public void setImage(BufferedImage image) {
		this.image = image;
	}

	public Map getPartsMap() {
		return partsMap;
	}

	public void setPartsMap(Map partsMap) {
		this.partsMap = partsMap;
	}

	public Dimension getSize() {
		return size;
	}

	public void setSize(Dimension size) {
		this.size = size;
	}

	public Color getBgColor() {
		return bgColor;
	}

	public void setBgColor(Color bgColor) {
		this.bgColor = bgColor;
	}

	public Color getFgColor() {
		return fgColor;
	}

	public void setFgColor(Color fgColor) {
		this.fgColor = fgColor;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public String getImagePath() {
		return imagePath;
	}

	public void setImagePath(String imagePath) {
		this.imagePath = imagePath;
	}

	public Point getPoint() {
		return point;
	}

	public void setPoint(Point point) {
		this.point = point;
	}

	public Point getLabelPoint() {
		return labelPoint;
	}

	public void setLabelPoint(Point labelPoint) {
		this.labelPoint = labelPoint;
	}

	public Map getHandlerMap() {
		return handlerMap;
	}

	public void setHandlerMap(Map handlerMap) {
		this.handlerMap = handlerMap;
	}

	public boolean isEnable() {
		return enable;
	}

	public void setEnable(boolean enable) {
		this.enable = enable;
	}

	public boolean isShadow() {
		return shadow;
	}

	public void setShadow(boolean shadow) {
		this.shadow = shadow;
	}

	public boolean isFontShadow() {
		return fontShadow;
	}

	public void setFontShadow(boolean fontShadow) {
		this.fontShadow = fontShadow;
	}

	public int getFontSize() {
		return fontSize;
	}

	public void setFontSize(int fontSize) {
		this.fontSize = fontSize;
	}

	public void setFontManager(CGFFontManager fontManager) {
		this.fontManager = fontManager;
	}

	public CGFFontManager getFontManager() {
		return fontManager;
	}

}
