/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.UnsupportedEncodingException;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.imageio.ImageIO;

public class CGFUtil {
	public static final String BR = System.getProperty("line.separator");
	public static final int OS_TYPE_UNKNOWN = 0;
	public static final int OS_TYPE_WINDOWS = 1;
	public static final int OS_TYPE_MACOSX = 2;

	static public BufferedImage createImage(CGFSize size) {
		BufferedImage boardImage = new BufferedImage(size.width, size.height,
				BufferedImage.TYPE_INT_ARGB);
		return boardImage;
	}

	static public BufferedImage getImage(String path) {
		try {
			return ImageIO.read(new File(path));
		} catch (IOException e) {
			return null;
		}
	}

	static public boolean isEmpty(String str) {
		if (str == null) {
			return true;
		}
		if ("".equals(str)) {
			return true;
		}

		return false;
	}

	static public List getLinePointList(CGFMain main, CGFLocation loc1,
			CGFLocation loc2, int width, int length1, int length2) {
		List result = new ArrayList();
		Point point1 = main.getGameData().getLocationPoint(loc1);
		Point point2 = main.getGameData().getLocationPoint(loc2);
		CGFSize sqSize = main.getGameData().getSquareSize();
		point1.x += sqSize.width / 2;
		point1.y += sqSize.height / 2;
		point2.x += sqSize.width / 2;
		point2.y += sqSize.height / 2;
		double dx2 = (point2.x - point1.x);
		double dy2 = (point2.y - point1.y);
		double dx = dx2 / Math.sqrt(dx2 * dx2 + dy2 * dy2);
		double dy = dy2 / Math.sqrt(dx2 * dx2 + dy2 * dy2);

		Point p = new Point(0, 0);
		p.x = (int) (point1.x + (dy * width / 2) - dx * length1 + 0.501);
		p.y = (int) (point1.y - (dx * width / 2) - dy * length1 + 0.501);
		result.add(p);
		p = new Point(0, 0);
		p.x = (int) (point2.x + (dy * width / 2) + dx * length2 + 0.501);
		p.y = (int) (point2.y - (dx * width / 2) + dy * length2 + 0.501);
		result.add(p);
		p = new Point(0, 0);
		p.x = (int) (point2.x - (dy * width / 2) + dx * length2 + 0.501);
		p.y = (int) (point2.y + (dx * width / 2) + dy * length2 + 0.501);
		result.add(p);
		p = new Point(0, 0);
		p.x = (int) (point1.x - (dy * width / 2) - dx * length1 + 0.501);
		p.y = (int) (point1.y + (dx * width / 2) - dy * length1 + 0.501);
		result.add(p);

		return result;
	}

	public static Polygon getLinePolygon(CGFMain main, CGFLocation loc1,
			CGFLocation loc2, int width, int length1, int length2) {
		Polygon pol = new Polygon();
		List list = getLinePointList(main, loc1, loc2, width, length1, length2);

		for (Iterator ite = list.iterator(); ite.hasNext();) {
			Point p = (Point) ite.next();
			pol.addPoint(p.x, p.y);
		}

		return pol;
	}

	public static Font changeFontStyleSize(Font font, int style, int point) {
		String fontName = font.getFontName();
		return new Font(fontName, style, point);
	}

	public static Color stringToColor(String str) {
		try {
			String[] array = str.split("-");
			int r = Integer.parseInt(array[0]);
			int g = Integer.parseInt(array[1]);
			int b = Integer.parseInt(array[2]);
			int a = Integer.parseInt(array[3]);
			Color color = new Color(r, g, b, a);
			return color;
		} catch (Exception e) {
		}
		return null;
	}

	public static String colorToString(Color color) {
		if (color == null) {
			return null;
		}
		return color.getRed() + "-" + color.getGreen() + "-" + color.getBlue()
				+ "-" + color.getAlpha();
	}

	public static List shuffleList(List list, int size) {
		List tmpList = new ArrayList(list.subList(0, size));
		List restList = null;
		if (list.size() > size) {
			restList = new ArrayList(list.subList(size, list.size()));
		}
		list.clear();
		while (tmpList.size() > 0) {
			int rnd = rollDice(tmpList.size()) - 1;
			Object obj = tmpList.get(rnd);
			tmpList.remove(rnd);
			list.add(obj);
		}
		if (restList != null) {
			list.addAll(restList);
		}
		return list;
	}

	public static int rollDice(int men) {
		long RND_NUMBER = (new Date()).getTime();
		int rnd = (int) (Math.random() * RND_NUMBER / 1000);
		return (rnd % men) + 1;
	}

	public static Object pullRandomFromMap(Map map) {
		if (map.keySet().size() == 0) {
			return null;
		}
		int d = rollDice(map.keySet().size());
		Object key = (new ArrayList(map.keySet())).get(d - 1);
		Object value = map.get(key);
		map.remove(key);
		return value;
	}
	
	public static Object choiceRandomFromCollection(Collection col) {
		if (col == null) {
			return null;
		}
		if (col.size() == 0) {
			return null;
		}
		List list = null;
		if (col instanceof List) {
			list = (List)col;
		} else {
			list = new ArrayList(col);
		}
		int d = rollDice(list.size());
		Object result = list.get(d - 1);
		
		return result;
	}

	public static Object deepCopy(Object obj) {
		Object result = null;

		try {
			ByteArrayOutputStream bout = new ByteArrayOutputStream();
			(new ObjectOutputStream(bout)).writeObject(obj);
			result = (new ObjectInputStream(new ByteArrayInputStream(bout
					.toByteArray()))).readObject();

		} catch (IOException e) {
			// ありえない
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			// ありえない
			e.printStackTrace();
		}

		return result;
	}

	public static void fileCopy(File src, File dest) throws IOException {
		FileChannel srcChannel = new FileInputStream(src).getChannel();
		FileChannel destChannel = new FileOutputStream(dest).getChannel();
		srcChannel.transferTo(0, srcChannel.size(), destChannel);
		srcChannel.close();
		destChannel.close();
		dest.setLastModified(src.lastModified());
	}

	public static Rectangle getInnerRectangle(Rectangle outRectangle,
			Rectangle inRectangle) {
		Rectangle result = new Rectangle();
		result.x = inRectangle.x - outRectangle.x;
		result.y = inRectangle.y - outRectangle.y;
		result.width = inRectangle.width;
		result.height = inRectangle.height;
		return result;
	}

	public static List divideStringByWidth(String str, int width) {
		List list = new ArrayList();
		byte[] msgBytes = null;
		try {
			msgBytes = str.getBytes("MS932");
			if (msgBytes.length > width) {
				String subMsg1 = str;
				String subMsg2 = null;
				for (int i = width / 2; i < str.length(); i++) {
					String subMsg = str.substring(0, i);
					msgBytes = subMsg.getBytes("MS932");
					if (msgBytes.length >= width) {
						subMsg1 = subMsg;
						subMsg2 = str.substring(i);
						break;
					}
				}
				list.add(subMsg1);
				if (subMsg2 != null) {
					list.add(subMsg2);
				}
			} else {
				list.add(str);
			}
		} catch (UnsupportedEncodingException e) {
		}
		return list;
	}
	
	public static String insertCRIntoString(String str, int width) {
		final String cr = System.getProperty("line.separator");
		List strList = CGFUtil.divideStringByWidth(str, width);
		StringBuffer strbuf = new StringBuffer(width * 10);
		if (strList.size() > 0) {
			strbuf.append((String)strList.get(0));
		}
		if (strList.size() > 1) {
			strbuf.append(cr);
			strbuf.append((String)strList.get(1));
		}
		return strbuf.toString();
	}
	
	public static int osType() {
		String osName = System.getProperty("os.name");
		if (osName.indexOf("Windows") >= 0) {
			return OS_TYPE_WINDOWS;
		} else if (osName.indexOf("Mac OS X") >= 0) {
			return OS_TYPE_MACOSX;
		}
		return OS_TYPE_UNKNOWN;
	}
	
	public static Polygon movePolygon(Polygon polygon, int x, int y) {
		Polygon result = new Polygon();
		for (int i = 0; i < polygon.npoints; i++) {
			result.addPoint(polygon.xpoints[i] + x, polygon.ypoints[i] + y);
		}
		return result;
	}
	
	public static String substring(String str, int startix, int endix) {
		int length = str.length();
		if (startix >= length) {
			return "";
		}
		if (endix > length) {
			endix = length;
		}
		return str.substring(startix, endix);
	}
}
