/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class CGFXMLUtil {

	public static Document loadXML(String path) throws Exception {
		try {
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = dbf.newDocumentBuilder();
			Document doc = builder.parse(new File(path));
			return doc;
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			throw e;
		} catch (SAXException e) {
			e.printStackTrace();
			throw e;
		} catch (IOException e) {
			e.printStackTrace();
			throw e;
		}
	}

	public static Document loadXML(InputStream stream) throws Exception {
		try {
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = dbf.newDocumentBuilder();
			Document doc = builder.parse(stream);
			return doc;
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			throw e;
		} catch (SAXException e) {
			e.printStackTrace();
			throw e;
		} catch (IOException e) {
			e.printStackTrace();
			throw e;
		}

	}

	public static void saveXML(Document doc, String path) throws Exception {
		try {
			File outfile = new File(path);
			OutputStream out = new FileOutputStream(outfile);
			saveXML(doc, out);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			throw e;
		} catch (IOException e) {
			e.printStackTrace();
			throw e;
		}
	}

	public static void saveXML(Document doc, OutputStream out) throws Exception {
		TransformerFactory tfactory = TransformerFactory.newInstance();
		tfactory.setAttribute("indent-number", 2);
		Transformer transformer;
		try {
			transformer = tfactory.newTransformer();
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			transformer.setOutputProperty(OutputKeys.METHOD, "xml");
			transformer.transform(new DOMSource(doc), new StreamResult(
					new OutputStreamWriter(out, "utf-8")));
		} catch (TransformerConfigurationException e) {
			e.printStackTrace();
			throw e;
		} catch (TransformerException e) {
			e.printStackTrace();
			throw e;
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
			throw e;
		} finally {
			try {
				out.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public static String readByXPath(Node doc, String xpath) {
		try {
			XPathFactory xf = XPathFactory.newInstance();
			XPath xp = xf.newXPath();
			XPathExpression xexp;
			xexp = xp.compile(xpath);
			String result = xexp.evaluate(doc);
			return result;
		} catch (XPathExpressionException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	public static List readListByXPath(Node doc, String xpath) {
		try {
			XPathFactory xf = XPathFactory.newInstance();
			XPath xp = xf.newXPath();
			XPathExpression xexp;
			xexp = xp.compile(xpath);
			Object obj = xexp.evaluate(doc, XPathConstants.NODESET);
			NodeList nodeList = (NodeList) obj;
			List list = new ArrayList();
			for (int i = 0; i < nodeList.getLength(); i++) {
				list.add(nodeList.item(i));
			}
			return list;
		} catch (XPathExpressionException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	public static Node readNodeByXPath(Node doc, String xpath) {
		try {
			XPathFactory xf = XPathFactory.newInstance();
			XPath xp = xf.newXPath();
			XPathExpression xexp;
			xexp = xp.compile(xpath);
			Node node = (Node) xexp.evaluate(doc, XPathConstants.NODE);
			return node;
		} catch (XPathExpressionException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	public static String getAttributeByName(Node node, String name) {
		Node attribute = (Node) node.getAttributes().getNamedItem(name);
		if (attribute == null) {
			return null;
		}
		return attribute.getNodeValue();
	}

	public static int getAttributeIntegerByName(Node node, String name) {
		int result = 0;

		String str = getAttributeByName(node, name);
		if (str != null) {
			try {
				result = Integer.parseInt(str);
			} catch (NumberFormatException e) {
			}
		}

		return result;
	}

	public static double getAtributeDoubleByName(Node node, String name) {
		double result = 0;

		String str = getAttributeByName(node, name);
		if (str != null) {
			try {
				result = Double.parseDouble(str);
			} catch (NumberFormatException e) {
			}
		}

		return result;
	}

	public static Node getChildNode(Node node, String string) {
		NodeList childs = node.getChildNodes();
		for (int i = 0; i < childs.getLength(); i++) {
			Node child = childs.item(i);
			if (string.equals(child.getNodeName())) {
				return child;
			}
		}
		return null;
	}

	public static void clearChild(Element node, String tagName) {
		NodeList nodeList = node.getChildNodes();
		if (nodeList == null) {
			return;
		}
		for (int i = 0; i < nodeList.getLength(); i++) {
			Node childNode = nodeList.item(i);
			if (!(childNode instanceof Element)) {
				continue;
			}
			Element bizNode = (Element) childNode;
			if (tagName.equals(bizNode.getTagName())) {
				node.removeChild(bizNode);
			}
		}
	}

	public static Document createNewDocument() {
		try {
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = dbf.newDocumentBuilder();
			return builder.newDocument();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			return null;
		}

	}

}
