/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf.layer;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

import sjima.cgf.CGFLayer;
import sjima.cgf.CGFLocation;
import sjima.cgf.CGFMain;
import sjima.cgf.CGFSize;
import sjima.cgf.CGFUtil;

public class CGFCursolLayer extends CGFLayer {
	public static final String LAYER_NAME = "cursolLayer";
	public static final Color CLEAR_COLOR = new Color(255, 255, 255, 0);

	private BufferedImage cursolImage = null;
	private BufferedImage clearImage = null;
	private BufferedImage cursolLayerImage = null;

	public CGFCursolLayer(CGFMain main, CGFSize size) {
		super(main, size);
		setTransparency(0.5f);
		setCursolImage(CGFUtil.getImage("resource/focus.png"));
		setCursolLayerImage(CGFUtil.createImage(new CGFSize(getCursolImage()
				.getWidth(), getCursolImage().getHeight())));
		CGFSize imageSize = new CGFSize(getCursolImage().getWidth(),
				getCursolImage().getHeight());
		BufferedImage ci = CGFUtil.createImage(imageSize);
		ci.getGraphics().setColor(CLEAR_COLOR);
		ci.getGraphics().fillRect(0, 0, imageSize.width, imageSize.height);
		setClearImage(ci);
	}

	public BufferedImage getClearImage() {
		return clearImage;
	}

	public void setClearImage(BufferedImage clearImage) {
		this.clearImage = clearImage;
	}

	public void update() {
		CGFLocation loc = getMain().getGameData().getCursolLocation();
		if (loc != null) {
			clearCursol(loc);
			drawCursol(loc);
		}
		return;
	}

	public Rectangle drawCursol(CGFLocation loc) {
		Rectangle rect = getCursolRectangle(loc);
		Graphics2D gc = getCursolLayerImage().createGraphics();
		gc.drawImage(getCursolImage(), 0, 0, null);

		return rect;
	}

	public Rectangle clearCursol(CGFLocation loc) {
		Rectangle rect = getCursolRectangle(loc);
		Graphics2D gc = getCursolLayerImage().createGraphics();
		gc.setBackground(CLEAR_COLOR);
		gc.clearRect(0, 0, rect.width, rect.height);

		return rect;
	}

	public Rectangle getCursolRectangle(CGFLocation loc) {
		Point point = getMain().getGameData().getLocationPoint(loc);

		int width = getCursolImage().getWidth(getMain().getMainFrame());
		int height = getCursolImage().getHeight(getMain().getMainFrame());
		point.x -= (width - getMain().getGameData().getSquareSize().width) / 2;
		point.y -= (height - getMain().getGameData().getSquareSize().height) / 2;

		Rectangle rect = new Rectangle(point.x, point.y, width, height);

		return rect;
	}

	public java.util.List moveToCursol(CGFLocation from, CGFLocation to) {
		List list = new ArrayList();
		if (from != null) {
			list.add(clearCursol(from));
		}
		if (to != null) {
			list.add(drawCursol(to));
		}
		return list;
	}

	public String getLayerName() {
		return LAYER_NAME;
	}

	public BufferedImage getCursolImage() {
		return cursolImage;
	}

	public void setCursolImage(BufferedImage cursolImage) {
		this.cursolImage = cursolImage;
	}

	public void clearAll() {
		CGFLocation loc = getMain().getGameData().getCursolLocation();
		if (loc != null) {
			clearCursol(loc);
		}
	}

	public void paint(BufferedImage boardImage) {
		CGFLocation loc = getMain().getGameData().getCursolLocation();
		if (loc != null) {
			Rectangle rect = getCursolRectangle(loc);
			paintRectangle(boardImage, rect);
		}
	}

	public void paintRectangle(BufferedImage boardImage, Rectangle rect) {
		if (isVisible()) {
			CGFLocation loc = getMain().getGameData().getCursolLocation();
			if (loc != null) {
				Rectangle curRect = getCursolRectangle(loc);
				if (! curRect.intersects(rect)) {
					return;
				}
				Rectangle drawRect = curRect.intersection(rect);
				Rectangle inRect = CGFUtil.getInnerRectangle(curRect, drawRect);
				Graphics2D gc = boardImage.createGraphics();
				AlphaComposite ac = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, getTransparency());
				gc.setComposite(ac);
				gc.drawImage(getCursolLayerImage(), drawRect.x, drawRect.y,
						drawRect.x + drawRect.width, drawRect.y + drawRect.height,
						inRect.x, inRect.y, inRect.width + inRect.x, inRect.height
								+ inRect.y, null);
			}
		}
	}

	public void setCursolLayerImage(BufferedImage cursolLayerImage) {
		this.cursolLayerImage = cursolLayerImage;
	}

	public BufferedImage getCursolLayerImage() {
		return cursolLayerImage;
	}
	
	public float getDefaultTransparency() {
		return 0.5f;
	}
}
