/*
 * Common Game Fundation
 *
 * Copyright 2008 M.SHIOJIMA
 */

package sjima.cgf.layer;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import sjima.cgf.CGFLayer;
import sjima.cgf.CGFMain;
import sjima.cgf.CGFSize;
import sjima.cgf.CGFUtil;

public class CGFInfoBoardLayer extends CGFLayer {

	public static final String LAYER_NAME = "InfoBoardLayer";
	public static final Dimension DEFAULT_SIZE = new Dimension(250, 40);
	public static final Color CLEAR_COLOR = new Color(255, 255, 255, 0);

	private int margin = 5;
	private int padding = 10;
	private List messageList = new ArrayList();
	private Rectangle boardRectangle;
	private Color boardColor = new Color(31, 31, 31, 127);
	private Color fontColor = new Color(255, 255, 255, 255);
	private Color shadowColor = new Color(0, 0, 0, 255);
	private Color borderColor = new Color(255, 255, 255, 255);
	private BufferedImage infoBoardImage = null;

	public CGFInfoBoardLayer(CGFMain main, CGFSize size) {
		super(main, size);
	}

	public List drawBoard() {
		List result = new ArrayList();
		Rectangle newBoardRectangle = calcBoardRectangle();
		result.addAll(clearBoard());
		setInfoBoardImage(CGFUtil.createImage(new CGFSize(
				newBoardRectangle.width, newBoardRectangle.height)));
		setBoardRectangle(newBoardRectangle);
		Graphics2D gc = getInfoBoardImage().createGraphics();
		gc.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		gc.setBackground(CLEAR_COLOR);
		gc.clearRect(0, 0, getInfoBoardImage().getWidth(), getInfoBoardImage()
				.getHeight());
		gc.setColor(boardColor);
		gc
				.fillRect(0, 0, getBoardRectangle().width,
						getBoardRectangle().height);
		gc.setColor(borderColor);
		gc.drawRect(0, 0, getBoardRectangle().width - 1,
				getBoardRectangle().height - 1);
		int line = 1;
		for (Iterator ite = getMessageList().iterator(); ite.hasNext(); line++) {
			String msg = (String) ite.next();
			Point p = new Point(getMargin(), getMargin() + line * 16 - 8);
			gc.setFont(CGFUtil.changeFontStyleSize(getMain().getFontManager()
					.getMonoFont(), getMain().getFontManager().getMonoFont()
					.getStyle(), 10));
			gc.setColor(shadowColor);
			gc.drawString(msg, p.x + 1, p.y + 1);
			gc.setColor(fontColor);
			gc.drawString(msg, p.x, p.y);
		}
		result.add(getBoardRectangle().clone());

		return result;
	}

	public void update() {
		// drawBoard();
	}

	public void changeMessage(List newMessageList) {
		clearBoard();
		setMessageList(newMessageList);
		update();
	}

	public List clearBoard() {
		List result = new ArrayList();
		if (getBoardRectangle() == null) {
			return result;
		}
		Rectangle rect = getBoardRectangle();
		Graphics2D gc = getInfoBoardImage().createGraphics();
		gc.setBackground(CLEAR_COLOR);
		gc.clearRect(0, 0, rect.width, rect.height);
		result.add(rect.clone());
		return result;
	}

	public Rectangle calcBoardRectangle() {
		Rectangle viewPoint = getMain().getMainFrame().getViewPort();
		Rectangle r = new Rectangle();
		r.width = getBoardSize().width;
		r.height = getBoardSize().height;
		int width = Math.min(viewPoint.width, getMain().getGameData()
				.getBoardImageSize().width);
		int height = Math.min(viewPoint.height, getMain().getGameData()
				.getBoardImageSize().height);
		r.x = viewPoint.x + width - getPadding() - getBoardSize().width;
		r.y = viewPoint.y + getPadding();
		if (getMain().getGameData().getCursolLocation() == null) {
			return r;
		}
		Rectangle cursolArea = new Rectangle(r.x - getPadding(), r.y
				- getPadding(), getBoardSize().width + getPadding() * 2,
				getBoardSize().height + getPadding() * 2);
		Point cursolPoint = getMain().getGameData().getLocationPoint(
				getMain().getGameData().getCursolLocation());
		CGFSize cursolSize = getMain().getGameData().getSquareSize();
		Rectangle cursolRect = new Rectangle(cursolPoint.x, cursolPoint.y,
				cursolSize.width, cursolSize.height);
		if (!cursolArea.intersection(cursolRect).isEmpty()) {
			r.x = viewPoint.x + width - getPadding() - getBoardSize().width;
			r.y = viewPoint.y + height - getPadding() - getBoardSize().height;
		}
		return r;
	}

	public Dimension getTextAreaSize() {
		Dimension result = new Dimension(getBoardSize().width - margin * 2,
				getBoardSize().height - margin * 2);
		return result;
	}

	public String getLayerName() {
		return LAYER_NAME;
	}

	public Dimension getBoardSize() {
		Dimension boardSize = new Dimension(DEFAULT_SIZE);
		int h = getMessageList().size() * 16 + 4;
		if (boardSize.height < h) {
			boardSize.height = h;
		}

		return boardSize;
	}

	public int getMargin() {
		return margin;
	}

	public void setMargin(int margin) {
		this.margin = margin;
	}

	public List getMessageList() {
		return messageList;
	}

	public void setMessageList(List messageList) {
		this.messageList = messageList;
	}

	public Rectangle getBoardRectangle() {
		return boardRectangle;
	}

	public void setBoardRectangle(Rectangle boardRectangle) {
		this.boardRectangle = boardRectangle;
	}

	public int getPadding() {
		return padding;
	}

	public void setPadding(int padding) {
		this.padding = padding;
	}

	public Color getBoardColor() {
		return boardColor;
	}

	public void setBoardColor(Color boardColor) {
		this.boardColor = boardColor;
	}

	public Color getFontColor() {
		return fontColor;
	}

	public void setFontColor(Color fontColor) {
		this.fontColor = fontColor;
	}

	public Color getShadowColor() {
		return shadowColor;
	}

	public void setShadowColor(Color shadowColor) {
		this.shadowColor = shadowColor;
	}

	public void setInfoBoardImage(BufferedImage infoBoardImage) {
		this.infoBoardImage = infoBoardImage;
	}

	public BufferedImage getInfoBoardImage() {
		return infoBoardImage;
	}

	public void clearAll() {
		if (getInfoBoardImage() == null) {
			return;
		}
		Graphics2D gc = getInfoBoardImage().createGraphics();
		gc.setBackground(CLEAR_COLOR);
		gc.clearRect(0, 0, getInfoBoardImage().getWidth(), getInfoBoardImage()
				.getHeight());
	}

	public void paint(BufferedImage boardImage) {
		paintRectangle(boardImage, getBoardRectangle());
	}

	public void paintRectangle(BufferedImage boardImage, Rectangle rect) {
		if (!isVisible()) {
			return;
		}
		if (getBoardRectangle() == null) {
			return;
		}

		Rectangle boardRect = getBoardRectangle();
		if (boardRect == null) {
			return;
		}
		if (!boardRect.intersects(rect)) {
			return;
		}
		Rectangle drawRect = boardRect.intersection(rect);
		Rectangle inRect = CGFUtil.getInnerRectangle(boardRect, drawRect);
		Graphics2D gc = boardImage.createGraphics();
		AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
				getTransparency());
		gc.setComposite(ac);
		gc.drawImage(getInfoBoardImage(), drawRect.x, drawRect.y, drawRect.x
				+ drawRect.width, drawRect.y + drawRect.height, inRect.x,
				inRect.y, inRect.width + inRect.x, inRect.height + inRect.y,
				null);
	}

}
