package sjima.wop;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import sjima.cgf.CGFLocation;
import sjima.cgf.CGFUtil;

public class WOPAIStandard implements WOPAI {
	private WOPMain main;
	private WOPPlayer player;
	private String enemy;

	public WOPAIStandard(WOPMain main, WOPPlayer player) {
		setMain(main);
		setPlayer(player);
	}

	public void setMain(WOPMain main) {
		this.main = main;
	}

	public WOPMain getMain() {
		return main;
	}

	public void setPlayer(WOPPlayer player) {
		this.player = player;
	}

	public WOPPlayer getPlayer() {
		return player;
	}

	public void decideTargetLoc() {
		setEnemy(decideEnemy());

		CGFLocation barLoc = getMain().getGameData().calBarycenter(
				getPlayer().getNo());
		CGFLocation target = null;
		List cellList = getMain().getGameData().getAllCellList();
		cellList = CGFUtil.shuffleList(cellList, cellList.size());
		for (Iterator ite = cellList.iterator(); ite.hasNext();) {
			WOPCell cell = (WOPCell) ite.next();
			// 生産力のない土地は無視
			if (cell.getTerrain().getFertility(getPlayer()) == 0) {
				continue;
			}
			// 入れない土地は無視
			if (cell.getTerrain().getCapacity(getPlayer()) == 0) {
				continue;
			}
			String ctrl = getMain().getGameData().getForceManager()
					.getControll(cell.getLocation());
			// 自分の支配している土地は無視
			if (getPlayer().getNo().equals(ctrl)) {
				continue;
			}
			if (target == null) {
				target = cell.getLocation();
				continue;
			}
			// 敵がいる場合は、敵の領土以外は無視
			if (getEnemy() != null) {
				if (!getEnemy().equals(ctrl)) {
					continue;
				}
			}
			// 重心に近い土地をターゲットに設定
			if (barLoc.distance(cell.getLocation()) < barLoc.distance(target)) {
				target = cell.getLocation();
			}
		}
		// ターゲットを設定
		getPlayer().setTargetLoc(target);
	}

	// 敵を決める
	public String decideEnemy() {
		String result = null;

		// 10ターン前までさかのぼって判定
		int startTurn = getMain().getGameData().getTurnManager().getTurn() - 10;
		Map damageMap = new HashMap();
		for (Iterator ite = getMain().getGameData().getCombatResultList()
				.iterator(); ite.hasNext();) {
			WOPCombatResult cr = (WOPCombatResult) ite.next();
			// １０ターンより前の戦闘結果は無視
			if (cr.getTurn() < startTurn) {
				continue;
			}
			// 防御側がプレイヤーと一致する場合、防御側で負けたときの土地の肥沃さを加算する。
			if (cr.getDefender().equals(getPlayer().getNo())) {
				if (cr.getResult() < 0) {
					continue;
				}
				Integer damage = (Integer) damageMap.get(cr.getAttacker());
				if (damage == null) {
					damage = new Integer(cr.getCell().getTerrain()
							.getFertility(getPlayer()));
				} else {
					damage = new Integer(cr.getCell().getTerrain()
							.getFertility(getPlayer())
							+ cr.getDefenderDamage());
				}
				damageMap.put(cr.getAttacker(), damage);
			}
		}
		// 自分が防御側として戦って、もっとも損害の大きいプレイヤーを敵とする。
		int maxDamage = 0;
		for (Iterator ite = damageMap.keySet().iterator(); ite.hasNext();) {
			String pno = (String) ite.next();
			Integer damage = (Integer) damageMap.get(pno);
			if (maxDamage < damage.intValue()) {
				result = pno;
				maxDamage = damage.intValue();
			}
		}

		return result;
	}

	public void setEnemy(String enemy) {
		this.enemy = enemy;
	}

	public String getEnemy() {
		return enemy;
	}

}
