package sjima.wop;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import sjima.cgf.CGFLocation;
import sjima.cgf.CGFXMLUtil;

public class WOPCombatResult {
	public static final String TYPE_BATTLE = "BT";
	public static final String TYPE_REVOLT = "RV";
	private WOPMain main;
	private int turn;
	private WOPCell cell;
	private String defender;
	private String attacker;
	private int defenderStrength;
	private int attackerStrength;
	private int defenderForce;
	private int attackerForce;
	private int defenderDamage;
	private int attackerDamage;
	private int result;
	private WOPCell retreat;
	private String type;

	public WOPCombatResult(WOPMain main, int turn, WOPCell cell, String type,
			String defender, String attacker, int defenderStrength,
			int attackerStrength, int defenderForce, int attackerForce,
			int defenderDamage, int attackerDamage, int result, WOPCell retreat) {
		init(main, turn, cell, type, defender, attacker, defenderStrength,
				attackerStrength, defenderForce, attackerForce, defenderDamage,
				attackerDamage, result, retreat);
	}

	public WOPCombatResult(WOPMain main, Node node) {
		setMain(main);
		setTurn(CGFXMLUtil.getAttributeIntegerByName(node, "turn"));
		String locString = CGFXMLUtil.getAttributeByName(node, "loc");
		if (locString != null) {
			CGFLocation loc = new CGFLocation(locString);
			setCell(getMain().getGameData().getCell(loc));
		}
		setType(CGFXMLUtil.getAttributeByName(node, "type"));
		setDefender(CGFXMLUtil.getAttributeByName(node, "defender"));
		setAttacker(CGFXMLUtil.getAttributeByName(node, "attacker"));
		setDefenderStrength(CGFXMLUtil.getAttributeIntegerByName(node,
				"defenderStrength"));
		setAttackerStrength(CGFXMLUtil.getAttributeIntegerByName(node,
				"attackerStrength"));
		setDefenderForce(CGFXMLUtil.getAttributeIntegerByName(node,
				"defenderForce"));
		setAttackerForce(CGFXMLUtil.getAttributeIntegerByName(node,
				"attackerForce"));
		setDefenderDamage(CGFXMLUtil.getAttributeIntegerByName(node,
				"defenderDamage"));
		setAttackerDamage(CGFXMLUtil.getAttributeIntegerByName(node,
				"attackerDamage"));
		setResult(CGFXMLUtil.getAttributeIntegerByName(node, "result"));
		locString = CGFXMLUtil.getAttributeByName(node, "retreat");
		if (locString != null) {
			CGFLocation loc = new CGFLocation(locString);
			setRetreat(getMain().getGameData().getCell(loc));
		}
	}

	public void init(WOPMain main, int turn, WOPCell cell, String type,
			String defender, String attacker, int defenderStrength,
			int attackerStrength, int defenderForce, int attackerForce,
			int defenderDamage, int attackerDamage, int result, WOPCell retreat) {
		setMain(main);
		setTurn(turn);
		setCell(cell);
		setType(type);
		setDefender(defender);
		setAttacker(attacker);
		setDefenderStrength(defenderStrength);
		setAttackerStrength(attackerStrength);
		setDefenderForce(defenderForce);
		setAttackerForce(attackerForce);
		setDefenderDamage(defenderDamage);
		setAttackerDamage(attackerDamage);
		setResult(result);
		setRetreat(retreat);
	}

	public Node createNode(Document doc) {
		Element elm = doc.createElement("combatResult");
		elm.setAttribute("turn", String.valueOf(getTurn()));
		elm.setAttribute("loc", getCell().getLocation().toString());
		elm.setAttribute("type", getType());
		elm.setAttribute("defender", getDefender());
		elm.setAttribute("attacker", getAttacker());
		elm.setAttribute("defenderStrength", String
				.valueOf(getDefenderStrength()));
		elm.setAttribute("attackerStrength", String
				.valueOf(getAttackerStrength()));
		elm.setAttribute("defenderForce", String.valueOf(getDefenderForce()));
		elm.setAttribute("attackerForce", String.valueOf(getAttackerForce()));
		elm.setAttribute("defenderDamage", String.valueOf(getDefenderDamage()));
		elm.setAttribute("result", String.valueOf(getResult()));
		if (getRetreat() != null) {
			elm.setAttribute("retreat", getRetreat().getLocation().toString());
		}

		return elm;
	}

	public WOPMain getMain() {
		return main;
	}

	public void setMain(WOPMain main) {
		this.main = main;
	}

	public int getTurn() {
		return turn;
	}

	public void setTurn(int turn) {
		this.turn = turn;
	}

	public WOPCell getCell() {
		return cell;
	}

	public void setCell(WOPCell cell) {
		this.cell = cell;
	}

	public String getDefender() {
		return defender;
	}

	public void setDefender(String defender) {
		this.defender = defender;
	}

	public String getAttacker() {
		return attacker;
	}

	public void setAttacker(String attacker) {
		this.attacker = attacker;
	}

	public int getDefenderStrength() {
		return defenderStrength;
	}

	public void setDefenderStrength(int defenderStrength) {
		this.defenderStrength = defenderStrength;
	}

	public int getAttackerStrength() {
		return attackerStrength;
	}

	public void setAttackerStrength(int attackerStrength) {
		this.attackerStrength = attackerStrength;
	}

	public int getDefenderForce() {
		return defenderForce;
	}

	public void setDefenderForce(int defenderForce) {
		this.defenderForce = defenderForce;
	}

	public int getAttackerForce() {
		return attackerForce;
	}

	public void setAttackerForce(int attackerForce) {
		this.attackerForce = attackerForce;
	}

	public int getDefenderDamage() {
		return defenderDamage;
	}

	public void setDefenderDamage(int defenderDamage) {
		this.defenderDamage = defenderDamage;
	}

	public int getAttackerDamage() {
		return attackerDamage;
	}

	public void setAttackerDamage(int attackerDamage) {
		this.attackerDamage = attackerDamage;
	}

	public int getResult() {
		return result;
	}

	public void setResult(int result) {
		this.result = result;
	}

	public WOPCell getRetreat() {
		return retreat;
	}

	public void setRetreat(WOPCell retreat) {
		this.retreat = retreat;
	}

	public String toString() {
		StringBuffer buf = new StringBuffer(100);
		String cr = System.getProperty("line.separator");
		WOPPlayer attackPlayer = getMain().getGameData().getPlayer(
				getAttacker());
		WOPPlayer defencePlayer = getMain().getGameData().getPlayer(
				getDefender());
		if (TYPE_REVOLT.equals(getType())) {
			buf.append("反乱");
			buf.append(" at " + getCell().getLocation() + "("
					+ getCell().getTerrain().getName() + ")");
			buf.append(cr);
			buf.append(defencePlayer.getName() + " -> " + attackPlayer.getName());
			buf.append(" 兵力(戦力)：" + getAttackerStrength() + "("
					+ getAttackerForce() + ")");
			buf.append(cr);
			return buf.toString();
		} else {
			buf.append("戦闘");
		}
		buf.append(" at " + getCell().getLocation() + "("
				+ getCell().getTerrain().getName() + ")");
		buf.append(cr);
		buf.append("攻撃側：" + attackPlayer.getName());
		buf.append(" 兵力(戦力)：" + getAttackerStrength() + "("
				+ getAttackerForce() + ")");
		buf.append(" 損害：" + getAttackerDamage());
		buf.append(cr);
		buf.append("防御側：" + defencePlayer.getName());
		buf.append(" 兵力(戦力)：" + getDefenderStrength() + "("
				+ getDefenderForce() + ")");
		buf.append(" 損害：" + getDefenderDamage());
		buf.append(cr);
		if (getResult() < 0) {
			buf.append(defencePlayer.getName() + "の勝利、");
			if (getRetreat() == null) {
				buf.append(attackPlayer.getName() + "全滅");
			} else {
				buf.append(attackPlayer.getName() + "撤退->"
						+ getRetreat().getLocation());
			}
		} else if (getResult() > 0) {
			buf.append(attackPlayer.getName() + "の勝利、");
			if (getRetreat() == null) {
				buf.append(defencePlayer.getName() + "全滅");
			} else {
				buf.append(defencePlayer.getName() + "撤退->"
						+ getRetreat().getLocation());
			}
		} else {
			buf.append("両軍全滅");
		}

		return buf.toString();
	}

	public void displayMessage() {
		if ((getDefenderForce() + getAttackerForce() >= 2000)
				&& (getDefenderForce() > getAttackerForce() / 2)
				&& (getAttackerForce() > getDefenderForce() / 2)) {
			System.out.println(toString());
		}

	}

	public void setType(String type) {
		this.type = type;
	}

	public String getType() {
		return type;
	}

}
