package sjima.wop;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import sjima.cgf.CGFLocation;
import sjima.cgf.CGFXMLUtil;

public class WOPForceManager {
	private WOPMain main;
	private Map forceMap = new HashMap();
	public final static String NONE_CONTROLL = null;
	public final static String IN_BATTLE = "B";

	public WOPForceManager(WOPMain main, Node node) {
		setMain(main);
		init(node);
	}

	public void init(Node node) {
		List list = CGFXMLUtil.readListByXPath(node,
				"/gamedata/map/forcelist/force");
		for (Iterator ite = list.iterator(); ite.hasNext();) {
			Node forceNode = (Node) ite.next();
			String locString = CGFXMLUtil.getAttributeByName(forceNode, "loc");
			String pno = CGFXMLUtil.getAttributeByName(forceNode, "player");
			String strength = CGFXMLUtil.getAttributeByName(forceNode,
					"strength");
			CGFLocation loc = new CGFLocation(locString);
			LinkedHashMap map = (LinkedHashMap) getForceMap().get(loc);
			if (map == null) {
				map = new LinkedHashMap();
				getForceMap().put(loc, map);
			}
			map.put(pno, new Integer(strength));
		}
	}

	public Node createNode(Document doc) {
		Element parent = (Element) doc.createElement("forcelist");
		for (Iterator ite = getForceMap().keySet().iterator(); ite.hasNext();) {
			CGFLocation loc = (CGFLocation) ite.next();
			LinkedHashMap map = (LinkedHashMap) getForceMap().get(loc);
			for (Iterator ite2 = map.keySet().iterator(); ite2.hasNext();) {
				String pno = (String) ite2.next();
				int strength = ((Integer) map.get(pno)).intValue();
				Element node = doc.createElement("force");
				node.setAttribute("loc", loc.toString());
				node.setAttribute("player", pno);
				node.setAttribute("strength", String.valueOf(strength));
				parent.appendChild(node);
			}
		}
		return parent;
	}

	public String getControll(CGFLocation loc) {
		Map map = (Map) getForceMap().get(loc);
		if (map == null) {
			return NONE_CONTROLL;
		}
		if (map.size() > 1) {
			return IN_BATTLE;
		}
		List list = new ArrayList(map.keySet());
		if (list.size() == 0) {
			return NONE_CONTROLL;
		}
		return (String) list.get(0);
	}

	public int getStrength(CGFLocation loc, String pno) {
		Map map = (Map) getForceMap().get(loc);
		if (map == null) {
			return 0;
		}
		Integer strength = (Integer) map.get(pno);
		if (strength == null) {
			return 0;
		}
		return strength.intValue();
	}

	public void setStrength(CGFLocation loc, String pno, int strength) {
		LinkedHashMap map = (LinkedHashMap) getForceMap().get(loc);
		if (map == null) {
			map = new LinkedHashMap();
			if (strength > 0) {
				getForceMap().put(loc, map);
			}
		}
		if (strength <= 0) {
			map.remove(pno);
		} else {
			map.put(pno, new Integer(strength));
		}
	}

	public void setMain(WOPMain main) {
		this.main = main;
	}

	public WOPMain getMain() {
		return main;
	}

	public Map getForceMap() {
		return forceMap;
	}

	public int calDefenderForce(WOPCell cell, int defenderStrength) {
		return defenderStrength * 100;
	}

	public int calAttackerForce(WOPCell cell, int attackerStrength,
			WOPPlayer attacker, WOPPlayer defender) {
		return attackerStrength
				* (100 + cell.getTerrain().getAttack(attacker) - cell
						.getTerrain().getDefence(defender));
	}

	public void calTotalStrength() {
		for (Iterator ite = getMain().getGameData().getPlayerList().iterator(); ite
				.hasNext();) {
			WOPPlayer player = (WOPPlayer) ite.next();
			player.setTotalStreangth(0);
		}

		for (Iterator ite = getForceMap().values().iterator(); ite.hasNext();) {
			LinkedHashMap map = (LinkedHashMap) ite.next();
			for (Iterator ite2 = map.keySet().iterator(); ite2.hasNext();) {
				String pno = (String) ite2.next();
				Integer strength = (Integer) map.get(pno);
				WOPPlayer player = getMain().getGameData().getPlayer(pno);
				player.setTotalStreangth(player.getTotalStreangth()
						+ strength.intValue());
			}
		}
	}
}
