/**
 * Wars of Powers
 * 
 * copyright 2009 Mitsunobu SHIOJIMA
 */

package sjima.wop;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.JOptionPane;

import sjima.cgf.CGFDialogAction;
import sjima.cgf.CGFGamePhase;
import sjima.cgf.CGFGameSetup;
import sjima.cgf.CGFMode;
import sjima.cgf.CGFPhaseManager;
import sjima.wop.action.WOPMapSelectDialogAction;
import sjima.wop.action.WOPSetup0DialogAction;
import sjima.wop.action.WOPSetup1DialogAction;
import sjima.wop.mode.WOPDialogMode;
import sjima.wop.mode.WOPDisplayMode;

public class WOPGameSetup extends CGFGameSetup {

	public WOPGameSetup(WOPMain main) {
		super(main);
	}

	public void buildPhaseList() {
		List list = getSetupPhaseList();
		list.clear();
		list.add(new GameSelectPhase(this));
		list.add(new MapSelectPhase(this));
		list.add(new GameStartPhase(this));
	}

	public void start() {
		setPhaseList(getSetupPhaseList());
		getCurrentPhase().execute();
	}
	
	public WOPMain getMain() {
		return (WOPMain)super.getMain();
	}

	public void setupPlayerList() {
		if (getMain().getGameData().getMapDocument() == null) {
			return;
		}
	}

	private class GameSelectPhase extends CGFGamePhase {
		public GameSelectPhase(CGFPhaseManager manager) {
			setPhaseManager(manager);
		}

		public String getPhaseName() {
			return "Game Select Phase";
		}

		public CGFMode getDefaultMode() {
			return (CGFMode) getMain().getGameData().getModeList().get(
					WOPDialogMode.MODE_NAME);
		}

		public void run() {
			getMain().getGameData().changeCurrentMode(WOPDialogMode.MODE_NAME);
			WOPDialogMode mode = (WOPDialogMode) getMain().getGameData()
					.getCurrentMode();
			CGFDialogAction action = mode
					.getAction(WOPSetup0DialogAction.DIALOG_NAME);
			((WOPDialogMode) getMain().getGameData().getCurrentMode())
					.dispDialog(action);
			return;
		}

	}

	private class MapSelectPhase extends CGFGamePhase {
		public MapSelectPhase(CGFPhaseManager manager) {
			setPhaseManager(manager);
		}

		public String getPhaseName() {
			return "Map Select Phase";
		}

		public CGFMode getDefaultMode() {
			return (CGFMode) getMain().getGameData().getModeList().get(
					WOPDialogMode.MODE_NAME);
		}

		public void run() {
			try {
				getMain().getGameData().changeCurrentMode(WOPDialogMode.MODE_NAME);
				List list = getMain().readMapList();
				if (list.size() == 0) {
					JOptionPane.showMessageDialog(null, "地図ファイルが見つかりません");
					System.exit(1);
				}
	
				if (list.size() >= 1) {
					Collections.sort(list);
					WOPMapSelectDialogAction action = (WOPMapSelectDialogAction) getDialogMode()
							.getAction(WOPMapSelectDialogAction.DIALOG_NAME);
					action.setMapInfoList(list);
					getDialogMode().dispDialog(action);
				} else {
					CGFMode mode = getMain().getGameData().getCurrentMode();
					mode.setMouseEnable(false);
					getMain().getGameData().setMapInfo(
							(WOPMapInfo) list.get(0));
					List msgList = new ArrayList();
					msgList.add("地図を読み込んでいます。");
					msgList.add("しばらくお待ちください。");
					WOPMain WOPMain = (WOPMain) getMain();
					WOPMain.getGameData().displayMessageList(msgList, null, 0,
							false);
					WOPMain.setupGameData(WOPMain.getGameData().getMapInfo()
							.getPath());
					getMain().getGameData().setCurrentMode(getDialogMode());
					CGFDialogAction action = getDialogMode().getAction(
							WOPSetup1DialogAction.DIALOG_NAME);
					getDialogMode().dispDialog(action);
					getDialogMode().setMouseEnable(true);
				}
	
				return;
			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, "エラーが発生しました（" + e + ")");
				System.exit(1);
			}
		}

		public void commit() {
		}

	}

	private WOPDialogMode getDialogMode() {
		return (WOPDialogMode) getMain().getGameData()
				.getModeList().get(WOPDialogMode.MODE_NAME);
	}

	private class GameStartPhase extends CGFGamePhase {
		public GameStartPhase(CGFPhaseManager manager) {
			setPhaseManager(manager);
		}

		public void run() {
			setupPlayerList();

			((WOPDialogMode) (getMain().getGameData().getModeList()
					.get(WOPDialogMode.MODE_NAME))).closeDialog();
			getMain().getGameData().changeCurrentMode(WOPDisplayMode.MODE_NAME);
			List msgList = new ArrayList();
			msgList.add("ゲームを開始します");
			getMain().getGameData().displayMessageList(msgList,
					new FinishHandler());

			return;
		}

		public String getPhaseName() {
			return "Game Start Phase";
		}

		public CGFMode getDefaultMode() {
			return (CGFMode) getMain().getGameData().getModeList().get(
					WOPDisplayMode.MODE_NAME);
		}

		public class FinishHandler implements Runnable {

			public void run() {
				finish();
				getMain().getGameData().getTurnManager().init();
				getMain().getGameData().getTurnManager().startTurn();
			}

		}

	}

}
