package sjima.wop;

import java.awt.image.BufferedImage;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import sjima.cgf.CGFFileFilter;
import sjima.cgf.CGFFontManager;
import sjima.cgf.CGFGameData;
import sjima.cgf.CGFMain;
import sjima.cgf.CGFMainFrame;
import sjima.cgf.CGFSetting;
import sjima.cgf.CGFSize;
import sjima.cgf.CGFTurnManager;
import sjima.wop.mode.WOPDialogMode;
import sjima.wop.mode.WOPDisplayMode;
import sjima.wop.phase.WOPPhaseManager;

public class WOPMain extends CGFMain {
	private WOPGameSetup setup;
	private JFileChooser imageFileChooser;
	private JFileChooser mapFileChooser;

	public static void main(String argv[]) {
		try {
			WOPMain main = new WOPMain();
			main.start();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void createSaveFileChooser() {
		JFileChooser chooser = new JFileChooser();

		CGFFileFilter filter = new CGFFileFilter();
		filter.add(WOPMapDocument.SAVE_FILE_EXT);
		filter.setDescription("Wars of Powers Save File(*.trs)");
		chooser.setFileFilter(filter);
		setSaveFileChooser(chooser);
	}

	public void createImageFileChooser() {
		JFileChooser chooser = new JFileChooser();

		CGFFileFilter filter = new CGFFileFilter();
		filter.add(WOPMapDocument.IMAGE_FILE_EXT);
		filter.setDescription("PNG Image File(*.png)");
		chooser.setFileFilter(filter);
		setImageFileChooser(chooser);
	}

	public void createMapFileChooser() {
		JFileChooser chooser = new JFileChooser();

		CGFFileFilter filter = new CGFFileFilter();
		filter.add(WOPMapDocument.MAP_FILE_EXT);
		filter.setDescription("Wars of Powers Map File(*.map)");
		chooser.setFileFilter(filter);
		setMapFileChooser(chooser);
	}

	public void start() {
		try {
			createImageFileChooser();
			createMapFileChooser();
			setSetting(new CGFSetting());
			setFontManager(new CGFFontManager(this));
			setFactory(new WOPFactory(this));
			loadSoundLib();
			setSoundSetting(new WOPSoundSetting());
			getSoundManager().playSound(getSoundSetting().getOpeningSound());
			createSoundFileChooser();
			setMainFrame(createMainFrame());
			setupGameData();
			startSetup();
			setDialogDocument(getFactory().createDialogDocumnet(
					getGameData().createHandlerMap()));

			getMainFrame().activateMouseListner();
			getSetup().start();
		} catch (Exception ex) {
			ex.printStackTrace();
			JOptionPane.showMessageDialog(null, "エラーが発生しました（" + ex + ")");
		}
	}

	public void setupGameData() {
		if (getGameData() == null) {
			setGameData(createGameData());
		} else {
			getGameData().clear();
		}
		getGameData().createSaveDir();
		getGameData().setMapSize(new CGFSize(40, 30));
		getGameData().buildModes();
		getGameData().buildLayers();
	}

	public void setupGameData(String path) throws Exception {
		if (getGameData() == null) {
			setGameData(createGameData());
		} else {
			getGameData().clear();
		}
		System.gc();
		getGameData().loadGameData(path);
		getGameData().buildModes();
		WOPDialogMode mode = (WOPDialogMode) getGameData().getModeList().get(
				WOPDialogMode.MODE_NAME);
		setDialogDocument(getFactory().createDialogDocumnet(
				mode.getHandlerMap()));
		WOPDisplayMode displayMode = (WOPDisplayMode) getGameData()
				.getModeList().get(WOPDisplayMode.MODE_NAME);
		displayMode.setMouseEnable(false);
		getGameData().setDisplayMode();
		getGameData().setBoardImage((BufferedImage) getMainFrame().getImage());
		getMainFrame().setOpeningImage(null);
		getMainFrame().autoFitSize();
		WOPPhaseManager phaseManager = new WOPPhaseManager(this);
		getGameData().setPhaseManager(phaseManager);
		getGameData().setTurnManager(
				new WOPTurnManager(this, phaseManager, getGameData()
						.getMapDocument()));
		phaseManager.setTurnManager((CGFTurnManager) getGameData()
				.getTurnManager());
		getGameData().getTurnManager().readTurnPlayerPhase(
				((WOPGameData) getGameData()).getMapDocument());
	}

	public WOPDialogMode startSetup() {
		WOPGameSetup setup = new WOPGameSetup(this);
		setSetup(setup);
		getSetup().buildPhaseList();
		WOPDialogMode mode = (WOPDialogMode) getGameData().getModeList().get(
				WOPDialogMode.MODE_NAME);
		getGameData().setHandlerMap(mode.getHandlerMap());
		return mode;
	}

	public CGFMainFrame createMainFrame() {
		return getFactory().createMainFrame();
	}

	public CGFGameData createGameData() {
		return getFactory().createGameData();
	}

	public WOPGameSetup getSetup() {
		return setup;
	}

	public void setSetup(WOPGameSetup setup) {
		this.setup = setup;
	}

	public WOPFactory getFactory() {
		return (WOPFactory) super.getFactory();
	}

	public WOPGameData getGameData() {
		return (WOPGameData) super.getGameData();
	}

	public List readMapList() {
		List list = new ArrayList();
		File resourceDir = new File("resource");
		CGFFileFilter filter = new CGFFileFilter();
		filter.add("map");
		File[] files = resourceDir.listFiles(filter);
		if (files != null) {
			for (int i = 0; i < files.length; i++) {
				if (files[i].getName().charAt(0) == '.') {
					continue;
				}
				if (files[i].isDirectory()) {
					continue;
				}
				try {
					list.add(new WOPMapInfo(this, files[i].getPath()));
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}

		return list;
	}

	public void setImageFileChooser(JFileChooser imageFileChooser) {
		this.imageFileChooser = imageFileChooser;
	}

	public JFileChooser getImageFileChooser() {
		return imageFileChooser;
	}

	public void setMapFileChooser(JFileChooser mapFileChooser) {
		this.mapFileChooser = mapFileChooser;
	}

	public JFileChooser getMapFileChooser() {
		return mapFileChooser;
	}

	public void setSoundSetting(WOPSoundSetting soundSetting) {
		super.setSoundSetting(soundSetting);
	}

	public WOPSoundSetting getSoundSetting() {
		return (WOPSoundSetting) super.getSoundSetting();
	}
}
