package sjima.wop;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import sjima.cgf.CGFMain;
import sjima.cgf.CGFMapDocument;
import sjima.cgf.CGFTerrain;
import sjima.cgf.CGFXMLUtil;
import sjima.wop.event.WOPEventManager;

public class WOPMapDocument extends CGFMapDocument {

	public static final String SAVE_FILE_EXT = "wop";
	public static final String MAP_FILE_EXT = "map";
	public static final String IMAGE_FILE_EXT = "png";

	public WOPMapDocument(CGFMain main, Document doc) {
		super(main, doc);
	}

	public WOPTerrain createTerrainInstance(Node node) {
		WOPTerrain terrain = WOPTerrain.create(getMain(), node);
		return terrain;
	}

	public void saveGameData(String path) {
		setPlayerList();
		setForceList();
		setTurnPlayerNo();
		setCombatResultList();
		try {
			CGFXMLUtil.saveXML(getMapDocument(), path);
		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "エラーが発生しました（" + e + ")");
			return;
		}
	}

	private void setTurnPlayerNo() {
		getMain().getGameData().getTurnManager().setTurnPlayerPhase(this);
	}

	public void loadGameData(String path) throws Exception {
		Document doc = CGFXMLUtil.loadXML(path);
		setMapDocument(doc);
	}

	public void setPlayerList() {
		List playerList = getMain().getGameData().getPlayerList();
		Node gameDataNode = CGFXMLUtil.readNodeByXPath(getMapDocument(),
				"/gamedata");
		Node oldPlayerListNode = (CGFXMLUtil.readNodeByXPath(getMapDocument(),
				"/gamedata/playerlist"));
		Element playerListNode = getMapDocument().createElement("playerlist");
		for (Iterator ite = playerList.iterator(); ite.hasNext();) {
			WOPPlayer player = (WOPPlayer) ite.next();
			Node node = player.createNode(getMapDocument());
			playerListNode.appendChild(node);
		}
		playerListNode.setAttribute("count", String.valueOf(getMain()
				.getGameData().getNinzu()));
		gameDataNode.removeChild(oldPlayerListNode);
		gameDataNode.appendChild(playerListNode);
	}

	public void setForceList() {
		Node mapNode = CGFXMLUtil.readNodeByXPath(getMapDocument(),
				"/gamedata/map");
		Node oldForceListNode = CGFXMLUtil.getChildNode(mapNode, "forcelist");
		Node forceListNode = getMain().getGameData().getForceManager()
				.createNode(getMapDocument());

		mapNode.removeChild(oldForceListNode);
		mapNode.appendChild(forceListNode);
	}

	public void setCombatResultList() {
		Node gameDataNode = CGFXMLUtil.readNodeByXPath(getMapDocument(),
				"/gamedata");
		Node oldCombatResultListNode = (CGFXMLUtil.readNodeByXPath(
				getMapDocument(), "/gamedata/combatResultList"));
		Node combatResultListNode = getMapDocument().createElement(
				"combatResultList");
		for (Iterator ite = getMain().getGameData().getCombatResultList()
				.iterator(); ite.hasNext();) {
			WOPCombatResult cr = (WOPCombatResult) ite.next();
			Node node = cr.createNode(getMapDocument());
			combatResultListNode.appendChild(node);
		}
		if (oldCombatResultListNode != null) {
			gameDataNode.removeChild(oldCombatResultListNode);
		}
		gameDataNode.appendChild(combatResultListNode);
	}

	public WOPForceManager getForceManager() {
		return new WOPForceManager(getMain(), getMapDocument());
	}

	public String getMaxPlayer() {
		return CGFXMLUtil.readByXPath(getMapDocument(), "/gamedata/@maxplayer");
	}

	public String getEndTurn() {
		return CGFXMLUtil.readByXPath(getMapDocument(), "/gamedata/@endturn");
	}

	public WOPMain getMain() {
		return (WOPMain) super.getMain();
	}

	public List getCombatResultList() {
		List result = new ArrayList();

		List nodeList = CGFXMLUtil.readListByXPath(getMapDocument(),
				"/gamedata/combatResultList/combatResult");
		if (nodeList != null) {
			for (Iterator ite = nodeList.iterator(); ite.hasNext();) {
				Node node = (Node) ite.next();
				WOPCombatResult cr = new WOPCombatResult(getMain(), node);
				result.add(cr);
			}
		}

		return result;
	}

	public WOPEventManager getEventManager() {
		return new WOPEventManager(getMain(), getMapDocument());
	}

	public Map getTerrainMap() {
		String xpath = "/gamedata/map/ttypelist/ttype";
		List list = CGFXMLUtil.readListByXPath(getMapDocument(), xpath);
		Map map = new LinkedHashMap();
		for (Iterator ite = list.iterator(); ite.hasNext();) {
			Node node = (Node) ite.next();
			CGFTerrain terrain = createTerrainInstance(node);
			map.put(terrain.getTerrain(), terrain);
		}
		return map;
	}


}
