/**
 * Wars of Powers 
 * 
 * copyright 2009 Mitsunobu SHIOJIMA
 */

package sjima.wop;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import sjima.cgf.CGFXMLUtil;

public class WOPMapInfo implements Comparable {
	private WOPMain main;
	private String path;
	private String no;
	private String title;
	private String region;
	private String players;
	private String version;
	private String maxPlayer;
	private List commentList = new ArrayList();
	private String endTurn;
	private List playerList = new ArrayList();

	public WOPMapInfo(WOPMain main, String path) throws Exception {
		setMain(main);
		setPath(path);
		Document doc = CGFXMLUtil.loadXML(getPath());
		Node node = doc.getDocumentElement();
		setNo(CGFXMLUtil.getAttributeByName(node, "no"));
		setTitle(CGFXMLUtil.getAttributeByName(node, "title"));
		setRegion(CGFXMLUtil.getAttributeByName(node, "region"));
		setPlayers(CGFXMLUtil.getAttributeByName(node, "players"));
		setVersion(CGFXMLUtil.getAttributeByName(node, "version"));
		setMaxPlayer(CGFXMLUtil.getAttributeByName(node, "maxplayer"));
		setEndTurn(CGFXMLUtil.getAttributeByName(node, "endturn"));
		List nodeList = CGFXMLUtil.readListByXPath(node, "commentlist/comment");
		getCommentList().clear();
		for (Iterator ite = nodeList.iterator(); ite.hasNext();) {
			Node commentNode = (Node) ite.next();
			String text = commentNode.getTextContent();
			getCommentList().add(text);
		}
		nodeList = CGFXMLUtil.readListByXPath(node, "playerlist/player");
		getPlayerList().clear();
		for (Iterator ite = nodeList.iterator(); ite.hasNext();) {
			Node playerNode = (Node) ite.next();
			String name = CGFXMLUtil.getAttributeByName(playerNode, "name");
			getPlayerList().add(name);
		}
	}

	public WOPMain getMain() {
		return main;
	}

	public void setMain(WOPMain main) {
		this.main = main;
	}

	public String getPath() {
		return path;
	}

	public void setPath(String path) {
		this.path = path;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getRegion() {
		return region;
	}

	public void setRegion(String region) {
		this.region = region;
	}

	public String getPlayers() {
		return players;
	}

	public void setPlayers(String players) {
		this.players = players;
	}

	public void setVersion(String version) {
		this.version = version;
	}

	public String getVersion() {
		return version;
	}

	public String getTextString() {
		return String.format("%-16s%-14s%-10s", getTitle(), getRegion(),
				getPlayers());
	}

	public void setNo(String no) {
		this.no = no;
	}

	public String getNo() {
		return no;
	}

	public int compareTo(Object o) {
		return getNo().compareTo(((WOPMapInfo) o).getNo());
	}

	public void setMaxPlayer(String maxPlayer) {
		this.maxPlayer = maxPlayer;
	}

	public String getMaxPlayer() {
		return maxPlayer;
	}

	public List getCommentList() {
		return commentList;
	}

	public void setEndTurn(String endTurn) {
		this.endTurn = endTurn;
	}

	public String getEndTurn() {
		return endTurn;
	}

	public List getPlayerList() {
		return playerList;
	}
}
