package sjima.wop;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import sjima.cgf.CGFLocation;
import sjima.cgf.CGFPlayer;
import sjima.cgf.CGFXMLUtil;
import sjima.wop.event.WOPEvent;

public class WOPPlayer extends CGFPlayer {
	public static final int DFC_EASY = 0;
	public static final int DFC_NORMAL = 1;
	public static final int DFC_HARD = 2;
	public static final int DFC_HARDEST = 3;
	public static final int DEFAULT_HUMAN_DFC = DFC_NORMAL;
	public static final int DEFAULT_AI_DFC = DFC_EASY;

	public static final int AFP_EASY = 50;
	public static final int AFP_NORMAL = 53;
	public static final int AFP_HARD = 57;
	public static final int AFP_HARDEST = 62;

	private int difficulty = DFC_EASY;
	private Map dfcMap = new HashMap();
	private CGFLocation targetLoc;
	private CGFLocation lastTargetLoc;
	private int totalStreangth = 0;
	private int totalFertility = 0;
	private double vprate = 1.0;
	private String enemy;
	private CGFLocation home;
	private int vp;
	private LinkedHashMap paraMap = new LinkedHashMap();
	private List commentList = new ArrayList();

	private final int[] afpTable = { AFP_EASY, AFP_NORMAL, AFP_HARD,
			AFP_HARDEST };

	private int recruitment = 0;

	public WOPPlayer(WOPMain main, Node node) {
		super(main, node);
		setDifficulty(CGFXMLUtil.getAttributeIntegerByName(node, "difficulty"));
		setRecruitment(CGFXMLUtil
				.getAttributeIntegerByName(node, "recruitment"));
		setHome(new CGFLocation(CGFXMLUtil.getAttributeByName(node, "home")));
		setVp(CGFXMLUtil.getAttributeIntegerByName(node, "vp"));
		String str = CGFXMLUtil.getAttributeByName(node, "targetloc");
		if (str != null) {
			setTargetLoc(new CGFLocation(str));
		} else {
			setTargetLoc(null);
		}
		str = CGFXMLUtil.getAttributeByName(node, "lasttargetloc");
		if (str != null) {
			setLastTargetLoc(new CGFLocation(str));
		} else {
			setLastTargetLoc(null);
		}
		setTotalStreangth(CGFXMLUtil.getAttributeIntegerByName(node,
				"totalstrength"));
		setTotalFertility(CGFXMLUtil.getAttributeIntegerByName(node,
				"totalfertility"));
		str = CGFXMLUtil.getAttributeByName(node, "enemy");
		if ("".equals(str)) {
			str = null;
		}
		setEnemy(str);

		Double d = CGFXMLUtil.getAtributeDoubleByName(node, "vprate");
		if (d != 0) {
			setVprate(d);
		}

		getDfcMap().put(new Integer(DFC_EASY), "簡単");
		getDfcMap().put(new Integer(DFC_NORMAL), "普通");
		getDfcMap().put(new Integer(DFC_HARD), "困難");
		getDfcMap().put(new Integer(DFC_HARDEST), "至難");

		List nodeList = CGFXMLUtil.readListByXPath(node, "para");
		for (Iterator ite = nodeList.iterator(); ite.hasNext();) {
			Node paraNode = (Node) ite.next();
			Parameter para = new Parameter(paraNode);
			getParaMap().put(para.getKey(), para);
		}

		nodeList = CGFXMLUtil.readListByXPath(node, "comment");
		for (Iterator ite = nodeList.iterator(); ite.hasNext();) {
			Node commentNode = (Node) ite.next();
			getCommentList().add(commentNode.getTextContent());
		}
	}

	public int getTerrainModify(String ttype, String attrName) {
		Parameter para = (Parameter) getParaMap().get(
				Parameter.TYPE_TERAIN + "-" + ttype);
		if (para == null) {
			return 0;
		}
		String mod = (String) para.getMap().get(attrName);
		if (mod == null) {
			return 0;
		}
		if (mod.charAt(0) == '+') {
			mod = mod.substring(1);
		}
		return Integer.parseInt(mod);
	}

	public int getEventModify(String eno, String attrName) {
		Parameter para = (Parameter) getParaMap().get(
				Parameter.TYPE_EVENT + "-" + eno);
		if (para == null) {
			return 0;
		}
		String mod = (String) para.getMap().get(attrName);
		if (mod == null) {
			return 0;
		}
		if (mod.charAt(0) == '+') {
			mod = mod.substring(1);
		}
		return Integer.parseInt(mod);
	}

	public Node createNode(Document doc) {
		Element node = (Element) super.createNode(doc);
		node.setAttribute("difficulty", String.valueOf(getDifficulty()));
		node.setAttribute("recruitment", String.valueOf(getRecruitment()));
		node.setAttribute("home", getHome().toString());
		node.setAttribute("vp", String.valueOf(getVp()));
		if (getTargetLoc() != null) {
			node.setAttribute("targetloc", getTargetLoc().toString());
		}
		if (getLastTargetLoc() != null) {
			node.setAttribute("lasttargetloc", getLastTargetLoc().toString());
		}
		node.setAttribute("totalstrength", String.valueOf(getTotalStreangth()));
		node
				.setAttribute("totalfertility", String
						.valueOf(getTotalFertility()));
		node.setAttribute("enemy", getEnemy());
		node.setAttribute("vprate", String.valueOf(getVprate()));

		for (Iterator ite = getParaMap().values().iterator(); ite.hasNext();) {
			Parameter para = (Parameter) ite.next();
			node.appendChild(para.createNode(doc));
		}

		for (Iterator ite = getCommentList().iterator(); ite.hasNext();) {
			String comment = (String) ite.next();
			Element elm = doc.createElement("comment");
			elm.setTextContent(comment);
			node.appendChild(elm);
		}

		return node;
	}

	public int getAddForcePoint() {
		return getAfpTable()[getDifficulty()];
	}

	public WOPPlayer(WOPMain main, String no) {
		super(main, no);
	}

	public WOPMain getMain() {
		return (WOPMain) super.getMain();
	}

	public void setDifficulty(int difficulty) {
		this.difficulty = difficulty;
	}

	public int getDifficulty() {
		return difficulty;
	}

	public Map getDfcMap() {
		return dfcMap;
	}

	public String getDifficultyText() {
		return (String) getDfcMap().get(new Integer(getDifficulty()));
	}

	public void setRecruitment(int recruitment) {
		this.recruitment = recruitment;
	}

	public int getRecruitment() {
		return recruitment;
	}

	public int[] getAfpTable() {
		return afpTable;
	}

	public void setTargetLoc(CGFLocation targetLoc) {
		this.targetLoc = targetLoc;
		if (targetLoc != null) {
			if (getMain().getGameData().getForceManager() != null) {
				String ctrl = getMain().getGameData().getForceManager()
						.getControll(targetLoc);
				setEnemy(ctrl);
				setLastTargetLoc(targetLoc);
			}
		}
	}

	public CGFLocation getTargetLoc() {
		return targetLoc;
	}

	public int getTotalStreangth() {
		return totalStreangth;
	}

	public void setTotalStreangth(int totalStreangth) {
		this.totalStreangth = totalStreangth;
	}

	public int getTotalFertility() {
		return totalFertility;
	}

	public void setTotalFertility(int totalFertility) {
		this.totalFertility = totalFertility;
	}

	public int getMaxStreangth() {
		return (int) ((double) getTotalFertility() * 1.0 + 0.001);
	}

	public void setEnemy(String enemy) {
		if (getNo().equals(enemy)) {
			this.enemy = null;
		} else {
			this.enemy = enemy;
		}
	}

	public String getEnemy() {
		return enemy;
	}

	public void setLastTargetLoc(CGFLocation lastTargetLoc) {
		this.lastTargetLoc = lastTargetLoc;
	}

	public CGFLocation getLastTargetLoc() {
		return lastTargetLoc;
	}

	public void setHome(CGFLocation home) {
		this.home = home;
	}

	public CGFLocation getHome() {
		return home;
	}

	public void setVp(int vp) {
		this.vp = vp;
	}

	public int getVp() {
		return vp;
	}

	public Map getParaMap() {
		return this.paraMap;
	}

	public List getCommentList() {
		return this.commentList;
	}

	public void setVprate(double vprate) {
		this.vprate = vprate;
	}

	public double getVprate() {
		return vprate;
	}

	public static class VPComparator implements Comparator {

		public int compare(Object o1, Object o2) {
			WOPPlayer p1 = (WOPPlayer) o1;
			WOPPlayer p2 = (WOPPlayer) o2;

			return p2.getVp() - p1.getVp();
		}

	}

	public class Parameter {
		public static final String TYPE_EVENT = "EV";
		public static final String TYPE_TERAIN = "TR";

		private String type;
		private LinkedHashMap map = new LinkedHashMap();

		public Parameter(Node node) {
			String eno = CGFXMLUtil.getAttributeByName(node, "eno");
			if (eno != null) {
				setType(TYPE_EVENT);
			} else {
				setType(TYPE_TERAIN);
			}

			NamedNodeMap attrs = node.getAttributes();
			for (int i = 0; i < attrs.getLength(); i++) {
				Node attr = attrs.item(i);
				getMap().put(attr.getNodeName(), attr.getNodeValue());
			}
		}

		public Node createNode(Document doc) {
			Element node = doc.createElement("para");
			for (Iterator ite = getMap().keySet().iterator(); ite.hasNext();) {
				String name = (String) ite.next();
				String value = (String) getMap().get(name);
				node.setAttribute(name, value);
			}
			return node;
		}

		public void setType(String type) {
			this.type = type;
		}

		public String getType() {
			return type;
		}

		public Map getMap() {
			return map;
		}

		public String getKey() {
			if (TYPE_EVENT.equals(getType())) {
				return TYPE_EVENT + "-" + getMap().get("eno");
			}
			return TYPE_TERAIN + "-" + getMap().get("ttype");
		}

		public String toString() {
			StringBuffer buf = new StringBuffer(100);
			WOPPlayer player = getMain().getGameData().getPlayer(getNo());

			if (WOPPlayer.Parameter.TYPE_TERAIN.equals(getType())) {
				WOPTerrain terrain = (WOPTerrain) getMain().getGameData()
						.getTerrainMap().get(getMap().get("ttype"));
				buf.append(terrain.getName() + ": ");
				String value = (String) getMap().get("fertility");
				if (value != null) {
					buf.append("肥沃度");
					buf
							.append(value + "(" + terrain.getFertility(player)
									+ ")");
				}
				value = (String) getMap().get("attack");
				if (value != null) {
					if (buf.charAt(buf.length() - 1) != ' ') {
						buf.append(", ");
					}
					buf.append("攻撃戦力修正");
					buf
							.append(value + "%(+" + terrain.getAttack(player)
									+ "%)");
				}
				value = (String) getMap().get("defence");
				if (value != null) {
					if (buf.charAt(buf.length() - 1) != ' ') {
						buf.append(", ");
					}
					buf.append("防御戦力修正");
					buf.append(value + "%(+" + terrain.getDefence(player)
							+ "%)");
				}
				value = (String) getMap().get("capacity");
				if (value != null) {
					if (buf.charAt(buf.length() - 1) != ' ') {
						buf.append(", ");
					}
					buf.append("最大兵力");
					buf.append(value + "(" + terrain.getCapacity(player) + ")");
				}
				value = (String) getMap().get("garrison");
				if (value != null) {
					if (buf.charAt(buf.length() - 1) != ' ') {
						buf.append(", ");
					}
					buf.append("守備兵力");
					buf.append(value + "(" + terrain.getGarrison(player) + ")");
				}
				value = (String) getMap().get("maxgrowth");
				if (value != null) {
					if (buf.charAt(buf.length() - 1) != ' ') {
						buf.append(", ");
					}
					buf.append("最大増加兵力");
					buf
							.append(value + "(" + terrain.getMaxGrowth(player)
									+ ")");
				}
				value = (String) getMap().get("movestart");
				if (value != null) {
					if (buf.charAt(buf.length() - 1) != ' ') {
						buf.append(", ");
					}
					buf.append("移動開始兵力");
					buf
							.append(value + "(" + terrain.getMoveStart(player)
									+ ")");
				}
				value = (String) getMap().get("attackMorale");
				if (value != null) {
					if (buf.charAt(buf.length() - 1) != ' ') {
						buf.append(", ");
					}
					buf.append("攻撃士気");
					if (terrain.getAttackMorale(player) >= 0) {
						buf.append(value + "(+"
								+ terrain.getAttackMorale(player) + ")");
					} else {
						buf.append(value + "("
								+ terrain.getAttackMorale(player) + ")");
					}
				}
				value = (String) getMap().get("defenceMorale");
				if (value != null) {
					if (buf.charAt(buf.length() - 1) != ' ') {
						buf.append(", ");
					}
					buf.append("防御士気");
					if (terrain.getDefenceMorale(player) >= 0) {
						buf.append(value + "(+"
								+ terrain.getDefenceMorale(player) + ")");
					} else {
						buf.append(value + "("
								+ terrain.getDefenceMorale(player) + ")");
					}
				}
			} else {
				WOPEvent event = (WOPEvent) getMain().getGameData()
						.getEventManager().getEventMap().get(
								getMap().get("eno"));
				buf.append(event.getName() + ":");
				String drate = (String) getMap().get("drate");
				if (drate != null) {
					drate = drate.substring(0, drate.length() - 1) + "."
							+ drate.substring(drate.length() - 1);
					buf.append("損害率" + drate + "%");
				}
			}

			return buf.toString();
		}

	}
}
