package sjima.wop;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Node;

import sjima.cgf.CGFGamePhase;
import sjima.cgf.CGFMain;
import sjima.cgf.CGFPhaseManager;
import sjima.cgf.CGFTurnManager;
import sjima.cgf.CGFXMLUtil;

public class WOPTurnManager extends CGFTurnManager {
	private List stageList = new ArrayList();

	public WOPTurnManager(CGFMain main, CGFPhaseManager phaseManager,
			WOPMapDocument mapDocument) {
		super(main, phaseManager);
		List nodeList = CGFXMLUtil.readListByXPath(
				mapDocument.getMapDocument(), "/gamedata/stagelist/stage");
		for (Iterator ite = nodeList.iterator(); ite.hasNext(); ) {
			Node node = (Node)ite.next();
			StageData stage = new StageData(node);
			getStageList().add(stage);
		}
	}

	public void nextPhase() {
		CGFGamePhase nextPhase = getPhaseManager().nextPhase();
		if (nextPhase != null) {
			nextPhase.execute();
		} else {
			nextTurn();
		}
	}

	public WOPPlayer nextPlayerTurn() {
		int nextPlayerNo = getPlayerNo() + 1;
		if (nextPlayerNo < getMain().getGameData().getNinzu()) {
			setPlayerNo(nextPlayerNo);
			return getTurnPlayer();
		}
		return null;
	}

	public WOPPlayer getTurnPlayer() {
		return (WOPPlayer) getPlayerList().get(getPlayerNo());
	}

	public List getStageList() {
		return stageList;
	}
	
	public String getStage() {
		String stageNo = null;
		for (Iterator ite = getStageList().iterator(); ite.hasNext(); ) {
			StageData stageData = (StageData)ite.next();
			if (getTurn() < stageData.getTurn()) {
				break;
			}
			stageNo = stageData.getNo();
		}
		return stageNo;
	}

	public class StageData {
		public StageData(Node node) {
			setNo(CGFXMLUtil.getAttributeByName(node, "no"));
			setTurn(CGFXMLUtil.getAttributeIntegerByName(node, "turn"));
		}

		private String no = null;
		private int turn = 0;

		public void setNo(String no) {
			this.no = no;
		}

		public String getNo() {
			return no;
		}

		public void setTurn(int turn) {
			this.turn = turn;
		}

		public int getTurn() {
			return turn;
		}
	}
}
