/**
 * Wars of Powers 
 * 
 * copyright 2009 Mitsunobu SHIOJIMA
 */
package sjima.wop.action;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.imageio.ImageIO;
import javax.swing.JOptionPane;

import sjima.cgf.CGFDialogAction;
import sjima.cgf.CGFUIHandler;
import sjima.cgf.CGFUIImage;
import sjima.cgf.CGFUtil;
import sjima.wop.WOPGameData;
import sjima.wop.WOPMain;
import sjima.wop.WOPMapInfo;
import sjima.wop.mode.WOPDialogMode;

public class WOPMapSelectDialogAction extends CGFDialogAction {

	public static final String DIALOG_NAME = "mapselect";
	public static final int ITEM_COUNT = 24;
	public static final int COLUMN_COUNT = 8;
	private final static Color CLEAR_COLOR = new Color(255, 255, 255, 0);
	private List mapInfoList;
	private int offset = 0;
	private SortedMap createImageMap = Collections
			.synchronizedSortedMap(new TreeMap());
	private Thread imageCreator = null;

	public WOPMapSelectDialogAction(WOPMain main) {
		setMain(main);
	}

	public void commit() {
	}

	public Map createHandlerMap() {
		Map map = new HashMap();
		for (int i = 0; i < 8; i++) {
			CGFUIHandler handler = new ItemHandler(ItemHandler.HANDLER_NAME
					+ String.valueOf(i + 1), i);
			map.put(handler.getName(), handler);
			handler = new InfoHandler(InfoHandler.HANDLER_NAME
					+ String.valueOf(i + 1), i);
			map.put(handler.getName(), handler);
		}
		CGFUIHandler handler = new UpHandler();
		map.put(handler.getName(), handler);
		handler = new DownHandler();
		map.put(handler.getName(), handler);
		handler = new ReturnHandler();
		map.put(handler.getName(), handler);

		return map;
	}

	public String getDialogName() {
		return DIALOG_NAME;
	}

	public void setup() {
		setOffset(0);
		updateDialog();
	}

	private void updateDialog() {
		if (getDialog() == null) {
			return;
		}
		if (!DIALOG_NAME.equals(getDialog().getName())) {
			return;
		}
		for (int i = 0; i < ITEM_COUNT; i++) {
			int ix = i + getOffset();
			if (ix < getMapInfoList().size()) {
				byte[] bytes = null;
				CGFUIImage part = (CGFUIImage) getDialog().getPartsMap().get(
						ItemHandler.HANDLER_NAME + String.valueOf(i + 1));
				WOPMapInfo mapInfo = (WOPMapInfo) getMapInfoList().get(ix);
				String mapImage = "resource/" + mapInfo.getNo() + ".png";
				BufferedImage image = CGFUtil.getImage(mapImage);
				if (image == null) {
					getCreateImageMap().put(mapInfo.getNo(), mapInfo);
				} else {
					part.setImagePath(mapImage);
				}
				part.setEnable(true);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						DIALOG_NAME + ".name" + String.valueOf(i + 1));
				part.setLabel(CGFUtil
						.insertCRIntoString(mapInfo.getTitle(), 11));
				part.setEnable(true);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						DIALOG_NAME + ".area" + String.valueOf(i + 1));
				try {
					bytes = mapInfo.getRegion().getBytes("MS932");
					if (bytes.length > 6) {
						part.setFontSize(9);
					} else {
						part.setFontSize(10);
					}
				} catch (UnsupportedEncodingException e) {
					e.printStackTrace();
				}
				part.setLabel(mapInfo.getRegion());
				part.setEnable(true);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						DIALOG_NAME + ".ninzu" + String.valueOf(i + 1));
				part.setLabel(mapInfo.getPlayers());
				part.setEnable(true);
			} else {
				CGFUIImage part = (CGFUIImage) getDialog().getPartsMap().get(
						ItemHandler.HANDLER_NAME + String.valueOf(i + 1));
				part.setEnable(false);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						DIALOG_NAME + ".name" + String.valueOf(i + 1));
				part.setLabel("");
				part.setEnable(false);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						DIALOG_NAME + ".area" + String.valueOf(i + 1));
				part.setLabel("");
				part.setEnable(false);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						DIALOG_NAME + ".ninzu" + String.valueOf(i + 1));
				part.setLabel("");
				part.setEnable(false);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						DIALOG_NAME + ".info" + String.valueOf(i + 1));
				part.setEnable(false);
			}
		}
		CGFUIImage part = (CGFUIImage) getDialog().getPartsMap().get(
				"mapselect.up");
		if (getOffset() > 0) {
			part.setEnable(true);
		} else {
			part.setEnable(false);
		}
		part = (CGFUIImage) getDialog().getPartsMap().get("mapselect.down");
		if ((getOffset() + ITEM_COUNT) < getMapInfoList().size()) {
			part.setEnable(true);
		} else {
			part.setEnable(false);
		}
		if (getCreateImageMap().size() > 0) {
			if (getImageCreator() == null) {
				Thread thread = (new Thread(new ImageCreator()));
				thread.setPriority(Thread.MIN_PRIORITY);
				setImageCreator(thread);
				thread.start();
			}
		}
	}

	public WOPMain getMain() {
		return (WOPMain) super.getMain();
	}

	public void setMapInfoList(List mapInfoList) {
		this.mapInfoList = mapInfoList;
	}

	public List getMapInfoList() {
		return mapInfoList;
	}

	public void setOffset(int offset) {
		this.offset = offset;
	}

	public int getOffset() {
		return offset;
	}

	public class ItemHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "mapselect.gazo";
		private String name;
		private int no;

		public ItemHandler(String name, int no) {
			setName(name);
			setNo(no);
		}

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public void onClick() {
			((WOPMain) getMain()).getGameData().setMapInfo(
					(WOPMapInfo) getMapInfoList().get(getNo() + getOffset()));
			List msgList = new ArrayList();
			msgList.add("地図を読み込んでいます。");
			msgList.add("しばらくお待ちください。");
			((WOPMain) getMain()).getGameData().displayMessageList(msgList,
					null, 0, false);
			try {
				((WOPMain) getMain()).setupGameData(((WOPMain) getMain())
						.getGameData().getMapInfo().getPath());
			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, "エラーが発生しました（" + e + ")");
				return;
			}
			getMain().getGameData().setCurrentMode(getDialogMode());
			CGFDialogAction action = getDialogMode().getAction(
					WOPSetup1DialogAction.DIALOG_NAME);
			getDialogMode().dispDialog(action);
		}

		public void onMove() {
		}

		public void onRightClick() {
		}

		public void setNo(int no) {
			this.no = no;
		}

		public int getNo() {
			return no;
		}

	}

	public class InfoHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "mapselect.info";
		private String name;
		private int no;

		public InfoHandler(String name, int no) {
			setName(name);
			setNo(no);
		}

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public void onClick() {
			closeDialog();
			WOPMapInfo mapInfo = (WOPMapInfo) getMapInfoList().get(
					getNo() + getOffset());

			WOPDialogMode dialogMode = (WOPDialogMode) ((WOPMain) getMain())
					.getGameData().getCurrentMode();
			dialogMode.setFinishHandler(null);
			WOPScenarioDialogAction action = (WOPScenarioDialogAction) dialogMode
					.getAction(WOPScenarioDialogAction.DIALOG_NAME);
			action.setReturnDialog(getDialogName());
			action.setMapInfo(mapInfo);
			dialogMode.dispDialog(action);
		}

		public void onMove() {
		}

		public void onRightClick() {
		}

		public void setNo(int no) {
			this.no = no;
		}

		public int getNo() {
			return no;
		}

	}

	public class UpHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "mapselect.up";

		public UpHandler() {
		}

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			setOffset(getOffset() - COLUMN_COUNT);
			updateDialog();
			((WOPDialogMode) getDialogMode()).repaintDialog();
		}

		public void onMove() {
		}

		public void onRightClick() {
		}
	}

	public class DownHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "mapselect.down";

		public DownHandler() {
		}

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			setOffset(getOffset() + COLUMN_COUNT);
			updateDialog();
			((WOPDialogMode) getDialogMode()).repaintDialog();
		}

		public void onMove() {
		}

		public void onRightClick() {
		}
	}

	public class ReturnHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "mapselect.return";

		public ReturnHandler() {
		}

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			getDialogMode().closeDialog();
			getMain().startSetup();
			getMain().getSetup().start();
		}

		public void onMove() {
		}

		public void onRightClick() {
		}
	}

	public Map getCreateImageMap() {
		return createImageMap;
	}

	public void setImageCreator(Thread imageCreator) {
		this.imageCreator = imageCreator;
	}

	public Thread getImageCreator() {
		return imageCreator;
	}

	public class ImageCreator implements Runnable {

		public void run() {
			BufferedImage image = new BufferedImage(64, 64,
					BufferedImage.TYPE_INT_ARGB);
			while (getCreateImageMap().size() > 0) {
				WOPMapInfo mapInfo = (WOPMapInfo) getCreateImageMap().values()
						.iterator().next();
				String mapImagePath = "resource/" + mapInfo.getNo() + ".png";
				getMain().getGameData();
				BufferedImage mapImage = WOPGameData.createMapImage(mapInfo
						.getPath());
				int width = 64;
				int height = 64;
				if (mapImage.getWidth() < mapImage.getHeight()) {
					height = (int) (64.0 * ((double) mapImage.getHeight() / mapImage
							.getWidth()));
				} else {
					width = (int) (64.0 * ((double) mapImage.getWidth() / mapImage
							.getHeight()));
				}
				Image img = mapImage.getScaledInstance(width, height,
						Image.SCALE_SMOOTH);
				Graphics2D gc = image.createGraphics();
				gc.setBackground(CLEAR_COLOR);
				gc.clearRect(0, 0, image.getWidth(), image.getHeight());
				gc.drawImage(img, 0, 0, 64, 64, null);
				try {
					ImageIO.write(image, "PNG", new File(mapImagePath));
				} catch (IOException e) {
					e.printStackTrace();
				}
				getCreateImageMap().remove(mapInfo.getNo());
				updateDialog();
				((WOPDialogMode) getDialogMode()).repaintDialog();
			}
			setImageCreator(null);
		}
	}
}
