/**
 * Wars of Powers 
 * 
 * copyright 2009 Mitsunobu SHIOJIMA
 */
package sjima.wop.action;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import sjima.cgf.CGFDialogAction;
import sjima.cgf.CGFUIHandler;
import sjima.wop.WOPGameData;
import sjima.wop.WOPMain;
import sjima.wop.WOPMapDocument;
import sjima.wop.mode.WOPDialogMode;

public class WOPMenu1Action extends CGFDialogAction {
	public static final String MENU_NAME = "menu1";
	private String returnDialog = null;

	public WOPMenu1Action(WOPMain main) {
		setMain(main);
	}

	public void commit() {
	}

	public Map createHandlerMap() {
		Map map = new HashMap();
		CGFUIHandler handler = new SaveHandler();
		map.put(handler.getName(), handler);
		handler = new LoadHandler();
		map.put(handler.getName(), handler);
		handler = new ExitHandler();
		map.put(handler.getName(), handler);
		handler = new ImageHandler();
		map.put(handler.getName(), handler);
		handler = new ScenarioHandler();
		map.put(handler.getName(), handler);
		handler = new TerrainHandler();
		map.put(handler.getName(), handler);
		handler = new EventHandler();
		map.put(handler.getName(), handler);
		return map;
	}

	public String getDialogName() {
		return MENU_NAME;
	}

	public void setup() {
		setCloseEnable(true);
	}

	public void exitMenu() {
		((WOPDialogMode) (getMain().getGameData().getModeList()
				.get(WOPDialogMode.MODE_NAME))).closeMenu();
	}

	public void setReturnDialog(String returnDialog) {
		this.returnDialog = returnDialog;
	}

	public String getReturnDialog() {
		return returnDialog;
	}

	public void closeDialog() {
		getMain().getGameData().closeDialog();
		if (getReturnDialog() != null) {
			((WOPMain) getMain()).getGameData()
					.displayDialog(getReturnDialog());
		}
	}

	public class SaveHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "menu1.save";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			JFileChooser chooser = getMain().getSaveFileChooser();
			chooser.setCurrentDirectory(WOPGameData.SAVE_DIR);
			int returnVal = chooser.showSaveDialog(null);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = chooser.getSelectedFile();
				String path = file.getAbsolutePath();
				if (!chooser.accept(file)) {
					path += "." + WOPMapDocument.SAVE_FILE_EXT;
				}
				((WOPMain) getMain()).getGameData().saveGameData(path);
			}
		}

		public void onRightClick() {
		}

		public void onMove() {
		}
	}

	public class LoadHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "menu1.load";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			JFileChooser chooser = getMain().getSaveFileChooser();
			chooser.setCurrentDirectory(WOPGameData.SAVE_DIR);
			int returnVal = chooser.showOpenDialog(null);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = chooser.getSelectedFile();
				String path = file.getAbsolutePath();
				if (!chooser.accept(file)) {
					path += "." + WOPMapDocument.SAVE_FILE_EXT;
				}

				try {
					List msgList = new ArrayList();
					msgList.add("地図を読み込んでいます。");
					msgList.add("しばらくお待ちください。");
					((WOPMain) getMain()).getGameData().displayMessageList(
							msgList, null, 0, false);

					((WOPMain) getMain()).setupGameData(path);
				} catch (Exception e) {
					e.printStackTrace();
					JOptionPane
							.showMessageDialog(null, "エラーが発生しました（" + e + ")");
					return;
				}
				getMain().getGameData().repaintBoard();
				getMain().getGameData().getPhaseManager().getCurrentPhase()
						.restart();
			}
		}

		public void onRightClick() {
		}

		public void onMove() {
		}
	}

	public class ImageHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "menu1.image";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			JFileChooser chooser = ((WOPMain) getMain()).getImageFileChooser();
			chooser.setCurrentDirectory(WOPGameData.SAVE_DIR);
			int returnVal = chooser.showSaveDialog(null);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = chooser.getSelectedFile();
				String path = file.getAbsolutePath();
				if (!chooser.accept(file)) {
					path += "." + WOPMapDocument.IMAGE_FILE_EXT;
				}
				((WOPMain) getMain()).getGameData().saveImage(path);
				exitMenu();
			}
		}

		public void onRightClick() {
		}

		public void onMove() {
		}
	}

	public class ExitHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "menu1.exit";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			closeDialog();
		}

		public void onRightClick() {
		}

		public void onMove() {
		}
	}

	public class ScenarioHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "menu1.scenario";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			WOPDialogMode dialogMode = (WOPDialogMode) ((WOPMain) getMain())
					.getGameData().getCurrentMode();
			dialogMode.setFinishHandler(null);
			WOPScenarioDialogAction action = (WOPScenarioDialogAction) dialogMode
					.getAction(WOPScenarioDialogAction.DIALOG_NAME);
			action.setReturnDialog(getDialogName());
			action.setReturnDialog2(getReturnDialog());
			action.setMapInfo(((WOPMain) getMain()).getGameData().getMapInfo());
			dialogMode.dispDialog(action);
		}

		public void onMove() {
		}

		public void onRightClick() {
		}

	}

	public class TerrainHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "menu1.terrain";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			WOPDialogMode dialogMode = (WOPDialogMode) ((WOPMain) getMain())
					.getGameData().getCurrentMode();
			dialogMode.setFinishHandler(null);
			WOPTerrainListDialogAction action = (WOPTerrainListDialogAction) dialogMode
					.getAction(WOPTerrainListDialogAction.DIALOG_NAME);
			action.setReturnDialog(getDialogName());
			action.setReturnDialog2(getReturnDialog());
			dialogMode.dispDialog(action);
		}

		public void onMove() {
		}

		public void onRightClick() {
		}

	}
	
	public class EventHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "menu1.event";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			WOPDialogMode dialogMode = (WOPDialogMode) ((WOPMain) getMain())
					.getGameData().getCurrentMode();
			dialogMode.setFinishHandler(null);
			WOPEventListDialogAction action = (WOPEventListDialogAction) dialogMode
					.getAction(WOPEventListDialogAction.DIALOG_NAME);
			action.setReturnDialog(getDialogName());
			action.setReturnDialog2(getReturnDialog());
			dialogMode.dispDialog(action);
		}

		public void onMove() {
		}

		public void onRightClick() {
		}

	}
}
