/**
 * Wars of Powers 
 * 
 * copyright 2009 Mitsunobu SHIOJIMA
 */
package sjima.wop.action;

import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import sjima.cgf.CGFDialogAction;
import sjima.cgf.CGFUIHandler;
import sjima.cgf.CGFUIImage;
import sjima.wop.WOPMain;

public class WOPMessageDialogAction extends CGFDialogAction {

	public static final String DIALOG_NAME = "message";
	private WOPMain main;
	private int timeout = 0;
	private boolean timerEnable = false;

	public WOPMessageDialogAction(WOPMain main) {
		setMain(main);
	}

	public void commit() {
	}

	public Map createHandlerMap() {
		Map map = new HashMap();
		CGFUIHandler handler = new CloseHandler();
		map.put(handler.getName(), handler);

		return map;
	}

	public String getDialogName() {
		return DIALOG_NAME;
	}

	public void setup() {
		for (int i = 0; i < 3; i++) {
			String msg = "";
			if (getMain().getGameData().getMessageList().size() == 1) {
				if (i == 1) {
					msg = (String) getMain().getGameData().getMessageList()
							.get(0);
				}
			} else if (i < getMain().getGameData().getMessageList().size()) {
				msg = (String) getMain().getGameData().getMessageList().get(i);
			}
			String partName = "message.line" + (i + 1);
			CGFUIImage part = (CGFUIImage) this.getDialog().getPartsMap().get(
					partName);
			if (part != null) {
				part.setLabel(msg);
			}
		}
		CGFUIImage part = (CGFUIImage) this.getDialog().getPartsMap().get(
				CloseHandler.HANDLER_NAME);
		part.setEnable(isCloseEnable());
		if (getTimeout() > 0) {
			(new Timer()).schedule(new TimeoutHandler(), getTimeout());
			setTimerEnable(true);
		}
	}

	public WOPMain getMain() {
		return main;
	}

	public void setMain(WOPMain main) {
		this.main = main;
	}

	public void setTimeout(int timeout) {
		this.timeout = timeout;
	}

	public int getTimeout() {
		return timeout;
	}

	public void setTimerEnable(boolean timerEnable) {
		this.timerEnable = timerEnable;
	}

	public boolean isTimerEnable() {
		return timerEnable;
	}
	
	public synchronized void closeDialog() {
		setTimerEnable(false);
		getMain().getGameData().closeDialog();
		
	}

	public class CloseHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "message.close";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			closeDialog();
		}

		public void onRightClick() {
		}

		public void onMove() {
		}
	}

	public class TimeoutHandler extends TimerTask {

		public void run() {
			if (isTimerEnable()) {
				getMain().getGameData().closeDialog();
				setTimerEnable(false);
			}
		}

	}

	public CGFUIHandler createDialogHandler() {
		return new DialogHandler();
	}

	public class DialogHandler implements CGFUIHandler {
		public String getName() {
			return getDialogName();
		}

		public void onClick() {
			if (isCloseEnable()) {
				closeDialog();
			}
		}

		public void onMove() {
			// 何もしない
		}

		public void onRightClick() {
			if (isCloseEnable()) {
				closeDialog();
			}
		}

	}
}
