package sjima.wop.action;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import sjima.cgf.CGFDialogAction;
import sjima.cgf.CGFUIHandler;
import sjima.cgf.CGFUIImage;
import sjima.cgf.CGFUtil;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;
import sjima.wop.WOPPlayer.Parameter;
import sjima.wop.mode.WOPDialogMode;

public class WOPPersonDialogAction extends CGFDialogAction {
	private static final int LINE_LENGTH = 80;
	public static final String DIALOG_NAME = "person";
	public static final int PAGE_TOP = 1;
	public static final int PAGE_DETAIL = 2;
	private int page;
	private String pno;
	private String returnDialog;

	public WOPPersonDialogAction(WOPMain main) {
		setMain(main);
	}

	@Override
	public void commit() {
	}

	@Override
	public Map createHandlerMap() {
		Map map = new HashMap();
		CGFUIHandler handler = new CloseHandler();
		map.put(handler.getName(), handler);
		handler = new NextHandler();
		map.put(handler.getName(), handler);

		return map;
	}

	@Override
	public String getDialogName() {
		return DIALOG_NAME;
	}

	@Override
	public void setup() {
		setPage(PAGE_TOP);
		updateDialog();
	}

	private void updateDialog() {
		WOPPlayer player = (WOPPlayer) getMain().getGameData().getPlayer(
				getPno());
		List lineList = new ArrayList();
		CGFUIImage part = (CGFUIImage) getDialog().getPartsMap().get(
				NextHandler.HANDLER_NAME);
		if (getPage() == PAGE_TOP) {
			part.setLabel("説明");
			lineList.add(String.format("[勝利得点修正] %1.1f", player.getVprate()));
			lineList.add("[地形修正]");
			for (Iterator ite = player.getParaMap().values().iterator(); ite
					.hasNext();) {
				WOPPlayer.Parameter para = (Parameter) ite.next();
				if (WOPPlayer.Parameter.TYPE_TERAIN.equals(para.getType())) {
					lineList.addAll(CGFUtil.divideStringByWidth(
							para.toString(), LINE_LENGTH));
				}
			}
			lineList.add("");
			lineList.add("[イベント修正]");
			for (Iterator ite = player.getParaMap().values().iterator(); ite
					.hasNext();) {
				WOPPlayer.Parameter para = (Parameter) ite.next();
				if (WOPPlayer.Parameter.TYPE_EVENT.equals(para.getType())) {
					lineList.addAll(CGFUtil.divideStringByWidth(
							para.toString(), LINE_LENGTH));
				}
			}
		} else {
			part.setLabel("特性");
			for (Iterator ite = player.getCommentList().iterator(); ite
					.hasNext();) {
				String comment = (String) ite.next();
				lineList.addAll(CGFUtil.divideStringByWidth(comment, LINE_LENGTH));
			}
		}
		part = (CGFUIImage) getDialog().getPartsMap().get(
				"person.name");
		part.setLabel(player.getName());
		for (int i = 0; i < 13; i++) {
			part = (CGFUIImage) getDialog().getPartsMap()
					.get("person.line" + i);
			if (i >= lineList.size()) {
				part.setLabel("");
			} else {
				part.setLabel((String) lineList.get(i));
			}
		}
	}

	public void setPage(int page) {
		this.page = page;
	}

	public int getPage() {
		return page;
	}

	public void setPno(String pno) {
		this.pno = pno;
	}

	public String getPno() {
		return pno;
	}

	public void setReturnDialog(String returnDialog) {
		this.returnDialog = returnDialog;
	}

	public String getReturnDialog() {
		return returnDialog;
	}

	public class CloseHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "person.close";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			closeDialog();
			if (getReturnDialog() != null) {
				((WOPMain) getMain()).getGameData().displayDialog(
						getReturnDialog());
			}
		}

		public void onRightClick() {
		}

		public void onMove() {
		}
	}

	public class NextHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "person.next";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			if (getPage() == PAGE_TOP) {
				setPage(PAGE_DETAIL);
			} else {
				setPage(PAGE_TOP);
			}
			updateDialog();
			WOPDialogMode mode = (WOPDialogMode) getMain().getGameData()
					.getCurrentMode();
			mode.repaintDialog();
		}

		public void onRightClick() {
		}

		public void onMove() {
		}
	}
}
