/**
 * Wars of Powers 
 * 
 * copyright 2009 Mitsunobu SHIOJIMA
 */
package sjima.wop.action;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import sjima.cgf.CGFDialogAction;
import sjima.cgf.CGFMode;
import sjima.cgf.CGFTurnManager;
import sjima.cgf.CGFUIHandler;
import sjima.cgf.CGFUtil;
import sjima.cgf.CGFXMLUtil;
import sjima.wop.WOPGameData;
import sjima.wop.WOPMain;
import sjima.wop.WOPMapDocument;
import sjima.wop.phase.WOPTurnEndPhase;

public class WOPSetup0DialogAction extends CGFDialogAction {

	public static final String DIALOG_NAME = "setup0";

	public WOPSetup0DialogAction(WOPMain main) {
		super();
		setMain(main);
	}

	public WOPMain getMain() {
		return (WOPMain) super.getMain();
	}

	public void commit() {
	}

	public Map createHandlerMap() {
		Map map = new HashMap();
		CGFUIHandler handler = new NewGameHandler();
		map.put(handler.getName(), handler);
		handler = new LoadHandler();
		map.put(handler.getName(), handler);
		handler = new AppendMapHandler();
		map.put(handler.getName(), handler);
		return map;
	}

	public void setup() {
	}

	public class NewGameHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "setup0.newgame";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			commit();
			((WOPMain) getMain()).getSetup().finishPhase();
		}

		public void onRightClick() {
		}

		public void onMove() {
			// 何もしない
		}

	}

	public class LoadHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "setup0.loadgame";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			JFileChooser chooser = getMain().getSaveFileChooser();
			chooser.setCurrentDirectory(WOPGameData.SAVE_DIR);
			int returnVal = chooser.showOpenDialog(null);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				CGFMode mode = getMain().getGameData().getCurrentMode();
				if (mode != null) {
					mode.setMouseEnable(false);
				}
				File file = chooser.getSelectedFile();
				String path = file.getAbsolutePath();
				if (!chooser.accept(file)) {
					path += "." + WOPMapDocument.SAVE_FILE_EXT;
				}
				file = new File(path);
				if (!file.canRead()) {
					String msg = "ファイルが読み込めません";
					JOptionPane.showMessageDialog(null, msg);
					return;
				}
				List msgList = new ArrayList();
				msgList.add("地図を読み込んでいます。");
				msgList.add("しばらくお待ちください。");
				((WOPMain) getMain()).getGameData().displayMessageList(msgList,
						null, 0, false);
				try {
					((WOPMain) getMain()).setupGameData(path);
				} catch (Exception ex) {
					ex.printStackTrace();
					String msg = "ファイルが読み込めません";
					JOptionPane.showMessageDialog(null, msg);
					System.exit(1);
					return;
				}
				getMain().getGameData().repaintBoard();
				CGFTurnManager turnManager = getMain().getGameData()
						.getTurnManager();
				msgList = new ArrayList();
				if (turnManager.getPhaseManager().getCurrentPhase()
						.getPhaseName().equals(WOPTurnEndPhase.PHASE_NAME)) {
					turnManager.getPhaseManager().getCurrentPhase().finish();
				}
				getMain().getGameData().getPhaseManager().getCurrentPhase()
						.restart();
				mode = getMain().getGameData().getCurrentMode();
				if (mode != null) {
					mode.setMouseEnable(true);
				}
			}
		}

		public void onRightClick() {
		}

		public void onMove() {
		}
	}

	public class AppendMapHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "setup0.appendmap";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			JFileChooser chooser = getMain().getMapFileChooser();
			chooser.setCurrentDirectory(WOPGameData.USER_DIR);
			int returnVal = chooser.showOpenDialog(null);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = chooser.getSelectedFile();
				String path = file.getAbsolutePath();
				if (!chooser.accept(file)) {
					path += "." + WOPMapDocument.MAP_FILE_EXT;
				}
				file = new File(path);
				if (!file.canRead()) {
					String msg = "ファイルが読み込めません";
					JOptionPane.showMessageDialog(null, msg);
					return;
				}
				try {
					CGFXMLUtil.loadXML(path);
				} catch (Exception ex) {
					String msg = "ファイルの形式が不正です";
					JOptionPane.showMessageDialog(null, msg);
					return;
				}
				File resourceDir = new File("resource");
				File newFile = new File(resourceDir, file.getName());
				if (newFile.canRead()) {
					int rtn = JOptionPane.showConfirmDialog(null,
							"すでに同じ名前のマップファイルがあります。\n置き換えてよろしいですか？", "確認",
							JOptionPane.YES_NO_OPTION);
					if (rtn != JOptionPane.YES_OPTION) {
						return;
					}
				}
				try {
					CGFUtil.fileCopy(file, newFile);
					String msg = "ファイルを取り込みました";
					JOptionPane.showMessageDialog(null, msg);
				} catch (Exception ex) {
					String msg = "ファイルの取り込みに失敗しました";
					JOptionPane.showMessageDialog(null, msg);
					return;
				}
			}
		}

		public void onRightClick() {
		}

		public void onMove() {
		}
	}

	public String getDialogName() {
		return DIALOG_NAME;
	}

}
