/**
 * Wars of Powers 
 * 
 * copyright 2009 Mitsunobu SHIOJIMA
 */
package sjima.wop.action;

import java.awt.Color;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JOptionPane;

import sjima.cgf.CGFDialogAction;
import sjima.cgf.CGFUIHandler;
import sjima.cgf.CGFUIImage;
import sjima.wop.WOPGameData;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;
import sjima.wop.mode.WOPDialogMode;

public class WOPSetup1DialogAction extends CGFDialogAction {
	public static final String DIALOG_NAME = "setup1";
	public static final Color DISABLE_COLOR = new Color(255, 255, 255, 0);
	private static final String CHECK_ON_IMAGE_PATH = "resource/checkon.png";
	private static final String CHECK_OFF_IMAGE_PATH = "resource/checkoff.png";

	public WOPSetup1DialogAction(WOPMain main) {
		setMain(main);
	}

	public String getDialogName() {
		return DIALOG_NAME;
	}

	public void setup() {
		updateParts();
	}

	public void updateParts() {
		List playerList = getMain().getGameData().getPlayerList();
		for (int i = playerList.size(); i < getMain().getGameData().getNinzu(); i++) {
			WOPPlayer player = new WOPPlayer(getMain(), String.valueOf(i + 1));
			playerList.add(player);
		}
		for (int i = 0; i < WOPGameData.MAX_PLAYER; i++) {
			if (i < getMain().getGameData().getNinzu()) {
				WOPPlayer player = (WOPPlayer) playerList.get(i);
				CGFUIImage part = (CGFUIImage) getDialog().getPartsMap().get(
						"setup1.no" + (i + 1));
				part.setLabel(player.getNo());
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"setup1.name" + (i + 1));
				part.setLabel(player.getName());
				part.setBgColor(player.getColor());
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"setup1.ai" + (i + 1));
				if ("1".equals(player.getAi())) {
					part.setImagePath(CHECK_ON_IMAGE_PATH);
				} else {
					part.setImagePath(CHECK_OFF_IMAGE_PATH);
				}
				part.setEnable(true);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"setup1.difficulty" + (i + 1));
				part.setLabel(player.getDifficultyText());
				part.setEnable(true);
			} else {
				CGFUIImage part = (CGFUIImage) getDialog().getPartsMap().get(
						"setup1.no" + (i + 1));
				part.setLabel("");
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"setup1.name" + (i + 1));
				part.setLabel("");
				part.setBgColor(DISABLE_COLOR);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"setup1.ai" + (i + 1));
				part.setEnable(false);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"setup1.difficulty" + (i + 1));
				part.setLabel("");
				part.setEnable(false);
			}
		}
	}

	public void commit() {
		List playerList = getMain().getGameData().getPlayerList();
		for (int i = 0; i < getMain().getGameData().getNinzu(); i++) {
			WOPPlayer player = (WOPPlayer) playerList.get(i);
			CGFUIImage part = (CGFUIImage) getDialog().getPartsMap().get(
					"setup1.name" + (i + 1));
			player.setName(part.getLabel());
		}
	}

	public Map createHandlerMap() {
		Map map = new HashMap();
		CGFUIHandler handler = new NextHandler();
		map.put(handler.getName(), handler);
		for (int i = 0; i < getMain().getGameData().getMaxPlayer(); i++) {
			handler = new NameHandler(i);
			map.put(handler.getName(), handler);
			handler = new AIHandler(i);
			map.put(handler.getName(), handler);
			handler = new DfcHandler(i);
			map.put(handler.getName(), handler);
			handler = new DetailHandler(i);
			map.put(handler.getName(), handler);
		}
		return map;
	}

	public class NameHandler implements CGFUIHandler {
		private String handlerName;
		private int ix;

		public NameHandler(int ix) {
			setIx(ix);
			setName("setup1.name" + (ix + 1));
		}

		public String getName() {
			return handlerName;
		}

		public void setName(String name) {
			handlerName = name;
		}

		public void onClick() {
			if (getMain().getGameData().getNinzu() <= getIx()) {
				return;
			}
			CGFUIImage label = (CGFUIImage) getDialog().getPartsMap().get(
					getName());
			String input = JOptionPane.showInputDialog("名前を入力してください", label
					.getLabel());
			if (input != null) {
				label.setLabel(input);
				getDialogMode().repaintDialog();
			}
		}

		public void onRightClick() {
		}

		public void onMove() {
			// 何もしない
		}

		public int getIx() {
			return ix;
		}

		public void setIx(int ix) {
			this.ix = ix;
		}

	}

	public class DfcHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "setup1.difficulty";

		private String handlerName;
		private int ix;

		public DfcHandler(int ix) {
			setIx(ix);
			setName(HANDLER_NAME + (ix + 1));
		}

		public String getName() {
			return handlerName;
		}

		public void setName(String name) {
			handlerName = name;
		}

		public void onClick() {
			if (getMain().getGameData().getNinzu() <= getIx()) {
				return;
			}
			WOPPlayer player = (WOPPlayer) ((WOPMain) getMain()).getGameData()
					.getPlayerList().get(getIx());
			player.setDifficulty((player.getDifficulty() + 1) % 4);
			CGFUIImage part = (CGFUIImage) getDialog().getPartsMap().get(
					getName());
			part.setLabel(player.getDifficultyText());
			getDialogMode().repaintDialog();
		}

		public void onRightClick() {
		}

		public void onMove() {
			// 何もしない
		}

		public int getIx() {
			return ix;
		}

		public void setIx(int ix) {
			this.ix = ix;
		}

	}

	public WOPMain getMain() {
		return (WOPMain) super.getMain();
	}

	public class AIHandler implements CGFUIHandler {
		private String handlerName;
		private int ix;

		public AIHandler(int ix) {
			setIx(ix);
			setName("setup1.ai" + (ix + 1));
		}

		public String getName() {
			return handlerName;
		}

		public void setName(String name) {
			handlerName = name;
		}

		public void onClick() {
			if (getMain().getGameData().getNinzu() <= getIx()) {
				return;
			}
			CGFUIImage label = (CGFUIImage) getDialog().getPartsMap().get(
					getName());
			WOPPlayer player = (WOPPlayer) getMain().getGameData()
					.getPlayerList().get(getIx());
			if ("1".equals(player.getAi())) {
				player.setAi("0");
				label.setImagePath(CHECK_OFF_IMAGE_PATH);
				player.setDifficulty(WOPPlayer.DEFAULT_HUMAN_DFC);
				CGFUIImage dfcLabel = (CGFUIImage) getDialog().getPartsMap()
						.get(
								DfcHandler.HANDLER_NAME
										+ String.valueOf(getIx() + 1));
				dfcLabel.setLabel(player.getDifficultyText());
			} else {
				player.setAi("1");
				label.setImagePath(CHECK_ON_IMAGE_PATH);
				player.setDifficulty(WOPPlayer.DEFAULT_AI_DFC);
				CGFUIImage dfcLabel = (CGFUIImage) getDialog().getPartsMap()
						.get(
								DfcHandler.HANDLER_NAME
										+ String.valueOf(getIx() + 1));
				dfcLabel.setLabel(player.getDifficultyText());
			}
			getDialogMode().repaintDialog();
		}

		public void onRightClick() {
		}

		public void onMove() {
			// 何もしない
		}

		public int getIx() {
			return ix;
		}

		public void setIx(int ix) {
			this.ix = ix;
		}

	}

	public class DetailHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "setup1.detail";

		private String handlerName;
		private int ix;

		public DetailHandler(int ix) {
			setIx(ix);
			setName(HANDLER_NAME + (ix + 1));
		}

		public String getName() {
			return handlerName;
		}

		public void setName(String name) {
			handlerName = name;
		}

		public void onClick() {
			if (getMain().getGameData().getNinzu() <= getIx()) {
				return;
			}
			WOPPlayer player = (WOPPlayer) ((WOPMain) getMain()).getGameData()
					.getPlayerList().get(getIx());
			((WOPMain) getMain()).getGameData().changeCurrentMode(
					WOPDialogMode.MODE_NAME);
			WOPDialogMode dialogMode = (WOPDialogMode) ((WOPMain) getMain())
					.getGameData().getCurrentMode();
			dialogMode.setFinishHandler(null);
			WOPPersonDialogAction action = (WOPPersonDialogAction) dialogMode
					.getAction(WOPPersonDialogAction.DIALOG_NAME);
			action.setReturnDialog(getDialogName());
			action.setPno(player.getNo());
			dialogMode.dispDialog(action);
		}

		public void onRightClick() {
		}

		public void onMove() {
			// 何もしない
		}

		public int getIx() {
			return ix;
		}

		public void setIx(int ix) {
			this.ix = ix;
		}

	}

	public class NextHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "setup1.next";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			((WOPMain) getMain()).getSetup().getCurrentPhase().finish();
		}

		public void onRightClick() {
		}

		public void onMove() {
			// 何もしない
		}

	}

}
