/**
 * Wars of Powers 
 * 
 * copyright 2009 Mitsunobu SHIOJIMA
 */
package sjima.wop.action;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import sjima.cgf.CGFDialogAction;
import sjima.cgf.CGFUIHandler;
import sjima.cgf.CGFUIImage;
import sjima.wop.WOPGameData;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;
import sjima.wop.mode.WOPDialogMode;
import sjima.wop.mode.WOPDisplayMode;

public class WOPTargetListDialogAction extends CGFDialogAction {
	public static final String DIALOG_NAME = "targetList";
	public static final Color DISABLE_COLOR = new Color(255, 255, 255, 0);
	private static final String CHECK_ON_IMAGE_PATH = "resource/checkon.png";
	private static final String CHECK_OFF_IMAGE_PATH = "resource/checkoff.png";

	private String phasePlayerNo = null;
	private boolean targetSelected = false;
	private int timeout = 0;
	private boolean timerEnable = false;

	public WOPTargetListDialogAction(WOPMain main) {
		setMain(main);
	}

	public String getDialogName() {
		return DIALOG_NAME;
	}

	public void setup() {
		updateParts();
		if (getTimeout() > 0) {
			(new Timer()).schedule(new TimeoutHandler(), getTimeout());
			setTimerEnable(true);
		}
	}

	public void updateParts() {
		List playerList = getMain().getGameData().getPlayerList();
		if (getPhasePlayerNo() == null) {
			playerList = new ArrayList(playerList);
			Collections.sort(playerList, new WOPPlayer.VPComparator());
		}
		for (int i = playerList.size(); i < getMain().getGameData().getNinzu(); i++) {
			WOPPlayer player = new WOPPlayer(getMain(), String.valueOf(i + 1));
			playerList.add(player);
		}
		for (int i = 0; i < WOPGameData.MAX_PLAYER; i++) {
			if (i < getMain().getGameData().getNinzu()) {
				WOPPlayer player = (WOPPlayer) playerList.get(i);
				CGFUIImage part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.no" + (i + 1));
				part.setLabel(player.getNo());
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.name" + (i + 1));
				part.setLabel(player.getName());
				part.setBgColor(player.getColor());
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.ai" + (i + 1));
				if ("1".equals(player.getAi())) {
					part.setImagePath(CHECK_ON_IMAGE_PATH);
				} else {
					part.setImagePath(CHECK_OFF_IMAGE_PATH);
				}
				part.setEnable(true);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.target" + (i + 1));
				if (player.getTotalStreangth() == 0) {
					part.setBgColor(DISABLE_COLOR);
					part.setLabel("滅亡");
				} else if (player.getTargetLoc() != null) {
					part.setLabel(player.getTargetLoc().toString());
					if (player.getEnemy() != null) {
						WOPPlayer enemyPlayer = getMain().getGameData()
								.getPlayer(player.getEnemy());
						part.setBgColor(enemyPlayer.getColor());
					} else {
						part.setBgColor(DISABLE_COLOR);
					}
				} else {
					part.setBgColor(DISABLE_COLOR);
					part.setLabel("-");
				}
				part.setEnable(true);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.product" + (i + 1));
				part.setLabel(String.format("%5d", player.getTotalFertility()));
				part.setBgColor(player.getColor());
				part.setEnable(true);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.force" + (i + 1));
				part.setLabel(String.format("%5d", player.getTotalStreangth()));
				part.setBgColor(player.getColor());
				part.setEnable(true);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.point" + (i + 1));
				part.setLabel(String.format("%7d", player.getVp()));
				part.setBgColor(player.getColor());
				part.setEnable(true);
			} else {
				CGFUIImage part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.no" + (i + 1));
				part.setLabel("");
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.name" + (i + 1));
				part.setLabel("");
				part.setBgColor(DISABLE_COLOR);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.ai" + (i + 1));
				part.setEnable(false);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.target" + (i + 1));
				part.setLabel("");
				part.setEnable(false);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.product" + (i + 1));
				part.setLabel("");
				part.setEnable(false);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.force" + (i + 1));
				part.setLabel("");
				part.setEnable(false);
				part = (CGFUIImage) getDialog().getPartsMap().get(
						"targetList.point" + (i + 1));
				part.setLabel("");
				part.setEnable(false);
			}
		}
		setCloseEnable(false);
		if (getPhasePlayerNo() == null) {
			CGFUIImage part = (CGFUIImage) getDialog().getPartsMap().get(
					"targetList.select");
			part.setEnable(false);
			part = (CGFUIImage) getDialog().getPartsMap().get(
					"targetList.finish");
			part.setLabel("閉じる");
			part.setEnable(true);
			setTimeout(getMain().getGameData().getMessageWaitTime());
			setCloseEnable(true);
		} else {
			WOPPlayer player = getMain().getGameData().getPlayer(
					getPhasePlayerNo());
			CGFUIImage part = (CGFUIImage) getDialog().getPartsMap().get(
					"targetList.select");
			if ("1".equals(player.getAi())) {
				part.setEnable(false);
			} else {
				part.setEnable(true);
			}
			part = (CGFUIImage) getDialog().getPartsMap().get(
					"targetList.finish");
			CGFUIImage menuPart = (CGFUIImage) getDialog().getPartsMap().get(
					"targetList.menu");
			if (isTargetSelected()) {
				if ("1".equals(player.getAi())) {
					part.setEnable(false);
					menuPart.setEnable(false);
				} else {
					part.setLabel("確定");
					part.setEnable(true);
					setCloseEnable(true);
					menuPart.setEnable(true);
				}
			} else {
				part.setEnable(false);
			}
		}
	}

	public void commit() {
	}

	public Map createHandlerMap() {
		Map map = new HashMap();
		CGFUIHandler handler = new FinishHandler();
		map.put(handler.getName(), handler);
		handler = new SelectHandler();
		map.put(handler.getName(), handler);
		handler = new MenuHandler();
		map.put(handler.getName(), handler);
		for (int i = 0; i < WOPGameData.MAX_PLAYER; i++) {
			handler = new NameHandler(i);
			map.put(handler.getName(), handler);
		}
		return map;
	}

	public WOPMain getMain() {
		return (WOPMain) super.getMain();
	}

	public void setPhasePlayerNo(String phasePlayerNo) {
		this.phasePlayerNo = phasePlayerNo;
	}

	public String getPhasePlayerNo() {
		return phasePlayerNo;
	}

	public void setTargetSelected(boolean targetSelected) {
		this.targetSelected = targetSelected;
	}

	public boolean isTargetSelected() {
		return targetSelected;
	}

	public synchronized void closeDialog() {
		setTimerEnable(false);
		getMain().getGameData().closeDialog();
		((WOPMain) getMain()).getGameData().getPhaseManager().getCurrentPhase()
				.finish();
	}

	public void setTimeout(int timeout) {
		this.timeout = timeout;
	}

	public int getTimeout() {
		return timeout;
	}

	public void setTimerEnable(boolean timerEnable) {
		this.timerEnable = timerEnable;
	}

	public boolean isTimerEnable() {
		return timerEnable;
	}

	public class FinishHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "targetList.finish";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			closeDialog();
		}

		public void onRightClick() {
		}

		public void onMove() {
			// 何もしない
		}

	}

	public class SelectHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "targetList.select";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			getMain().getGameData().changeCurrentMode(WOPDisplayMode.MODE_NAME);
			getMain().getGameData().getCurrentMode().setMouseEnable(true);
		}

		public void onRightClick() {
		}

		public void onMove() {
			// 何もしない
		}

	}

	public class MenuHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "targetList.menu";

		public String getName() {
			return HANDLER_NAME;
		}

		public void onClick() {
			WOPDialogMode mode = (WOPDialogMode) getMain().getGameData()
					.getModeList().get(WOPDialogMode.MODE_NAME);
			WOPMenu1Action action = (WOPMenu1Action) mode
					.getAction(WOPMenu1Action.MENU_NAME);
			action.setReturnDialog(getDialogName());
			mode.closeDialog();
			mode.dispDialog(action);
		}

		public void onRightClick() {
		}

		public void onMove() {
			// 何もしない
		}
	}

	public class TimeoutHandler extends TimerTask {

		public void run() {
			if (isTimerEnable()) {
				closeDialog();
			}
		}

	}

	public class NameHandler implements CGFUIHandler {
		public static final String HANDLER_NAME = "targetList.name";

		private String handlerName;
		private int ix;

		public NameHandler(int ix) {
			setIx(ix);
			setName(HANDLER_NAME + (ix + 1));
		}

		public String getName() {
			return handlerName;
		}

		public void setName(String name) {
			handlerName = name;
		}

		public void onClick() {
			if (getMain().getGameData().getNinzu() <= getIx()) {
				return;
			}
			WOPPlayer player = (WOPPlayer) ((WOPMain) getMain()).getGameData()
					.getPlayerList().get(getIx());
			((WOPMain) getMain()).getGameData().changeCurrentMode(
					WOPDialogMode.MODE_NAME);
			WOPDialogMode dialogMode = (WOPDialogMode) ((WOPMain) getMain())
					.getGameData().getCurrentMode();
			dialogMode.setFinishHandler(null);
			WOPPersonDialogAction action = (WOPPersonDialogAction) dialogMode
					.getAction(WOPPersonDialogAction.DIALOG_NAME);
			action.setReturnDialog(getDialogName());
			action.setPno(player.getNo());
			dialogMode.dispDialog(action);
		}

		public void onRightClick() {
		}

		public void onMove() {
			// 何もしない
		}

		public int getIx() {
			return ix;
		}

		public void setIx(int ix) {
			this.ix = ix;
		}

	}
}
