/**
 * Wars of Powers 
 * 
 * copyright 2009 Mitsunobu SHIOJIMA
 */
package sjima.wop.event;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.w3c.dom.Node;

import sjima.cgf.CGFDiceRoller;
import sjima.cgf.CGFLocation;
import sjima.cgf.CGFUtil;
import sjima.cgf.CGFXMLUtil;
import sjima.wop.WOPCell;
import sjima.wop.WOPForceManager;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;
import sjima.wop.layer.WOPForceLayer;
import sjima.wop.layer.WOPTerrainLayer;

public class WOPEventCivilWar extends WOPEvent {

	public static final String EVENT_TYPE = "CW";

	private int minCond;
	private CGFDiceRoller force;

	public WOPEventCivilWar(WOPMain main, String eno, String name, int rate,
			int times, int minCond, CGFDiceRoller force, Color color,
			String desc) {
		super(main, eno, name, rate, times, color, desc);
		setMinCond(minCond);
		setForce(force);
	}

	public WOPEventCivilWar(WOPMain main, Node node) {
		super(main, node);
		setMinCond(CGFXMLUtil.getAttributeIntegerByName(node, "mincond"));
		setForce(new CGFDiceRoller(CGFXMLUtil.getAttributeByName(node, "force")));
	}

	public String getEType() {
		return EVENT_TYPE;
	}

	public void setMinCond(int minCond) {
		this.minCond = minCond;
	}

	public int getMinCond() {
		return minCond;
	}

	public void setForce(CGFDiceRoller force) {
		this.force = force;
	}

	public CGFDiceRoller getForce() {
		return force;
	}

	public void execute() {
		// まずは発生箇所を決める。
		List cellList = getMain().getGameData().getAllCellList();
		WOPCell cell = (WOPCell) CGFUtil.choiceRandomFromCollection(cellList);
		if (cell == null) {
			return;
		}

		// 発生場所の支配者がいなければ発生せず。
		String cno = getMain().getGameData().getForceManager().getControll(
				cell.getLocation());
		if ((cno == null) || (WOPForceManager.IN_BATTLE.equals(cno))) {
			return;
		}

		// 支配者の戦力がmincondに達していなければ発生せず。
		WOPPlayer player = getMain().getGameData().getPlayer(cno);
		if (player.getTotalStreangth() < getMinCond()) {
			return;
		}

		// 内戦の後継者を決める。
		// 　既に滅んだ国があれば、その中から決める。
		WOPPlayer sPlayer = null;
		List list = getMain().getGameData().getExtinctionList();
		if (list.size() > 0) {
			int minD = 999;
			for (Iterator ite = list.iterator(); ite.hasNext();) {
				WOPPlayer wPlayer = (WOPPlayer) ite.next();
				int d = wPlayer.getHome().distance(cell.getLocation());
				if (minD > d) {
					minD = d;
					sPlayer = wPlayer;
				}
			}
		}

		// 　滅んだ国がなければ、近隣の国から決める（本拠地が一番近い国）。
		if (sPlayer == null) {
			int minD = 999;
			list = getMain().getGameData().getPlayerList();
			list = CGFUtil.shuffleList(list, list.size());
			for (Iterator ite = list.iterator(); ite.hasNext();) {
				WOPPlayer wPlayer = (WOPPlayer) ite.next();
				if (cno.equals(wPlayer.getNo())) {
					continue;
				}
				int d = cell.getLocation().distance(wPlayer.getHome());
				if (d < minD) {
					sPlayer = wPlayer;
					minD = d;
				}
			}
		}
		if (sPlayer == null) {
			return;
		}

		// 反乱戦力を決定する
		int rebelForce = getForce().rollDice();
		int chicedForce = 0;

		// 戦力を越えるまで隣接の場所に反乱が飛び火する。
		Set ajustSet = new HashSet();
		Set targetSet = new HashSet();
		CGFLocation targetLoc = cell.getLocation();
		while (chicedForce < rebelForce) {
			targetSet.add(targetLoc);
			int strength = getMain().getGameData().getForceManager()
					.getStrength(targetLoc, cno);
			chicedForce += strength;
			List ajustList = getMain().getGameData().createAjustList(targetLoc);
			for (Iterator ite = ajustList.iterator(); ite.hasNext();) {
				CGFLocation loc = (CGFLocation) ite.next();
				if (ajustSet.contains(loc)) {
					continue;
				}
				if (targetSet.contains(loc)) {
					continue;
				}
				// 入れないエリアには内戦は及ばない
				if (getMain().getGameData().getCell(loc).getTerrain()
						.getCapacity(sPlayer) == 0) {
					continue;
				}
				String wno = getMain().getGameData().getForceManager()
						.getControll(loc);
				if (cno.equals(wno)) {
					ajustSet.add(loc);
				}
			}
			if (ajustSet.size() == 0) {
				break;
			}
			targetLoc = (CGFLocation) CGFUtil
					.choiceRandomFromCollection(ajustSet);
			ajustSet.remove(targetLoc);
		}

		this.setArea(new ArrayList(targetSet));
		getMain().getGameData().dispEventArea(cell.getLocation(), targetSet,
				getColor());
		List msgList = new ArrayList();
		msgList.add("イベント発生：" + getName());
		getMain().getGameData().displayMessageList(msgList, null,
				getMain().getGameData().getMessageWaitTime(), false);
		try {
			Thread.sleep(getMain().getGameData().getMessageWaitTime());
		} catch (InterruptedException e) {
		}

		// 反乱軍は元の国にとって戦闘の被害として戦闘結果に計上する。
		WOPForceManager forceManager = getMain().getGameData()
				.getForceManager();
		List repaintList = new ArrayList();
		for (Iterator ite = targetSet.iterator(); ite.hasNext();) {
			CGFLocation loc = (CGFLocation) ite.next();
			int strength = forceManager.getStrength(loc, cno);
			forceManager.setStrength(loc, sPlayer.getNo(), strength);
			forceManager.setStrength(loc, cno, 0);
			((WOPForceLayer) getMain().getGameData().getLayer(
					WOPForceLayer.LAYER_NAME)).repaintAtLocation(loc);
			repaintList.add(((WOPTerrainLayer) getMain().getGameData()
					.getLayer(WOPTerrainLayer.LAYER_NAME))
					.repaintAtLocation(loc));
			WOPCell tCell = getMain().getGameData().getCell(loc);
			getMain().getGameData().storeRevoltResult(
					getMain().getGameData().getTurnManager().getTurn(), tCell,
					cno, sPlayer.getNo(), strength);
		}
		getMain().getGameData().repaintRectangleList(repaintList);
	}
}
