package sjima.wop.event;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;

import org.w3c.dom.Node;

import sjima.cgf.CGFDiceRoller;
import sjima.cgf.CGFLocation;
import sjima.cgf.CGFUtil;
import sjima.cgf.CGFXMLUtil;
import sjima.wop.WOPCell;
import sjima.wop.WOPForceManager;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;
import sjima.wop.layer.WOPForceLayer;
import sjima.wop.layer.WOPTerrainLayer;

public class WOPEventEpidemic extends WOPEvent {
	public static final String EVENT_TYPE = "EP";

	private CGFDiceRoller range;
	private int drate;

	public WOPEventEpidemic(WOPMain main, String eno, String name, int rate,
			int times, CGFDiceRoller range, int drate, Color color, String desc) {
		super(main, eno, name, rate, times, color, desc);
		setRange(range);
		setDrate(drate);
	}

	public WOPEventEpidemic(WOPMain main, Node node) {
		super(main, node);
		setRange(new CGFDiceRoller(CGFXMLUtil.getAttributeByName(node, "range")));
		setDrate(CGFXMLUtil.getAttributeIntegerByName(node, "drate"));
	}

	@Override
	public void execute() {
		// まずは発生箇所を決める。
		List cellList = getMain().getGameData().getAllCellList();
		cellList = CGFUtil.shuffleList(cellList, cellList.size());
		WOPCell cell = null;
		for (Iterator ite = cellList.iterator(); ite.hasNext();) {
			cell = (WOPCell) ite.next();
			String cno = getMain().getGameData().getForceManager().getControll(
					cell.getLocation());
			if (cno != WOPForceManager.NONE_CONTROLL) {
				break;
			}
		}

		// 発生エリア数を決める。
		int areaCount = getRange().rollDice();

		// エリア数に達するまで隣接の場所に飛び火する。
		Set ajustSet = new HashSet();
		Set targetSet = new HashSet();
		CGFLocation targetLoc = cell.getLocation();
		while (targetSet.size() < areaCount) {
			targetSet.add(targetLoc);
			List ajustList = getMain().getGameData().createAjustList(targetLoc);
			for (Iterator ite = ajustList.iterator(); ite.hasNext();) {
				CGFLocation loc = (CGFLocation) ite.next();
				if (ajustSet.contains(loc)) {
					continue;
				}
				if (targetSet.contains(loc)) {
					continue;
				}
				// 兵力が存在する場所にだけ飛び火する
				String ctrl = getMain().getGameData().getForceManager()
						.getControll(loc);
				if (ctrl == WOPForceManager.NONE_CONTROLL) {
					continue;
				}

				ajustSet.add(loc);
			}
			if (ajustSet.size() == 0) {
				break;
			}
			targetLoc = (CGFLocation) CGFUtil
					.choiceRandomFromCollection(ajustSet);
			ajustSet.remove(targetLoc);
		}

		this.setArea(new ArrayList(targetSet));
		getMain().getGameData().dispEventArea(cell.getLocation(), targetSet,
				getColor());
		List msgList = new ArrayList();
		msgList.add("イベント発生：" + getName());
		getMain().getGameData().displayMessageList(msgList, null,
				getMain().getGameData().getMessageWaitTime(), false);
		try {
			Thread.sleep(getMain().getGameData().getMessageWaitTime());
		} catch (InterruptedException e) {
		}

		// 被害を判定して適用する
		List repaintList = new ArrayList();
		for (Iterator ite = getArea().iterator(); ite.hasNext();) {
			CGFLocation loc = (CGFLocation) ite.next();
			repaintList.add(((WOPTerrainLayer) getMain().getGameData()
					.getLayer(WOPTerrainLayer.LAYER_NAME))
					.repaintAtLocation(loc));
			String cno = getMain().getGameData().getForceManager().getControll(
					loc);
			if (cno == null) {
				continue;
			}
			LinkedHashMap map = (LinkedHashMap) getMain().getGameData()
					.getForceManager().getForceMap().get(loc);
			for (Iterator ite2 = map.keySet().iterator(); ite2.hasNext();) {
				cno = (String) ite2.next();
				WOPPlayer player = getMain().getGameData().getPlayer(cno);
				int strength = getMain().getGameData().getForceManager()
						.getStrength(loc, cno);
				if (strength == 0) {
					continue;
				}
				int targetUnits = strength;
				int damage = 0;
				int drate = getDrate()
						+ player.getEventModify(getEno(), "drate");
				for (int i = 0; i < targetUnits; i++) {
					int d = CGFUtil.rollDice(1000);
					if (d <= drate) {
						damage++;
					}
				}
				if (damage == 0) {
					continue;
				}
				strength -= damage;
				getMain().getGameData().getForceManager().setStrength(loc, cno,
						strength);
			}
			repaintList.add(((WOPForceLayer) getMain().getGameData().getLayer(
					WOPForceLayer.LAYER_NAME)).repaintAtLocation(loc));
		}
		getMain().getGameData().repaintRectangleList(repaintList);
	}

	@Override
	public String getEType() {
		return EVENT_TYPE;
	}

	public void setRange(CGFDiceRoller range) {
		this.range = range;
	}

	public CGFDiceRoller getRange() {
		return range;
	}

	public void setDrate(int drate) {
		this.drate = drate;
	}

	public int getDrate() {
		return drate;
	}

}
