package sjima.wop.event;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import sjima.cgf.CGFUtil;
import sjima.cgf.CGFXMLUtil;
import sjima.wop.WOPMain;

public class WOPEventManager {
	private WOPMain main;
	private LinkedHashMap eventMap = new LinkedHashMap();
	
	public WOPEventManager(WOPMain main, Document doc) {
		setMain(main);
		init(doc);
	}

	public void init(Node node) {
		List list = CGFXMLUtil.readListByXPath(node,
				"/gamedata/eventlist/event");
		for (Iterator ite = list.iterator(); ite.hasNext();) {
			Node eventNode = (Node) ite.next();
			String etype = CGFXMLUtil.getAttributeByName(eventNode, "etype");
			WOPEvent event = createEvent(etype, eventNode);
			if (event != null) {
				getEventMap().put(event.getEno(), event);
			}
		}
	}
	
	private WOPEvent createEvent(String etype, Node eventNode) {
		if (WOPEventCivilWar.EVENT_TYPE.equals(etype)) {
			return new WOPEventCivilWar(getMain(), eventNode);
		} else if (WOPEventRevolt.EVENT_TYPE.equals(etype)) {
			return new WOPEventRevolt(getMain(), eventNode);
		} else if (WOPEventStorm.EVENT_TYPE.equals(etype)) {
			return new WOPEventStorm(getMain(), eventNode);
		} else if (WOPEventFamine.EVENT_TYPE.equals(etype)) {
			return new WOPEventFamine(getMain(), eventNode);
		} else if (WOPEventEpidemic.EVENT_TYPE.equals(etype)) {
			return new WOPEventEpidemic(getMain(), eventNode);
		} else if (WOPEventReinforcement.EVENT_TYPE.equals(etype)) {
			return new WOPEventReinforcement(getMain(), eventNode);
		}
		return null;
	}

	public Map getEventMap() {
		return eventMap;
	}

	public void setMain(WOPMain main) {
		this.main = main;
	}

	public WOPMain getMain() {
		return main;
	}
	
	public List createHappendList() {
		List result = new ArrayList();
		
		for (Iterator ite = getEventMap().values().iterator(); ite.hasNext(); ) {
			WOPEvent event = (WOPEvent)ite.next();
			int rate = event.getRate();
			int times = event.getTimes();
			for (int i = 0; i < times; i++) {
				int d = CGFUtil.rollDice(1000);
				if (d <= rate) {
					result.add(event);
				}
			}
		}
		
		return result;
	}
	
}
