/**
 * Wars of Powers 
 * 
 * copyright 2009 Mitsunobu SHIOJIMA
 */
package sjima.wop.event;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.w3c.dom.Node;

import sjima.cgf.CGFDiceRoller;
import sjima.cgf.CGFLocation;
import sjima.cgf.CGFUtil;
import sjima.cgf.CGFXMLUtil;
import sjima.wop.WOPCell;
import sjima.wop.WOPForceManager;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;
import sjima.wop.layer.WOPForceLayer;
import sjima.wop.layer.WOPTerrainLayer;

public class WOPEventReinforcement extends WOPEvent {

	public static final String EVENT_TYPE = "RF";

	private CGFDiceRoller force;

	public WOPEventReinforcement(WOPMain main, String eno, String name, int rate,
			int times, CGFDiceRoller force, Color color, String desc) {
		super(main, eno, name, rate, times, color, desc);
		setForce(force);
	}

	public WOPEventReinforcement(WOPMain main, Node node) {
		super(main, node);
		setForce(new CGFDiceRoller(CGFXMLUtil.getAttributeByName(node, "force")));
	}

	public String getEType() {
		return EVENT_TYPE;
	}

	public void setForce(CGFDiceRoller force) {
		this.force = force;
	}

	public CGFDiceRoller getForce() {
		return force;
	}

	public void execute() {
		// まずは発生箇所を決める。
		List cellList = getMain().getGameData().getAllCellList();
		WOPCell cell = (WOPCell) CGFUtil.choiceRandomFromCollection(cellList);
		if (cell == null) {
			return;
		}

		// 発生場所の支配者がいなければ発生せず。
		String cno = getMain().getGameData().getForceManager().getControll(
				cell.getLocation());
		if ((cno == null) || (WOPForceManager.IN_BATTLE.equals(cno))) {
			return;
		}
		WOPPlayer player = getMain().getGameData().getPlayer(cno);

		// 増援戦力を決定する
		int reinforce = getForce().rollDice();
		int chicedForce = 0;

		// 戦力を越えるまで隣接の場所に増援が飛び火する。
		Set ajustSet = new HashSet();
		Set targetSet = new HashSet();
		CGFLocation targetLoc = cell.getLocation();
		while (chicedForce < reinforce) {
			targetSet.add(targetLoc);
			int strength = getMain().getGameData().getForceManager()
					.getStrength(targetLoc, cno);
			WOPCell wCell = getMain().getGameData().getCell(targetLoc);
			int force = wCell.getTerrain().getCapacity(player) - strength;
			chicedForce += force;
			List ajustList = getMain().getGameData().createAjustList(targetLoc);
			for (Iterator ite = ajustList.iterator(); ite.hasNext();) {
				CGFLocation loc = (CGFLocation) ite.next();
				if (ajustSet.contains(loc)) {
					continue;
				}
				if (targetSet.contains(loc)) {
					continue;
				}
				String wno = getMain().getGameData().getForceManager()
						.getControll(loc);
				if (cno.equals(wno)) {
					ajustSet.add(loc);
				}
			}
			if (ajustSet.size() == 0) {
				break;
			}
			targetLoc = (CGFLocation) CGFUtil
					.choiceRandomFromCollection(ajustSet);
			ajustSet.remove(targetLoc);
		}

		this.setArea(new ArrayList(targetSet));
		getMain().getGameData().dispEventArea(cell.getLocation(), targetSet,
				getColor());
		List msgList = new ArrayList();
		msgList.add("イベント発生：" + getName());
		getMain().getGameData().displayMessageList(msgList, null,
				getMain().getGameData().getMessageWaitTime(), false);
		try {
			Thread.sleep(getMain().getGameData().getMessageWaitTime());
		} catch (InterruptedException e) {
		}

		// 増援戦力を追加する。
		WOPForceManager forceManager = getMain().getGameData()
				.getForceManager();
		List repaintList = new ArrayList();
		for (Iterator ite = targetSet.iterator(); ite.hasNext();) {
			CGFLocation loc = (CGFLocation) ite.next();
			WOPCell wCell = getMain().getGameData().getCell(loc);
			int strength = wCell.getTerrain().getCapacity(player);
			forceManager.setStrength(loc, cno, strength);
			((WOPForceLayer) getMain().getGameData().getLayer(
					WOPForceLayer.LAYER_NAME)).repaintAtLocation(loc);
			repaintList.add(((WOPTerrainLayer) getMain().getGameData()
					.getLayer(WOPTerrainLayer.LAYER_NAME))
					.repaintAtLocation(loc));
		}
		getMain().getGameData().repaintRectangleList(repaintList);
	}
}
