package sjima.wop.event;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.w3c.dom.Node;

import sjima.cgf.CGFDiceRoller;
import sjima.cgf.CGFLocation;
import sjima.cgf.CGFUtil;
import sjima.cgf.CGFXMLUtil;
import sjima.wop.WOPCell;
import sjima.wop.WOPForceManager;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;
import sjima.wop.layer.WOPForceLayer;
import sjima.wop.layer.WOPTerrainLayer;
import sjima.wop.phase.WOPCombatPhase;
import sjima.wop.phase.WOPPhaseManager;

public class WOPEventRevolt extends WOPEvent {

	public static final String EVENT_TYPE = "RV";

	private CGFDiceRoller force;
	private String cno;
	private boolean enemyOnly = false;

	public WOPEventRevolt(WOPMain main, String eno, String name, int rate,
			int times, CGFDiceRoller force, Color color, String desc,
			String cno, boolean enemyOnly) {
		super(main, eno, name, rate, times, color, desc);
		setForce(force);
		setCno(cno);
		setEnemyOnly(enemyOnly);
	}

	public WOPEventRevolt(WOPMain main, Node node) {
		super(main, node);

		setForce(new CGFDiceRoller(CGFXMLUtil.getAttributeByName(node, "force")));
		setCno(CGFXMLUtil.getAttributeByName(node, "cno"));
		String str = CGFXMLUtil.getAttributeByName(node, "enemyonly");
		setEnemyOnly("1".equals(str));
	}

	public String getEType() {
		return EVENT_TYPE;
	}

	public void setForce(CGFDiceRoller force) {
		this.force = force;
	}

	public CGFDiceRoller getForce() {
		return force;
	}

	public void execute() {
		// まずは発生箇所を決める。
		List cellList = getMain().getGameData().getAllCellList();
		WOPCell cell = null;

		String eno = null;
		if (isEnemyOnly()) {
			if (getCno() != null) {
				eno = getMain().getGameData().getPlayer(getCno()).getEnemy();
			} else {
				return;
			}
		}

		while (cellList.size() > 0) {
			cell = (WOPCell) CGFUtil.choiceRandomFromCollection(cellList);
			if (cell == null) {
				return;
			}
			cellList.remove(cell);

			// 発生場所の支配者がいなければ別の場所
			String cno = getMain().getGameData().getForceManager().getControll(
					cell.getLocation());
			if ((cno == null) || (WOPForceManager.IN_BATTLE.equals(cno))) {
				continue;
			}

			// 発生場所が敵のみの場合には、敵の支配エリアだけが対象。
			if (eno != null) {
				if (!eno.equals(cno)) {
					continue;
				}
			}
			break;
		}

		// 反乱の後継者を決める。
		WOPPlayer sPlayer = null;

		if (getCno() == null) {
			// 　既に滅んだ国があれば、その中から決める。
			List list = getMain().getGameData().getExtinctionList();
			if (list.size() > 0) {
				int minD = 999;
				for (Iterator ite = list.iterator(); ite.hasNext();) {
					WOPPlayer wPlayer = (WOPPlayer) ite.next();
					int d = wPlayer.getHome().distance(cell.getLocation());
					if (minD > d) {
						minD = d;
						sPlayer = wPlayer;
					}
				}
			}
		} else {
			sPlayer = getMain().getGameData().getPlayer(getCno());
		}

		// 　滅んだ国がなければ、反乱は発生しない。
		if (sPlayer == null) {
			return;
		}

		// 反乱戦力を決定する
		int rebelForce = getForce().rollDice();
		int chicedForce = 0;

		// 戦力を越えるまで隣接の場所に反乱が飛び火する。
		Set ajustSet = new HashSet();
		Set targetSet = new HashSet();
		CGFLocation targetLoc = cell.getLocation();
		while (chicedForce < rebelForce) {
			targetSet.add(targetLoc);
			WOPCell wCell = getMain().getGameData().getCell(targetLoc);
			int strength = wCell.getTerrain().getCapacity(sPlayer);
			chicedForce += strength;
			List ajustList = getMain().getGameData().createAjustList(targetLoc);
			for (Iterator ite = ajustList.iterator(); ite.hasNext();) {
				CGFLocation loc = (CGFLocation) ite.next();
				if (ajustSet.contains(loc)) {
					continue;
				}
				if (targetSet.contains(loc)) {
					continue;
				}
				// 入れないエリアには内戦は及ばない
				if (getMain().getGameData().getCell(loc).getTerrain()
						.getCapacity(sPlayer) == 0) {
					continue;
				}
				String wno = getMain().getGameData().getForceManager()
						.getControll(loc);
				if ((wno != null) && (!WOPForceManager.IN_BATTLE.equals(wno))) {
					ajustSet.add(loc);
				}
			}
			if (ajustSet.size() == 0) {
				break;
			}
			targetLoc = (CGFLocation) CGFUtil
					.choiceRandomFromCollection(ajustSet);
			ajustSet.remove(targetLoc);
		}

		this.setArea(new ArrayList(targetSet));
		getMain().getGameData().dispEventArea(cell.getLocation(), targetSet,
				getColor());
		List msgList = new ArrayList();
		msgList.add("イベント発生：" + getName());
		getMain().getGameData().displayMessageList(msgList, null,
				getMain().getGameData().getMessageWaitTime(), false);
		try {
			Thread.sleep(getMain().getGameData().getMessageWaitTime());
		} catch (InterruptedException e) {
		}

		WOPForceManager forceManager = getMain().getGameData()
				.getForceManager();
		List repaintList = new ArrayList();
		for (Iterator ite = targetSet.iterator(); ite.hasNext();) {
			CGFLocation loc = (CGFLocation) ite.next();
			WOPCell wCell = getMain().getGameData().getCell(loc);
			int strength = wCell.getTerrain().getCapacity(sPlayer);
			forceManager.setStrength(loc, sPlayer.getNo(), strength);
			((WOPForceLayer) getMain().getGameData().getLayer(
					WOPForceLayer.LAYER_NAME)).repaintAtLocation(loc);
			repaintList.add(((WOPTerrainLayer) getMain().getGameData()
					.getLayer(WOPTerrainLayer.LAYER_NAME))
					.repaintAtLocation(loc));
		}
		getMain().getGameData().repaintRectangleList(repaintList);
		try {
			Thread.sleep(getMain().getGameData().getMessageWaitTime());
		} catch (InterruptedException e) {
		}
		WOPCombatPhase combatPhase = new WOPCombatPhase(getMain(),
				(WOPPhaseManager) getMain().getGameData().getPhaseManager());
		combatPhase.mainProc();
	}

	public void setCno(String cno) {
		this.cno = cno;
	}

	public String getCno() {
		return cno;
	}

	public void setEnemyOnly(boolean enemyOnly) {
		this.enemyOnly = enemyOnly;
	}

	public boolean isEnemyOnly() {
		return enemyOnly;
	}
}
