package sjima.wop.layer;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import sjima.cgf.CGFFullSizeLayer;
import sjima.cgf.CGFLocation;
import sjima.cgf.CGFMain;
import sjima.cgf.CGFSize;
import sjima.wop.WOPForceManager;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;

public class WOPForceLayer extends CGFFullSizeLayer {
	public static final String LAYER_NAME = "force";

	private final Color LIGHT_COLOR1 = new Color(223, 223, 223, 255);
	private final Color LIGHT_COLOR2 = new Color(255, 255, 255, 255);
	private final Color SHADOW_COLOR = new Color(31, 31, 31, 255);
	private final Color CLEAR_COLOR = new Color(255, 255, 255, 0);
	private final Color BATTLE_COLOR = new Color(255, 255, 255, 255);
	private final Color BATTLE_FONT_COLOR = new Color(255, 31, 0, 255);
	private final Color DUPLICAT_TARGET_COLOR = new Color(0, 0, 0, 255);

	private Font strengthFont;

	public WOPForceLayer(CGFMain main, CGFSize size) {
		super(main, size);
		Font font = getMain().getFontManager().getMonoFont();
		setStrengthFont(new Font(font.getName(), font.getStyle(), 15));
		setTransparency(0.9f);
	}

	public void updateAtLocation(CGFLocation loc) {
		CGFSize size = getMain().getGameData().getSquareSize();
		Point point = getMain().getGameData().getLocationPoint(loc);
		Rectangle rect = new Rectangle(point.x, point.y, size.width,
				size.height);

		CGFSize forceImgSize = new CGFSize(20, 20);
		Graphics2D gc = getLayerImage().createGraphics();
		gc.setBackground(CLEAR_COLOR);
		gc.clearRect(rect.x, rect.y, size.width, size.height);
		gc.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		List targetPlayerList = getMain().getGameData().getPlayerNoAtTargetLoc(
				loc);
		if (targetPlayerList.size() == 1) {
			WOPPlayer targetPlayer = getMain().getGameData().getPlayer(
					(String) targetPlayerList.get(0));
			gc.setColor(targetPlayer.getColor());
			gc.fillRect(rect.x, rect.y, rect.width, rect.height);
			gc.clearRect(rect.x + 3, rect.y + 3, rect.width - 6, rect.height - 6);
		} else if (targetPlayerList.size() > 1) {
			gc.setColor(DUPLICAT_TARGET_COLOR);
			gc.fillRect(rect.x, rect.y, rect.width, rect.height);
			gc.clearRect(rect.x + 3, rect.y + 3, rect.width - 6, rect.height - 6);
		}

		String ctrl = getForceManager().getControll(loc);
		if (ctrl == null) {
			return;
		}
		int x = rect.x + 6;
		int y = rect.y + 21;
		if (WOPForceManager.IN_BATTLE.equals(ctrl)) {
			// 戦闘表示
			String str = "戦";
			x += 2;
			gc.setColor(BATTLE_COLOR);
			gc.setFont(getStrengthFont());
			gc.fillRect(rect.x + 6, rect.y + 6, forceImgSize.width,
					forceImgSize.height);
			gc.setColor(SHADOW_COLOR);
			gc.drawString(str, x + 1, y);
			gc.drawString(str, x + 1, y + 1);
			gc.setColor(BATTLE_FONT_COLOR);
			gc.drawString(str, x, y);
		} else {
			// 戦力表示
			int strength = getForceManager().getStrength(loc, ctrl);
			String str = String.valueOf(strength);
			WOPPlayer player = getMain().getGameData().getPlayer(ctrl);
			gc.setColor(player.getColor());
			gc.setFont(getStrengthFont());
			gc.fillRect(rect.x + 6, rect.y + 6, forceImgSize.width,
					forceImgSize.height);
			gc.setColor(SHADOW_COLOR);
			if (str.length() == 1) {
				x += 4;
			}
			gc.drawString(str, x + 1, y);
			gc.drawString(str, x + 1, y + 1);
			if (str.length() == 1) {
				gc.setColor(LIGHT_COLOR1);
			} else {
				gc.setColor(LIGHT_COLOR2);
			}
			gc.drawString(str, x, y);
		}
	}

	public void update() {
		repaintAll();
	}

	public Rectangle repaintAtLocation(CGFLocation loc) {
		updateAtLocation(loc);
		CGFSize size = getMain().getGameData().getSquareSize();
		Point point = getMain().getGameData().getLocationPoint(loc);
		Rectangle rect = new Rectangle(point.x, point.y, size.width,
				size.height);
		return rect;
	}

	public List repaintAll() {
		clearAll();
		List result = new ArrayList();
		if (getForceManager() == null) {
			return result;
		}
		Map forceMap = getForceManager().getForceMap();
		for (Iterator ite = forceMap.keySet().iterator(); ite.hasNext();) {
			CGFLocation loc = (CGFLocation) ite.next();
			Rectangle rect = repaintAtLocation(loc);
			result.add(rect);
		}
		return result;
	}

	public String getLayerName() {
		return LAYER_NAME;
	}

	public WOPMain getMain() {
		return (WOPMain) super.getMain();
	}

	public WOPForceManager getForceManager() {
		return getMain().getGameData().getForceManager();
	}

	public void setStrengthFont(Font strengthFont) {
		this.strengthFont = strengthFont;
	}

	public Font getStrengthFont() {
		return strengthFont;
	}
	
	
}
