package sjima.wop.mode;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import sjima.cgf.CGFDialogAction;
import sjima.cgf.CGFDialogMode;
import sjima.cgf.CGFLocation;
import sjima.cgf.CGFMain;
import sjima.cgf.CGFMenuSet;
import sjima.cgf.layer.CGFBorderTerrainLayer;
import sjima.cgf.layer.CGFCursolLayer;
import sjima.cgf.layer.CGFDialogLayer;
import sjima.cgf.layer.CGFInfoBoardLayer;
import sjima.wop.WOPMain;
import sjima.wop.action.WOPEventListDialogAction;
import sjima.wop.action.WOPMapSelectDialogAction;
import sjima.wop.action.WOPMenu1Action;
import sjima.wop.action.WOPMessageDialogAction;
import sjima.wop.action.WOPPersonDialogAction;
import sjima.wop.action.WOPPrizeDialogAction;
import sjima.wop.action.WOPScenarioDialogAction;
import sjima.wop.action.WOPSetup0DialogAction;
import sjima.wop.action.WOPSetup1DialogAction;
import sjima.wop.action.WOPTargetListDialogAction;
import sjima.wop.action.WOPTerrainListDialogAction;
import sjima.wop.layer.WOPForceLayer;
import sjima.wop.layer.WOPTerrainLayer;

public class WOPDialogMode extends CGFDialogMode {
	private final static Class[] ENABLE_LAYERS = { 
			WOPForceLayer.class, WOPTerrainLayer.class,
			CGFBorderTerrainLayer.class, CGFCursolLayer.class,
			CGFInfoBoardLayer.class, CGFDialogLayer.class };
	private Set enableLayerSet = new HashSet(Arrays.asList(ENABLE_LAYERS));

	private Runnable finishHandler = null;

	public WOPDialogMode(CGFMain main) {
		super(main);
	}

	public void buildActionMap() {
		Map map = getActionMap();
		if (map == null) {
			map = new HashMap();
			setActionMap(map);
		}
		CGFDialogAction action = new WOPSetup0DialogAction((WOPMain) getMain());
		map.put(action.getDialogName(), action);
		action = new WOPSetup1DialogAction((WOPMain) getMain());
		map.put(action.getDialogName(), action);
		action = new WOPMapSelectDialogAction((WOPMain) getMain());
		map.put(action.getDialogName(), action);
		action = new WOPMessageDialogAction((WOPMain) getMain());
		map.put(action.getDialogName(), action);
		action = new WOPTargetListDialogAction((WOPMain) getMain());
		map.put(action.getDialogName(), action);
		action = new WOPPrizeDialogAction((WOPMain) getMain());
		map.put(action.getDialogName(), action);
		action = new WOPPersonDialogAction((WOPMain) getMain());
		map.put(action.getDialogName(), action);
		action = new WOPScenarioDialogAction((WOPMain) getMain());
		map.put(action.getDialogName(), action);
		action = new WOPTerrainListDialogAction((WOPMain) getMain());
		map.put(action.getDialogName(), action);
		action = new WOPEventListDialogAction((WOPMain) getMain());
		map.put(action.getDialogName(), action);
		action = new WOPMenu1Action((WOPMain) getMain());
		map.put(action.getDialogName(), action);
		setHandlerMap(createHandlerMap());
	}

	public void setFinishHandler(Runnable handler) {
		this.finishHandler = handler;
	}

	public Runnable getFinishHandler() {
		return this.finishHandler;
	}

	public Set getEnableLayerSet() {
		return enableLayerSet;
	}

	public void setEnableLayerSet(Set enableLayerSet) {
		this.enableLayerSet = enableLayerSet;
	}

	public synchronized void closeDialog() {
		CGFDialogLayer dialogLayer = (CGFDialogLayer) getMain().getGameData()
				.getLayer(CGFDialogLayer.LAYER_NAME);
		dialogLayer.getCurrentDispDialog().setEnable(false);
		List list = dialogLayer.clearDialog();
		CGFMenuSet menu = dialogLayer.getCurrentDispMenu();
		if (menu != null && menu.isEnable()) {
			list.addAll(dialogLayer.drawMenu());
		}
		getMain().getGameData().repaintRectangleList(list);
		if (getFinishHandler() != null) {
			Runnable handler = getFinishHandler();
			setFinishHandler(null);
			Thread thread = new Thread(handler);
			thread.setPriority(Thread.MIN_PRIORITY);
			thread.start();
		}
	}

	protected void moveMouse(CGFLocation newLoc) {
		System.out.println("DialogMode.moveMouse");
				super.moveMouse(newLoc);
			}
}
