package sjima.wop.phase;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;

import sjima.cgf.CGFGamePhase;
import sjima.cgf.CGFMode;
import sjima.cgf.CGFUtil;
import sjima.wop.WOPCell;
import sjima.wop.WOPForceManager;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;
import sjima.wop.layer.WOPForceLayer;
import sjima.wop.mode.WOPDisplayMode;

public class WOPCombatPhase extends CGFGamePhase {
	public static final String PHASE_NAME = "戦闘フェイズ";
	private final int DAMAGE_RATE = 30;

	private WOPMain main;
	private WOPPhaseManager phaseManager;
	private WOPCell retreat;

	public WOPCombatPhase(WOPMain main, WOPPhaseManager phaseManager) {
		super();
		setMain(main);
		setPhaseManager(phaseManager);
	}

	public void run() {
		List msgList = new ArrayList();
		msgList.add(getPhaseName());
		getMain().getGameData().displayMessageList(msgList, null, 0, false);

		mainProc();
		finish();
	}

	public void mainProc() {
		List repaintList = new ArrayList();
		List cellList = getMain().getGameData().getAllCellList();
		cellList = CGFUtil.shuffleList(cellList, cellList.size());
		for (Iterator ite = cellList.iterator(); ite.hasNext();) {
			WOPCell cell = (WOPCell) ite.next();
			String ctrl = getMain().getGameData().getForceManager()
					.getControll(cell.getLocation());
			if (!WOPForceManager.IN_BATTLE.equals(ctrl)) {
				continue;
			}
			repaintList.addAll(resolveCombat(cell));
		}
		getMain().getGameData().repaintRectangleList(repaintList);
	}

	private List resolveCombat(WOPCell cell) {
		List updateList = new ArrayList();

		WOPForceManager forceManager = getMain().getGameData()
				.getForceManager();
		LinkedHashMap map = (LinkedHashMap) forceManager.getForceMap().get(
				cell.getLocation());
		Iterator ite = map.keySet().iterator();
		String defender = (String) ite.next();
		String attacker = (String) ite.next();
		WOPPlayer defenderPlayer = getMain().getGameData().getPlayer(defender);
		WOPPlayer attackerPlayer = getMain().getGameData().getPlayer(attacker);

		int defenderStrength = ((Integer) map.get(defender)).intValue();
		int attackerStrength = ((Integer) map.get(attacker)).intValue();
		int defenderForce = forceManager.calDefenderForce(cell,
				defenderStrength);
		int attackerForce = forceManager.calAttackerForce(cell,
				attackerStrength, attackerPlayer, defenderPlayer);

		int attackerDamage = decideDamage(defenderForce);
		int defenderDamage = decideDamage(attackerForce);
		attackerDamage = (attackerDamage > attackerStrength) ? attackerStrength
				: attackerDamage;
		defenderDamage = (defenderDamage > defenderStrength) ? defenderStrength
				: defenderDamage;
		int attackerRest = attackerStrength - attackerDamage;
		int defenderRest = defenderStrength - defenderDamage;
		int result = 0;
		setRetreat(null);

		if ((attackerRest > 0) && (defenderRest <= 0)) {
			// 防御側全滅：攻撃側の勝利
			result = 1;
			forceManager
					.setStrength(cell.getLocation(), attacker, attackerRest);
			forceManager.setStrength(cell.getLocation(), defender, 0);
		} else if ((attackerRest <= 0) && (defenderRest > 0)) {
			// 攻撃側全滅：防御側の勝利
			result = -1;
			forceManager.setStrength(cell.getLocation(), attacker, 0);
			forceManager
					.setStrength(cell.getLocation(), defender, defenderRest);
		} else if ((attackerRest <= 0) && (defenderRest <= 0)) {
			// 両軍全滅
			result = 0;
			forceManager.setStrength(cell.getLocation(), attacker, 0);
			forceManager.setStrength(cell.getLocation(), defender, 0);
		} else {
			// 防御側損害 > 攻撃側損害 + 防御側士気修正：攻撃側勝利
			if (defenderDamage > attackerDamage
					+ cell.getTerrain().getDefenceMorale(defenderPlayer)
					- cell.getTerrain().getAttackMorale(attackerPlayer)) {
				result = 1;
				forceManager.setStrength(cell.getLocation(), attacker,
						attackerRest);
				forceManager.setStrength(cell.getLocation(), defender, 0);
				updateList.addAll(retreat(defender, defenderRest, cell));
			} else { // 防御側勝利
				result = -1;
				forceManager.setStrength(cell.getLocation(), attacker, 0);
				forceManager.setStrength(cell.getLocation(), defender,
						defenderRest);
				updateList.addAll(retreat(attacker, attackerRest, cell));
			}
		}
		getMain().getGameData().storeCombatResult(
				getMain().getGameData().getTurnManager().getTurn(), cell,
				defender, attacker, defenderStrength, attackerStrength,
				defenderForce, attackerForce, defenderDamage, attackerDamage,
				result, getRetreat());

		updateList.add(((WOPForceLayer) getMain().getGameData().getLayer(
				WOPForceLayer.LAYER_NAME))
				.repaintAtLocation(cell.getLocation()));
		return updateList;
	}

	private int decideDamage(int defenderForce) {
		int restForce = defenderForce;
		int damage = 0;
		while (restForce > 0) {
			int rate;
			if (restForce >= 100) {
				rate = DAMAGE_RATE;
				restForce -= 100;
			} else {
				rate = (DAMAGE_RATE * restForce + 50) / 100;
				restForce = 0;
			}
			int roll = CGFUtil.rollDice(100);
			if (roll <= rate) {
				damage++;
			}
		}
		return damage;
	}

	// 退却処理
	private List retreat(String pno, int force, WOPCell cell) {
		WOPPlayer player = getMain().getGameData().getPlayer(pno);
		List updateList = new ArrayList();
		List ajustList = getMain().getGameData().createAjustCellList(
				cell.getLocation());
		ajustList = CGFUtil.shuffleList(ajustList, ajustList.size());
		int retreatedForce = 0;
		WOPCell retreatedCell = null;
		for (Iterator ite = ajustList.iterator(); ite.hasNext();) {
			WOPCell aCell = (WOPCell) ite.next();
			String ctrl = getMain().getGameData().getForceManager()
					.getControll(aCell.getLocation());
			// 戦闘中の所には後退できない
			if (WOPForceManager.IN_BATTLE.equals(ctrl)) {
				continue;
			}
			// 他人の支配地域には後退できない
			if ((ctrl != null) && !(pno.equals(ctrl))) {
				continue;
			}
			int friendForce = getMain().getGameData().getForceManager()
					.getStrength(aCell.getLocation(), pno);
			int maxMove = aCell.getTerrain().getCapacity(player) - friendForce;
			if (maxMove >= force) {
				retreatedForce = force;
				retreatedCell = aCell;
				break;
			}
			if (maxMove > retreatedForce) {
				retreatedForce = maxMove;
				retreatedCell = aCell;
			}
		}
		if (retreatedCell != null) {
			int friendForce = getMain().getGameData().getForceManager()
					.getStrength(retreatedCell.getLocation(), pno);
			getMain().getGameData().getForceManager().setStrength(
					retreatedCell.getLocation(), pno,
					friendForce + retreatedForce);
			updateList.add(((WOPForceLayer) getMain().getGameData().getLayer(
					WOPForceLayer.LAYER_NAME)).repaintAtLocation(retreatedCell
					.getLocation()));
		}
		setRetreat(retreatedCell);

		return updateList;
	}

	public CGFMode getDefaultMode() {
		return (CGFMode) getMain().getGameData().getModeList().get(
				WOPDisplayMode.MODE_NAME);
	}

	public String getPhaseName() {
		return PHASE_NAME;
	}

	public void setMain(WOPMain main) {
		this.main = main;
	}

	public WOPMain getMain() {
		return main;
	}

	public void setPhaseManager(WOPPhaseManager phaseManager) {
		this.phaseManager = phaseManager;
	}

	public WOPPhaseManager getPhaseManager() {
		return phaseManager;
	}

	public class FinishHandler implements Runnable {

		public void run() {
			finish();
		}

	}

	public WOPCell getRetreat() {
		return retreat;
	}

	public void setRetreat(WOPCell retreat) {
		this.retreat = retreat;
	}

}
