package sjima.wop.phase;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import sjima.cgf.CGFGamePhase;
import sjima.cgf.CGFMode;
import sjima.cgf.CGFUtil;
import sjima.wop.WOPCell;
import sjima.wop.WOPForceManager;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;
import sjima.wop.WOPTerrain;
import sjima.wop.layer.WOPForceLayer;
import sjima.wop.mode.WOPDisplayMode;

public class WOPForceIncrementPhase extends CGFGamePhase {
	public static final String PHASE_NAME = "兵力増加フェイズ";

	private WOPMain main;
	private WOPPhaseManager phaseManager;

	public WOPForceIncrementPhase(WOPMain main, WOPPhaseManager phaseManager) {
		super();
		setMain(main);
		setPhaseManager(phaseManager);
	}

	public void run() {
		mainProc();
		finish();
	}

	private void mainProc() {
		getMain().getGameData().calTotalFertility();
		getMain().getGameData().getForceManager().calTotalStrength();
		List repaintList = new ArrayList();
		List cellList = getMain().getGameData().getAllCellList();
		cellList = CGFUtil.shuffleList(cellList, cellList.size());
		for (Iterator ite = cellList.iterator(); ite.hasNext();) {
			WOPCell cell = (WOPCell) ite.next();
			String ctrl = getMain().getGameData().getForceManager()
					.getControll(cell.getLocation());
			if (ctrl == null) {
				continue;
			}
			if (WOPForceManager.IN_BATTLE.equals(ctrl)) {
				continue;
			}
			int force = getMain().getGameData().getForceManager().getStrength(
					cell.getLocation(), ctrl);
			WOPPlayer player = getMain().getGameData().getPlayer(ctrl);
			WOPTerrain terrain = cell.getTerrain();
			if (force >= terrain.getMaxGrowth(player)) {
				continue;
			}
			if (terrain.getFertility(player) == 0) {
				continue;
			}
			if (player.getTotalStreangth() > player.getMaxStreangth()) {
				continue;
			}
			player.setRecruitment(player.getRecruitment()
					+ terrain.getFertility(player));
			if (player.getRecruitment() >= player.getAddForcePoint()) {
				force += 1;
				getMain().getGameData().getForceManager().setStrength(
						cell.getLocation(), ctrl, force);
				player.setRecruitment(player.getRecruitment()
						- player.getAddForcePoint());
				player.setTotalStreangth(player.getTotalStreangth() + 1);
				WOPForceLayer layer = (WOPForceLayer) getMain().getGameData()
						.getLayer(WOPForceLayer.LAYER_NAME);
				repaintList.add(layer.repaintAtLocation(cell.getLocation()));
			}
		}
		getMain().getGameData().repaintRectangleList(repaintList);
	}

	public CGFMode getDefaultMode() {
		return (CGFMode) getMain().getGameData().getModeList().get(
				WOPDisplayMode.MODE_NAME);
	}

	public String getPhaseName() {
		return PHASE_NAME;
	}

	public void setMain(WOPMain main) {
		this.main = main;
	}

	public WOPMain getMain() {
		return main;
	}

	public void setPhaseManager(WOPPhaseManager phaseManager) {
		this.phaseManager = phaseManager;
	}

	public WOPPhaseManager getPhaseManager() {
		return phaseManager;
	}

	public class FinishHandler implements Runnable {

		public void run() {
			finish();
		}

	}

}
