package sjima.wop.phase;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import sjima.cgf.CGFGamePhase;
import sjima.cgf.CGFLocation;
import sjima.cgf.CGFMode;
import sjima.cgf.CGFUtil;
import sjima.wop.WOPCell;
import sjima.wop.WOPForceManager;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;
import sjima.wop.WOPTerrain;
import sjima.wop.layer.WOPForceLayer;
import sjima.wop.mode.WOPDisplayMode;

public class WOPMovementPhase extends CGFGamePhase {
	public static final String PHASE_NAME = "移動フェイズ";

	private WOPMain main;
	private WOPPhaseManager phaseManager;

	public WOPMovementPhase(WOPMain main, WOPPhaseManager phaseManager) {
		super();
		setMain(main);
		setPhaseManager(phaseManager);
	}

	public void run() {
		List msgList = new ArrayList();
		msgList.add(getPhaseName());
		getMain().getGameData().displayMessageList(msgList, null, 0, false);

		mainProc();
		finish();
	}

	private void mainProc() {
		List repaintList = new ArrayList();
		List cellList = getMain().getGameData().getAllCellList();
		cellList = CGFUtil.shuffleList(cellList, cellList.size());
		for (Iterator ite = cellList.iterator(); ite.hasNext();) {
			WOPCell cell = (WOPCell) ite.next();
			String ctrl = getMain().getGameData().getForceManager()
					.getControll(cell.getLocation());
			if (ctrl == null) {
				continue;
			}
			WOPPlayer player = getMain().getGameData().getPlayer(ctrl);
			// 戦闘中の場所はこれ以上移動できない
			if (WOPForceManager.IN_BATTLE.equals(ctrl)) {
				continue;
			}
			int force = getMain().getGameData().getForceManager().getStrength(
					cell.getLocation(), ctrl);
			WOPTerrain terrain = cell.getTerrain();
			if (force < terrain.getMoveStart(player)) {
				continue;
			}
			int moveForce = force - terrain.getGarrison(player);
			List ajustList = getMain().getGameData().createAjustCellList(
					cell.getLocation());
			if ((ajustList == null) || (ajustList.size() == 0)) {
				continue;
			}
			ajustList = CGFUtil.shuffleList(ajustList, ajustList.size());
			WOPCell moveCell = decideMoveCell(ctrl, moveForce, ajustList, cell);
			if (moveCell == null) {
				continue;
			}
			repaintList.addAll(moveToCell(ctrl, moveForce, cell, moveCell,
					player));
		}
		getMain().getGameData().repaintRectangleList(repaintList);
	}

	private WOPCell decideMoveCell(String ctrl, int moveForce, List ajustList,
			WOPCell currentCell) {
		WOPCell moveCell = null;
		WOPPlayer player = getMain().getGameData().getPlayer(ctrl);
		for (Iterator ite = ajustList.iterator(); ite.hasNext();) {
			WOPCell cell = (WOPCell) ite.next();
			String moveCellCtrl = getMain().getGameData().getForceManager()
					.getControll(cell.getLocation());
			// 入れないエリアはパス
			if (cell.getTerrain().getCapacity(player) == 0) {
				continue;
			}
			
			// すでに他国同士で戦闘中の場所には入れない
			if (WOPForceManager.IN_BATTLE.equals(moveCellCtrl)) {
				int friendlyForce = getMain().getGameData().getForceManager()
						.getStrength(cell.getLocation(), ctrl);
				if (friendlyForce == 0) {
					continue;
				}
			} else if ((moveCellCtrl != null) && (!ctrl.equals(moveCellCtrl))) {
				// 敵が同じ国同士は同盟国なので入れない
				WOPPlayer moveCellPlayer = getMain().getGameData().getPlayer(
						moveCellCtrl);
				if ((player.getEnemy() != null)
						&& (player.getEnemy().equals(moveCellPlayer.getEnemy()))) {
					continue;
				}
			}

			if (moveCell == null) {
				moveCell = cell;
			} else if (compareMoveCell(ctrl, moveForce, moveCell, cell,
					currentCell) < 0) {
				moveCell = cell;
			}
		}

		return moveCell;
	}

	// 移動場所の比較
	private int compareMoveCell(String ctrl, int moveForce, WOPCell cell1,
			WOPCell cell2, WOPCell currentCell) {
		// 支配者
		String ctrl1 = getMain().getGameData().getForceManager().getControll(
				cell1.getLocation());
		String ctrl2 = getMain().getGameData().getForceManager().getControll(
				cell2.getLocation());
		WOPPlayer player = getMain().getGameData().getPlayer(ctrl);
		WOPPlayer player1 = null;
		if (ctrl1 != null) {
			player1 = getMain().getGameData().getPlayer(ctrl1);
		}
		WOPPlayer player2 = null;
		if (ctrl2 != null) {
			player2 = getMain().getGameData().getPlayer(ctrl2);
		}
		// 目標地点
		CGFLocation targetLoc = player.getTargetLoc();

		// 第１優先：戦闘が発生していたらそこが優先
		if (WOPForceManager.IN_BATTLE.equals(ctrl1)) {
			if (WOPForceManager.IN_BATTLE.equals(ctrl2)) {
				return 0; // 互角
			} else {
				return 1; // 1優先
			}
		} else {
			if (WOPForceManager.IN_BATTLE.equals(ctrl2)) {
				return -1; // 2優先
			}
		}

		// 第２優先：空き地で生産力が１以上なら生産力が高いほうが優先
		if ((ctrl1 == null) && (cell1.getTerrain().getFertility(player) > 0)) {
			if ((ctrl2 == null)
					&& (cell2.getTerrain().getFertility(player) > 0)) {
				int result = cell1.getTerrain().getFertility(player)
						- cell2.getTerrain().getFertility(player);
				if (result == 0) {
					// 同じ場合：目標に近いほうを優先
					if (targetLoc != null) {
						result = targetLoc.distance(cell2.getLocation())
								- targetLoc.distance(cell1.getLocation());
					}
				}
				return result;
			} else {
				return 1; // 1優先
			}
		} else {
			if ((ctrl2 == null)
					&& (cell2.getTerrain().getFertility(player) > 0)) {
				return -1; // 2優先
			}
		}

		// 第３優先：敵の土地なら、自分よりも戦力が下で戦力が多いところを優先
		// 第４優先：自分よりも戦力が多くて、戦力が少ないところを優先
		if ((ctrl1 != null)
				&& !ctrl.equals(ctrl1)
				&& ((player.getEnemy() == null) || (!player.getEnemy().equals(
						player1.getEnemy())))) {
			if ((ctrl2 != null)
					&& !ctrl.equals(ctrl2)
					&& ((player.getEnemy() == null) || (!player.getEnemy()
							.equals(player2.getEnemy())))) {
				int attackerForce1 = getMain().getGameData().getForceManager()
						.calAttackerForce(cell1, moveForce, player, player1);
				int attackerForce2 = getMain().getGameData().getForceManager()
						.calAttackerForce(cell2, moveForce, player, player2);
				int otherForce1 = getMain().getGameData().getForceManager()
						.getStrength(cell1.getLocation(), ctrl1);
				int otherForce2 = getMain().getGameData().getForceManager()
						.getStrength(cell2.getLocation(), ctrl2);
				int defenderForce1 = getMain().getGameData().getForceManager()
						.calDefenderForce(cell1, otherForce1);
				int defenderForce2 = getMain().getGameData().getForceManager()
						.calDefenderForce(cell2, otherForce2);
				if (attackerForce1 > defenderForce1) {
					if (attackerForce2 > defenderForce2) {
						int result = otherForce1 - otherForce2;
						if (result == 0) {
							// 同じ場合：目標に近いほうを優先
							if (targetLoc != null) {
								result = targetLoc
										.distance(cell2.getLocation())
										- targetLoc.distance(cell1
												.getLocation());
							}
						}
						return result;
					} else {
						return 1; // 1優先
					}
				} else {
					if (attackerForce2 > defenderForce2) {
						return -1; // 2優先
					}
					return otherForce2 - otherForce1;
				}

			} else {
				return 1; // 1優先
			}
		} else {
			if ((ctrl2 != null) && !ctrl.equals(ctrl2)) {
				return -1; // 2優先
			}
		}

		// 第５優先；遠ざかる方向はパス
		int currentDistance = currentCell.getLocation().distance(targetLoc);
		int distance1 = cell1.getLocation().distance(targetLoc);
		int distance2 = cell2.getLocation().distance(targetLoc);
		if ((currentDistance <= distance2) && (currentDistance > distance1)) {
			return 1; // 1優先
		}
		if ((currentDistance <= distance1) && (currentDistance > distance2)) {
			return -1; // 2優先
		}

		// 第６優先：移動できる数が多いほうを優先
		int cell1force = getMain().getGameData().getForceManager().getStrength(
				cell1.getLocation(), ctrl);
		int cell2force = getMain().getGameData().getForceManager().getStrength(
				cell2.getLocation(), ctrl);
		int moveRest1 = cell1.getTerrain().getCapacity(player) - cell1force;
		int moveRest2 = cell2.getTerrain().getCapacity(player) - cell2force;
		int moveForce1 = (moveForce > moveRest1) ? moveRest1 : moveForce;
		int moveForce2 = (moveForce > moveRest2) ? moveRest2 : moveForce;

		int result = moveForce1 - moveForce2;
		if (result != 0) {
			return result;
		}

		// 第７優先：目標に近いほうを優先
		if (targetLoc != null) {
			result = targetLoc.distance(cell2.getLocation())
					- targetLoc.distance(cell1.getLocation());
			return result;
		}
		return 0;
	}

	// 移動実行
	private List moveToCell(String ctrl, int moveForce, WOPCell startCell,
			WOPCell moveCell, WOPPlayer player) {
		int startStrength = getMain().getGameData().getForceManager()
				.getStrength(startCell.getLocation(), ctrl);
		int moveCellStrength = getMain().getGameData().getForceManager()
				.getStrength(moveCell.getLocation(), ctrl);
		int moveRest = moveCell.getTerrain().getCapacity(player)
				- moveCellStrength;
		int move = (moveForce > moveRest) ? moveRest : moveForce;
		getMain().getGameData().getForceManager().setStrength(
				startCell.getLocation(), ctrl, startStrength - move);
		getMain().getGameData().getForceManager().setStrength(
				moveCell.getLocation(), ctrl, moveCellStrength + move);

		List result = new ArrayList();
		WOPForceLayer layer = (WOPForceLayer) getMain().getGameData().getLayer(
				WOPForceLayer.LAYER_NAME);
		result.add(layer.repaintAtLocation(startCell.getLocation()));
		result.add(layer.repaintAtLocation(moveCell.getLocation()));

		return result;
	}

	public CGFMode getDefaultMode() {
		return (CGFMode) getMain().getGameData().getModeList().get(
				WOPDisplayMode.MODE_NAME);
	}

	public String getPhaseName() {
		return PHASE_NAME;
	}

	public void setMain(WOPMain main) {
		this.main = main;
	}

	public WOPMain getMain() {
		return main;
	}

	public void setPhaseManager(WOPPhaseManager phaseManager) {
		this.phaseManager = phaseManager;
	}

	public WOPPhaseManager getPhaseManager() {
		return phaseManager;
	}

	public class FinishHandler implements Runnable {

		public void run() {
			finish();
		}

	}

}
