package sjima.wop.phase;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import sjima.cgf.CGFGamePhase;
import sjima.cgf.CGFLocation;
import sjima.cgf.CGFMode;
import sjima.wop.WOPAI;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;
import sjima.wop.action.WOPTargetListDialogAction;
import sjima.wop.layer.WOPForceLayer;
import sjima.wop.mode.WOPDialogMode;
import sjima.wop.mode.WOPDisplayMode;

public class WOPTargetChoicePhase extends CGFGamePhase {
	public static final String PHASE_NAME = "目標決定フェイズ";

	private WOPMain main;
	private WOPPhaseManager phaseManager;

	public WOPTargetChoicePhase(WOPMain main, WOPPhaseManager phaseManager) {
		super();
		setMain(main);
		setPhaseManager(phaseManager);
	}

	public void run() {
		mainProc();
	}

	private void mainProc() {
		List repaintList = new ArrayList();
		List targetLocList = new ArrayList();

		int playerNo = getPhaseManager().getTurnManager().getPlayerNo();
		WOPPlayer player = (WOPPlayer) getMain().getGameData().getPlayerList()
				.get(playerNo);
		// 戦力がない国は滅亡しているのでパス
		if (player.getTotalStreangth() == 0) {
			player.setTargetLoc(null);
			finish();
			return;
		}
		if ("1".equals(player.getAi())) {
			WOPAI ai = getMain().getGameData().getAI(player.getNo());
			ai.decideTargetLoc();
		} else if (player.getLastTargetLoc() != null) {
			player.setTargetLoc(player.getLastTargetLoc());
		}
		targetLocList.addAll(getMain().getGameData().getAllTargetLocList());
		for (Iterator ite = targetLocList.iterator(); ite.hasNext();) {
			CGFLocation loc = (CGFLocation) ite.next();
			WOPForceLayer layer = (WOPForceLayer) getMain().getGameData()
					.getLayer(WOPForceLayer.LAYER_NAME);
			repaintList.add(layer.repaintAtLocation(loc));
		}

		getMain().getGameData().repaintRectangleList(repaintList);
		getMain().getGameData().changeCurrentMode(WOPDialogMode.MODE_NAME);
		WOPDialogMode dialogMode = (WOPDialogMode) getMain().getGameData()
				.getCurrentMode();
		WOPTargetListDialogAction action = (WOPTargetListDialogAction) dialogMode
				.getAction(WOPTargetListDialogAction.DIALOG_NAME);
		action.setPhasePlayerNo(player.getNo());
		if (!"1".equals(player.getAi())) {
			action.setTargetSelected((player.getTargetLoc() != null));
			action.setTimeout(0);
			CGFLocation targetLoc = player.getTargetLoc();
			if (targetLoc != null) {
				getMain().getGameData().scrollToLocation(targetLoc);
			}
		} else {
			action.setTimeout(10);
		}
		dialogMode.dispDialog(action);
	}

	public CGFMode getDefaultMode() {
		return (CGFMode) getMain().getGameData().getModeList().get(
				WOPDisplayMode.MODE_NAME);
	}

	public String getPhaseName() {
		return PHASE_NAME;
	}

	public void setMain(WOPMain main) {
		this.main = main;
	}

	public WOPMain getMain() {
		return main;
	}

	public void setPhaseManager(WOPPhaseManager phaseManager) {
		this.phaseManager = phaseManager;
	}

	public WOPPhaseManager getPhaseManager() {
		return phaseManager;
	}

	public class FinishHandler implements Runnable {

		public void run() {
			finish();
		}

	}

	public void click(CGFLocation loc) {
		List repaintList = new ArrayList();

		int playerNo = getPhaseManager().getTurnManager().getPlayerNo();
		WOPPlayer player = (WOPPlayer) getMain().getGameData().getPlayerList()
				.get(playerNo);
		CGFLocation oldLoc = player.getTargetLoc();
		player.setTargetLoc(loc);
		getMain().getGameData().changeCurrentMode(WOPDialogMode.MODE_NAME);
		WOPDialogMode dialogMode = (WOPDialogMode) getMain().getGameData()
				.getCurrentMode();
		WOPForceLayer layer = (WOPForceLayer) getMain().getGameData().getLayer(
				WOPForceLayer.LAYER_NAME);
		if (oldLoc != null) {
			repaintList.add(layer.repaintAtLocation(oldLoc));
		}
		repaintList.add(layer.repaintAtLocation(loc));

		getMain().getGameData().repaintRectangleList(repaintList);

		WOPTargetListDialogAction action = (WOPTargetListDialogAction) dialogMode
				.getAction(WOPTargetListDialogAction.DIALOG_NAME);
		action.setPhasePlayerNo(player.getNo());
		action.setTargetSelected(true);
		action.setTimeout(0);
		dialogMode.dispDialog(action);
	}

}
