package sjima.wop.phase;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import sjima.cgf.CGFGamePhase;
import sjima.cgf.CGFMode;
import sjima.wop.WOPMain;
import sjima.wop.WOPPlayer;
import sjima.wop.action.WOPPrizeDialogAction;
import sjima.wop.mode.WOPDisplayMode;

public class WOPTurnEndPhase extends CGFGamePhase {
	public static final String PHASE_NAME = "プレイヤーターン終了フェイズ";

	private WOPMain main;
	private WOPPhaseManager phaseManager;

	public WOPTurnEndPhase(WOPMain main, WOPPhaseManager phaseManager) {
		super();
		setMain(main);
		setPhaseManager(phaseManager);
	}

	public String getPhaseName() {
		return PHASE_NAME;
	}

	public void run() {
		calVictoryPoint();
		checkGameEnd();
	}

	private void checkGameEnd() {
		boolean isGameEnd = false;
		int messageWaitTime = getMain().getGameData().getMessageWaitTime();
		List msgList = new ArrayList();
		for (Iterator ite = getMain().getGameData().getPlayerList().iterator(); ite
				.hasNext();) {
			WOPPlayer player = (WOPPlayer) ite.next();
			if (!"1".equals(player.getAi())) {
				if (player.getTotalStreangth() == 0) {
					msgList.add(player.getName() + "は滅亡しました");
					player.setAi("1");
					if (getMain().getGameData().isAllAi()) {
						msgList.add("全てのプレイヤーが滅亡したので");
						msgList.add("ゲームを終了します");
						isGameEnd = true;
					}
				}
			}
		}
		int turn = getMain().getGameData().getTurnManager().getTurn();
		if (turn >= getMain().getGameData().getEndTurn()) {
			msgList.add(String.valueOf(turn) + "ターンが経過しました");
			msgList.add("ゲームを終了します");
			isGameEnd = true;
		}
		FinishHandler finishHandler = new FinishHandler(isGameEnd);
		if (msgList.size() > 0) {
			getMain().getGameData().displayMessageList(msgList, finishHandler,
					messageWaitTime, true);
		} else {
			finish();
		}
	}

	private void calVictoryPoint() {
		getMain().getGameData().calTotalFertility();
		getMain().getGameData().getForceManager().calTotalStrength();
		for (Iterator ite = getMain().getGameData().getPlayerList().iterator(); ite
				.hasNext();) {
			WOPPlayer player = (WOPPlayer) ite.next();
			int point = (int) (player.getTotalFertility() * player.getVprate() + 0.5001);
			if (point == 0) {
				player.setVp(0);
			} else {
				player.setVp(player.getVp() + point);
			}
		}
	}

	public void finish() {
		getPhaseManager().getTurnManager().nextTurn();
	}

	public WOPMain getMain() {
		return main;
	}

	public void setMain(WOPMain main) {
		this.main = main;
	}

	public WOPPhaseManager getPhaseManager() {
		return phaseManager;
	}

	public void setPhaseManager(WOPPhaseManager phaseManager) {
		this.phaseManager = phaseManager;
	}

	public CGFMode getDefaultMode() {
		return (CGFMode) getMain().getGameData().getModeList().get(
				WOPDisplayMode.MODE_NAME);
	}

	public class FinishHandler implements Runnable {
		private boolean isGameEnd = false;

		public FinishHandler(boolean isGameEnd) {
			setGameEnd(isGameEnd);
		}

		public void run() {
			if (isGameEnd()) {
				List playerList = new ArrayList(getMain().getGameData()
						.getPlayerList());
				Collections.sort(playerList, new WOPPlayer.VPComparator());
				WOPPlayer player = (WOPPlayer) playerList.get(0);
				String snd = null;
				if ("1".equals(player.getAi())) {
					snd = getMain().getSoundSetting().getEndingSound();
				} else {
					snd = getMain().getSoundSetting().getVictorySound();
				}
				getMain().getSoundManager().playSound(snd);
				getMain().getGameData().displayDialog(
						WOPPrizeDialogAction.DIALOG_NAME);
			} else {
				finish();
			}
		}

		public void setGameEnd(boolean isGameEnd) {
			this.isGameEnd = isGameEnd;
		}

		public boolean isGameEnd() {
			return isGameEnd;
		}

	}
}
