package sjima.wop.phase;

import java.util.ArrayList;
import java.util.List;

import sjima.cgf.CGFGamePhase;
import sjima.cgf.CGFMode;
import sjima.cgf.CGFTurnManager;
import sjima.cgf.CGFUtil;
import sjima.wop.WOPMain;
import sjima.wop.mode.WOPDisplayMode;

public class WOPTurnStartPhase extends CGFGamePhase {
	public static final String PHASE_NAME = "プレイヤーターン開始フェイズ";

	private WOPMain main;
	private WOPPhaseManager phaseManager;

	public WOPTurnStartPhase(WOPMain main, WOPPhaseManager phaseManager) {
		super();
		setMain(main);
		setPhaseManager(phaseManager);
	}
	
	public void run() {
		CGFTurnManager turnManager = getPhaseManager().getTurnManager();
		turnManager.setPlayerNo(0);
		getMain().getGameData().clearAllTarget();
		List playerList = getMain().getGameData().getPlayerList();
		playerList = CGFUtil.shuffleList(playerList, playerList.size());
		getMain().getGameData().setPlayerList(playerList);
		List msgList = new ArrayList();
		msgList.add("ターン" + turnManager.getTurn());
		String curBgm = getMain().getSoundManager().getCurrentSoundFile();
		String stageNo = getMain().getGameData().getTurnManager().getStage();
		String bgm = getMain().getSoundSetting().getStageSound(stageNo);
		if (bgm != null) {
			if (! bgm.equals(curBgm)) {
				getMain().getSoundManager().playSound(bgm);
			}
		}
		getMain().getGameData().displayMessageList(msgList,
				new FinishHandler(),
				getMain().getGameData().getMessageWaitTime(), false);
	}

	public CGFMode getDefaultMode() {
		return (CGFMode) getMain().getGameData().getModeList().get(
				WOPDisplayMode.MODE_NAME);
	}

	public String getPhaseName() {
		return PHASE_NAME;
	}


	public void setMain(WOPMain main) {
		this.main = main;
	}

	public WOPMain getMain() {
		return main;
	}

	public void setPhaseManager(WOPPhaseManager phaseManager) {
		this.phaseManager = phaseManager;
	}

	public WOPPhaseManager getPhaseManager() {
		return phaseManager;
	}

	public class FinishHandler implements Runnable {

		public void run() {
			finish();
		}

	}

}
